@@ -23,7 +23,7 @@
 
 #include "Group.h"
 #include "gnc-component-manager.h"
-#include "gnc-engine-util.h"
+#include "qof.h"
 #include "gnc-ui-util.h"
 
 
@@ -34,7 +34,7 @@
 typedef struct
 {
   GNCIdType entity_type;
-  GNCEngineEventType event_mask;
+  QofEventId event_mask;
 } EntityTypeEventInfo;
 
 typedef struct
@@ -64,7 +64,8 @@ typedef struct
 
 /** Static Variables ************************************************/
 static guint  suspend_counter = 0;
-static gint   next_component_id = 0;
+/* Some code foolishly uses 0 instead of NO_COMPONENT, so we start with 1. */
+static gint   next_component_id = 1;
 static GList *components = NULL;
 
 static ComponentEventInfo changes = { NULL, NULL, FALSE };
@@ -72,7 +73,7 @@ static ComponentEventInfo changes_backup = { NULL, NULL, FALSE };
 
 
 /* This static indicates the debugging module that this .o belongs to.  */
-static short module = MOD_GUI;
+static QofLogModule log_module = GNC_MOD_GUI;
 
 
 /** Prototypes ******************************************************/
@@ -107,12 +108,12 @@ dump_components (void)
 static void
 clear_mask_hash_helper (gpointer key, gpointer value, gpointer user_data)
 {
-  GNCEngineEventType * et = value;
+  QofEventId * et = value;
 
   *et = 0;
 }
 
-/* clear a hash table of the form string --> GNCEngineEventType,
+/* clear a hash table of the form string --> QofEventId,
  * where the values are g_malloced and the keys are in the engine
  * string cache. */
 static void
@@ -127,9 +128,7 @@ clear_mask_hash (GHashTable *hash)
 static gboolean
 destroy_mask_hash_helper (gpointer key, gpointer value, gpointer user_data)
 {
-  GCache *gc = gnc_engine_get_string_cache ();
-
-  g_cache_remove (gc, key);
+  qof_util_string_cache_remove (key);
   g_free (value);
 
   return TRUE;
@@ -184,7 +183,7 @@ clear_event_info (ComponentEventInfo *cei)
 
 static void
 add_event (ComponentEventInfo *cei, const GUID *entity,
-           GNCEngineEventType event_mask, gboolean or_in)
+           QofEventId event_mask, gboolean or_in)
 {
   GHashTable *hash;
 
@@ -235,9 +234,9 @@ add_event (ComponentEventInfo *cei, const GUID *entity,
 
 static void
 add_event_type (ComponentEventInfo *cei, GNCIdTypeConst entity_type,
-                GNCEngineEventType event_mask, gboolean or_in)
+                QofEventId event_mask, gboolean or_in)
 {
-  GNCEngineEventType *mask;
+  QofEventId *mask;
 
   g_return_if_fail (cei);
   g_return_if_fail (cei->event_masks);
@@ -246,9 +245,8 @@ add_event_type (ComponentEventInfo *cei, GNCIdTypeConst entity_type,
   mask = g_hash_table_lookup (cei->event_masks, entity_type);
   if (!mask)
   {
-    char * key = g_cache_insert (gnc_engine_get_string_cache (),
-                                 (gpointer) entity_type);
-    mask = g_new0 (GNCEngineEventType, 1);
+    char * key = qof_util_string_cache_insert ((gpointer) entity_type);
+    mask = g_new0 (QofEventId, 1);
     g_hash_table_insert (cei->event_masks, key, mask);
   }
 
@@ -259,33 +257,27 @@ add_event_type (ComponentEventInfo *cei, GNCIdTypeConst entity_type,
 }
 
 static void
-gnc_cm_event_handler (GUID *entity,
-                      GNCEngineEventType event_type,
-                      gpointer user_data)
+gnc_cm_event_handler (QofEntity *entity,
+                      QofEventId event_type,
+                      gpointer user_data,
+		      gpointer event_data)
 {
-  GNCIdType id_type;
-
+  const GUID *guid = qof_entity_get_guid(entity);
 #if CM_DEBUG
-  fprintf (stderr, "event_handler: event %d, guid %s\n", event_type,
-	   guid_to_string(entity));
+  fprintf (stderr, "event_handler: event %d, entity %p, guid %s\n", event_type,
+	   entity, guid);
 #endif
-  add_event (&changes, entity, event_type, TRUE);
+  add_event (&changes, guid, event_type, TRUE);
 
-  id_type = xaccGUIDType (entity, gnc_get_current_book ());
-  g_return_if_fail (id_type);
-#if CM_DEBUG
-    fprintf (stderr, "               guid type: %s\n", id_type);
-#endif
-
-  if (safe_strcmp (id_type, GNC_ID_SPLIT) == 0)
+  if (QOF_CHECK_TYPE(entity, GNC_ID_SPLIT))
   {
     /* split events are never generated by the engine, but might
      * be generated by a backend (viz. the postgres backend.)
      * Handle them like a transaction modify event. */
-    add_event_type (&changes, GNC_ID_TRANS, GNC_EVENT_MODIFY, TRUE);
+    add_event_type (&changes, GNC_ID_TRANS, QOF_EVENT_MODIFY, TRUE);
   }
   else
-    add_event_type (&changes, id_type, event_type, TRUE);
+    add_event_type (&changes, entity->e_type, event_type, TRUE);
 
   got_events = TRUE;
 
@@ -310,7 +302,7 @@ gnc_component_manager_init (void)
   changes_backup.event_masks = g_hash_table_new (g_str_hash, g_str_equal);
   changes_backup.entity_events = guid_hash_table_new ();
 
-  handler_id = gnc_engine_register_event_handler (gnc_cm_event_handler, NULL);
+  handler_id = qof_event_register_handler (gnc_cm_event_handler, NULL);
 }
 
 void
@@ -334,7 +326,7 @@ gnc_component_manager_shutdown (void)
   destroy_event_hash (changes_backup.entity_events);
   changes_backup.entity_events = NULL;
 
-  gnc_engine_unregister_event_handler (handler_id);
+  qof_event_unregister_handler (handler_id);
 }
 
 static ComponentInfo *
@@ -398,10 +390,17 @@ gnc_register_gui_component_internal (const char * component_class)
   /* look for a free handler id */
   component_id = next_component_id;
 
+  /* design warning: if we ever get 2^32-1 components, 
+     this loop is infinite.  Instead of fixing it, we'll just 
+     complain when (if) we get half way there (probably never).
+  */ 
   while (find_component (component_id))
     if (++component_id == NO_COMPONENT)
       component_id++;
 
+  if (component_id < 0) 
+    PERR("Amazing! Half way to running out of component_ids.");
+  
   /* found one, add the handler */
   ci = g_new0 (ComponentInfo, 1);
 
@@ -472,10 +471,10 @@ gnc_register_gui_component_scm (const char * component_class,
   g_return_val_if_fail (ci, NO_COMPONENT);
 
   ci->refresh_handler_scm = refresh_handler;
-  scm_protect_object (refresh_handler);
+  scm_gc_protect_object (refresh_handler);
 
   ci->close_handler_scm = close_handler;
-  scm_protect_object (close_handler);
+  scm_gc_protect_object (close_handler);
 
   return ci->component_id;
 }
@@ -483,7 +482,7 @@ gnc_register_gui_component_scm (const char * component_class,
 void
 gnc_gui_component_watch_entity (gint component_id,
                                 const GUID *entity,
-                                GNCEngineEventType event_mask)
+                                QofEventId event_mask)
 {
   ComponentInfo *ci;
 
@@ -503,7 +502,7 @@ gnc_gui_component_watch_entity (gint component_id,
 void
 gnc_gui_component_watch_entity_direct (gint component_id,
                                        GUID entity,
-                                       GNCEngineEventType event_mask)
+                                       QofEventId event_mask)
 {
   gnc_gui_component_watch_entity (component_id, &entity, event_mask);
 }
@@ -511,7 +510,7 @@ gnc_gui_component_watch_entity_direct (gint component_id,
 void
 gnc_gui_component_watch_entity_type (gint component_id,
                                      GNCIdTypeConst entity_type,
-                                     GNCEngineEventType event_mask)
+                                     QofEventId event_mask)
 {
   ComponentInfo *ci;
 
@@ -526,10 +525,10 @@ gnc_gui_component_watch_entity_type (gint component_id,
 }
 
 const EventInfo *
-gnc_gui_get_entity_events (GHashTable *changes, GUID *entity)
+gnc_gui_get_entity_events (GHashTable *changes, const GUID *entity)
 {
   if (!changes || !entity)
-    return GNC_EVENT_NONE;
+    return QOF_EVENT_NONE;
 
   return g_hash_table_lookup (changes, entity);
 }
@@ -557,7 +556,7 @@ gnc_unregister_gui_component (gint component_id)
   ci = find_component (component_id);
   if (!ci)
   {
-    PERR ("component not found");
+    PERR ("component %d not found", component_id);
     return;
   }
 
@@ -581,11 +580,11 @@ gnc_unregister_gui_component (gint component_id)
   ci->component_class = NULL;
 
   if (ci->refresh_handler_scm != SCM_BOOL_F)
-    scm_unprotect_object (ci->refresh_handler_scm);
+    scm_gc_unprotect_object (ci->refresh_handler_scm);
   ci->refresh_handler_scm = SCM_BOOL_F;
 
   if (ci->close_handler_scm != SCM_BOOL_F)
-    scm_unprotect_object (ci->close_handler_scm);
+    scm_gc_unprotect_object (ci->close_handler_scm);
   ci->close_handler_scm = SCM_BOOL_F;
 
   g_free (ci);
@@ -649,8 +648,8 @@ match_type_helper (gpointer key, gpointer value, gpointer user_data)
 {
   ComponentEventInfo *cei = user_data;
   GNCIdType id_type = key;
-  GNCEngineEventType * et = value;
-  GNCEngineEventType * et_2;
+  QofEventId * et = value;
+  QofEventId * et_2;
 
   et_2 = g_hash_table_lookup (cei->event_masks, id_type);
   if (!et_2)
@@ -747,7 +746,7 @@ gnc_gui_refresh_internal (gboolean force)
       continue;
 
     if (!ci->refresh_handler &&
-        !gh_procedure_p (ci->refresh_handler_scm)) {
+        !SCM_PROCEDUREP (ci->refresh_handler_scm)) {
 #if CM_DEBUG
       fprintf (stderr, "no handlers for %s:%d\n", ci->component_class, ci->component_id);
 #endif
@@ -765,7 +764,7 @@ gnc_gui_refresh_internal (gboolean force)
 #if CM_DEBUG
 	fprintf (stderr, "calling %s:%d SCM handler\n", ci->component_class, ci->component_id);
 #endif
-        gh_call0 (ci->refresh_handler_scm);
+        scm_call_0 (ci->refresh_handler_scm);
       }
     }
     else if (changes_match (&ci->watch_info, &changes_backup))
@@ -779,7 +778,7 @@ gnc_gui_refresh_internal (gboolean force)
 #if CM_DEBUG
 	fprintf (stderr, "calling %s:%d SCM handler\n", ci->component_class, ci->component_id);
 #endif
-        gh_call0 (ci->refresh_handler_scm);
+        scm_call_0 (ci->refresh_handler_scm);
       }
     }
     else
@@ -829,13 +828,13 @@ gnc_close_gui_component (gint component_id)
   }
 
   if (!ci->close_handler &&
-      !gh_procedure_p (ci->close_handler_scm))
+      !SCM_PROCEDUREP (ci->close_handler_scm))
     return;
 
   if (ci->close_handler)
     ci->close_handler (ci->user_data);
   else
-    gh_call0 (ci->close_handler_scm);
+    scm_call_0 (ci->close_handler_scm);
 }
 
 void
@@ -929,6 +928,10 @@ gnc_find_first_gui_component (const char *component_class,
   GList *list;
   gpointer user_data;
 
+#if CM_DEBUG
+  fprintf (stderr, "find: class %s, fn %p, data %p\n", component_class,
+	   find_handler, find_data);
+#endif
   if (!component_class)
     return NULL;
 
@@ -940,6 +943,9 @@ gnc_find_first_gui_component (const char *component_class,
 
   g_list_free (list);
 
+#if CM_DEBUG
+  fprintf (stderr, "found: data %p\n", user_data);
+#endif
   return user_data;
 }
 
