@@ -19,10 +19,11 @@
  *
 \********************************************************************/
 
-#include <string.h>
-#include <messages.h>
+#include "config.h"
 
-#include "i18n.h"
+#include <glib.h>
+#include <glib/gi18n.h>
+#include <string.h>
 
 #include "gnc-gettext-util.h"
 
@@ -31,18 +32,6 @@
 char *
 gnc_gettext_helper(const char *string)
 {
-  return strdup(_(string));
+  return strdup(string && *string ? _(string) : "");
 }
 
-/* ============================================================== */
-
-void
-gnc_setup_gettext(void)
-{
-#ifdef HAVE_GETTEXT
-  bindtextdomain (TEXT_DOMAIN, LOCALE_DIR);
-  textdomain (TEXT_DOMAIN);
-#endif
-}
-
-/* ============================================================== */
