@@ -17,8 +17,8 @@
  * along with this program; if not, contact:                        *
  *                                                                  *
  * Free Software Foundation           Voice:  +1-617-542-5942       *
- * 59 Temple Place - Suite 330        Fax:    +1-617-542-2652       *
- * Boston, MA  02111-1307,  USA       gnu@gnu.org                   *
+ * 51 Franklin Street, Fifth Floor    Fax:    +1-617-542-2652       *
+ * Boston, MA  02110-1301,  USA       gnu@gnu.org                   *
  *                                                                  *
  *******************************************************************/
 
@@ -29,7 +29,7 @@
 #include <string.h>
 
 #include "gnc-xml-helper.h"
-#include "gnc-engine-util.h"
+#include "qof.h"
 
 #include "sixtp.h"
 #include "sixtp-utils.h"
@@ -123,7 +123,7 @@ struct uiFreqTypeTuple uiFreqTypeStrs[] = {
 typedef struct
 {
         FreqSpec        *fs;              /* FreqSpec we're parsing into. */
-        GNCBook         *book;            /* Book we're loading into. */
+        QofBook         *book;            /* Book we're loading into. */
 
         /* fields used in the union of unions... :) */
         GDate                 once_day;     /* once */
@@ -154,10 +154,10 @@ gnc_freqSpec_dom_tree_create( FreqSpec *fs )
         xmlNodePtr                ret;
         xmlNodePtr                xmlSub;
 
-        ret = xmlNewNode( NULL, "gnc:freqspec" );
-        xmlSetProp( ret, "version", freqspec_version_string );
+        ret = xmlNewNode( NULL, BAD_CAST "gnc:freqspec" );
+        xmlSetProp( ret, BAD_CAST "version", BAD_CAST freqspec_version_string );
 
-        xmlAddChild( ret, guid_to_dom_tree( "fs:id", &fs->guid ) );
+        xmlAddChild( ret, guid_to_dom_tree( "fs:id", &fs->entity.guid ) );
 
         xmlSub = text_to_dom_tree( "fs:ui_type",
                                    uiFreqTypeStrs[ xaccFreqSpecGetUIType(fs) ].str );
@@ -166,11 +166,11 @@ gnc_freqSpec_dom_tree_create( FreqSpec *fs )
         switch( fs->type ) {
 
         case INVALID: {
-                xmlSub = xmlNewNode( NULL, "fs:none" );
+                xmlSub = xmlNewNode( NULL, BAD_CAST "fs:none" );
         } break;
 
         case ONCE: {
-                xmlSub = xmlNewNode( NULL, "fs:once" );
+                xmlSub = xmlNewNode( NULL, BAD_CAST "fs:once" );
                 xmlAddChild( xmlSub, 
                              gdate_to_dom_tree( "fs:date", 
                                                 &fs->s.once.date ) );
@@ -178,7 +178,7 @@ gnc_freqSpec_dom_tree_create( FreqSpec *fs )
         } break;
 
         case DAILY: {
-                        xmlSub = xmlNewNode( NULL, "fs:daily" );
+                        xmlSub = xmlNewNode( NULL, BAD_CAST "fs:daily" );
                         xmlAddChild( xmlSub, 
                                      guint_to_dom_tree(
                                              "fs:interval", 
@@ -193,7 +193,7 @@ gnc_freqSpec_dom_tree_create( FreqSpec *fs )
         } break;
         
         case WEEKLY: {
-                xmlSub = xmlNewNode( NULL, "fs:weekly" );
+                xmlSub = xmlNewNode( NULL, BAD_CAST "fs:weekly" );
                 xmlAddChild( xmlSub, 
                              guint_to_dom_tree( 
                                      "fs:interval", 
@@ -208,7 +208,7 @@ gnc_freqSpec_dom_tree_create( FreqSpec *fs )
         } break;
         
         case MONTHLY: {
-                xmlSub = xmlNewNode( NULL, "fs:monthly" );
+                xmlSub = xmlNewNode( NULL, BAD_CAST "fs:monthly" );
                 xmlAddChild( xmlSub, 
                              guint_to_dom_tree( 
                                      "fs:interval", 
@@ -228,7 +228,7 @@ gnc_freqSpec_dom_tree_create( FreqSpec *fs )
         } break;
         
         case MONTH_RELATIVE: {
-                xmlSub = xmlNewNode( NULL, "fs:month_relative" );
+                xmlSub = xmlNewNode( NULL, BAD_CAST "fs:month_relative" );
                 xmlAddChild( xmlSub, 
                              guint_to_dom_tree( 
                                      "fs:interval", 
@@ -255,7 +255,7 @@ gnc_freqSpec_dom_tree_create( FreqSpec *fs )
         case COMPOSITE: {
                 GList *subelts;
                 xmlNodePtr xmlComposites;
-                xmlComposites = xmlNewNode( NULL, "fs:composite" );
+                xmlComposites = xmlNewNode( NULL, BAD_CAST "fs:composite" );
                 subelts = fs->s.composites.subSpecs;
                 while( subelts ) {
                         xmlAddChild( xmlComposites,
@@ -527,7 +527,7 @@ fs_guid_handler( xmlNodePtr node, gpointer data)
         fsParseData *fspd = data;
         GUID        *guid;
         guid = dom_tree_to_guid( node );
-        fspd->fs->guid = *guid;
+        fspd->fs->entity.guid = *guid;
         return TRUE;
 }
 
@@ -605,7 +605,7 @@ gnc_freqSpec_sixtp_parser_create(void)
 }
 
 FreqSpec*
-dom_tree_to_freqSpec(xmlNodePtr node, GNCBook *book)
+dom_tree_to_freqSpec(xmlNodePtr node, QofBook *book)
 {
     gboolean        successful;
     fsParseData        fspd;
