@@ -16,14 +16,13 @@
  * along with this program; if not, contact:                        *
  *                                                                  *
  * Free Software Foundation           Voice:  +1-617-542-5942       *
- * 59 Temple Place - Suite 330        Fax:    +1-617-542-2652       *
- * Boston, MA  02111-1307,  USA       gnu@gnu.org                   *
+ * 51 Franklin Street, Fifth Floor    Fax:    +1-617-542-2652       *
+ * Boston, MA  02110-1301,  USA       gnu@gnu.org                   *
  *                                                                  *
  ********************************************************************/
 
 #include "config.h"
 
-#define _GNU_SOURCE
 #define __EXTENSIONS__
 
 #include <ctype.h>
@@ -35,25 +34,24 @@
 
 #include "sixtp.h"
 #include "sixtp-utils.h"
-#include "core-utils.h"
+#include <time.h>
+#include <errno.h>
 
-#include "date.h"
-#include "guid.h"
-#include "gnc-numeric.h"
-#include "gnc-engine-util.h"
+#ifdef GNUCASH_MAJOR_VERSION
 #ifndef HAVE_STRPTIME
 #include "strptime.h"
 #endif
 #ifndef HAVE_LOCALTIME_R
 #include "localtime_r.h"
 #endif
+#endif
 
-static short module = MOD_IO;
+static QofLogModule log_module = GNC_MOD_IO;
 
 gboolean
 isspace_str(const gchar *str, int nomorethan)
 {
-  const gchar *cursor = str;
+  const guchar *cursor = str;
   while(*cursor && (nomorethan != 0)) {
     if(!isspace(*cursor)) {
       return(FALSE);
@@ -175,17 +173,17 @@ string_to_double(const char *str, double *result)
     SCM conversion_result;
   
     if(!ready) {
-      string_to_number = gh_eval_str("string->number");
-      scm_protect_object(string_to_number);
+      string_to_number = scm_c_eval_string("string->number");
+      scm_gc_protect_object(string_to_number);
       ready = TRUE;
     }
     
-    conversion_result = gh_call1(string_to_number, gh_str02scm(str));
+    conversion_result = scm_call_1(string_to_number, scm_makfrom0str(str));
     if(!conversion_result == SCM_BOOL_F) {
       return(FALSE);
     }
   
-    *result = gh_scm2double(conversion_result);
+    *result = scm_num2dbl(conversion_result, __FUNCTION__);
   } 
   
 #else /* don't USE_GUILE_FOR_DOUBLE_CONVERSION */
@@ -209,7 +207,8 @@ string_to_double(const char *str, double *result)
 /*********/
 /* gint64
  */
-
+/* Maybe there should be a comment here explaining why this function
+   doesn't call g_ascii_strtoull, because it's not so obvious. -CAS */
 gboolean
 string_to_gint64(const gchar *str, gint64 *v)
 {
@@ -220,7 +219,7 @@ string_to_gint64(const gchar *str, gint64 *v)
   g_return_val_if_fail(str, FALSE);
   
   /* must use "<" here because %n's effects aren't well defined */
-  if(sscanf(str, " " GNC_SCANF_LLD "%n", &v_in, &num_read) < 1) {
+  if(sscanf(str, " " QOF_SCANF_LLD "%n", &v_in, &num_read) < 1) {
     return(FALSE);
   }
 
@@ -230,7 +229,7 @@ string_to_gint64(const gchar *str, gint64 *v)
    * is fixed in the next release 10.2 afaik
    */  
   while( (*((gchar*)str + num_read)!='\0') &&
-	 isspace(*((char*)str + num_read)))
+	 isspace(*((unsigned char*)str + num_read)))
     num_read++;
 
   if (v)
@@ -256,7 +255,7 @@ string_to_gint32(const gchar *str, gint32 *v)
     return(FALSE);
   }
   while( (*((gchar*)str + num_read)!='\0') &&
-	 isspace(*((char*)str + num_read)))
+	 isspace(*((unsigned char*)str + num_read)))
     num_read++;
 
   if (v)
@@ -274,7 +273,7 @@ gboolean
 hex_string_to_binary(const gchar *str,  void **v, guint64 *data_len)
 {
   /* Convert a hex string to binary.  No whitespace allowed. */
-  const gchar *cursor = str;
+  const guchar *cursor = str;
   guint64 str_len;
   gboolean error = FALSE;
   
@@ -387,27 +386,23 @@ static time_t
 gnc_timegm (struct tm *tm)
 {
   time_t result;
-  char *put_str;
   char *old_tz;
 
   old_tz = getenv ("TZ");
-
   /* FIXME: there's no way to report this error to the caller. */
-  if(gnc_setenv("TZ", "UTC", 1) != 0)
+  if (!g_setenv("TZ", "UTC", 1))
     PERR ("couldn't switch the TZ.");
-
   result = mktime (tm);
-
   if(old_tz)
   {
     /* FIXME: there's no way to report this error to the caller. */
-    if(gnc_setenv("TZ", old_tz, 1) != 0)
+    if (!g_setenv("TZ", old_tz, 1))
       PERR ("couldn't switch the TZ back.");
   }
   else
   {
     /* FIXME: there's no way to report this error to the caller. */
-    gnc_unsetenv("TZ");
+    g_unsetenv("TZ");
     if(errno != 0)
       PERR ("couldn't restore the TZ to undefined.");
   }
@@ -500,9 +495,12 @@ string_to_timespec_nsecs(const gchar *str, Timespec *ts)
 
   if (!str || !ts) return FALSE;
 
-  sscanf(str, " %ld%n", &nanosecs, &charcount);
+  /* The '%n' doesn't count as a conversion. */
+  if (1 != sscanf(str, " %ld%n", &nanosecs, &charcount))
+    return FALSE;
+
   while( (*((gchar*)str + charcount)!='\0') &&
-	 isspace(*((char*)str + charcount)))
+	 isspace(*((unsigned char*)str + charcount)))
     charcount++;
 
   if(charcount != strlen(str)) return(FALSE);
