@@ -16,20 +16,19 @@
  * along with this program; if not, contact:                        *
  *                                                                  *
  * Free Software Foundation           Voice:  +1-617-542-5942       *
- * 59 Temple Place - Suite 330        Fax:    +1-617-542-2652       *
- * Boston, MA  02111-1307,  USA       gnu@gnu.org                   *
+ * 51 Franklin Street, Fifth Floor    Fax:    +1-617-542-2652       *
+ * Boston, MA  02110-1301,  USA       gnu@gnu.org                   *
 \********************************************************************/
 
-
-#define _GNU_SOURCE
-
 #include "config.h"
 
-#include <ctype.h>
 #include <glib.h>
+#include <glib/gi18n.h>
+#include <ctype.h>
 #include <netdb.h>
 #include <pwd.h>
 #include <stdio.h>  
+#include <stdlib.h>  
 #include <string.h>  
 #include <sys/types.h>  
 #include <unistd.h>  
@@ -39,22 +38,12 @@
 #include <libpq-fe.h>  
 
 #include "AccountP.h"
-#include "Backend.h"
-#include "BackendP.h"
 #include "Group.h"
 #include "GroupP.h"
-#include "gnc-book.h"
-#include "gnc-book-p.h"
 #include "gnc-commodity.h"
 #include "gnc-engine.h"
-#include "gnc-engine-util.h"
-#include "gnc-event.h"
-#include "gnc-event-p.h"
 #include "gnc-pricedb.h"
 #include "gnc-pricedb-p.h"
-#include "guid.h"
-#include "GNCId.h"
-#include "GNCIdP.h"
 #include "TransactionP.h"
 
 #include "account.h"
@@ -64,14 +53,13 @@
 #include "events.h"
 #include "gncquery.h"
 #include "kvp-sql.h"
-#include "messages.h"
 #include "PostgresBackend.h"
 #include "price.h"
 #include "txn.h"
 #include "txnmass.h"
 #include "upgrade.h"
 
-static short module = MOD_BACKEND; 
+static QofLogModule log_module = GNC_MOD_BACKEND;
 
 #include "putil.h"
 
@@ -151,7 +139,7 @@ pgendAccountLookup (PGBackend *be, const GUID *acct_guid)
    ENTER("guid = %s", guid_to_string(acct_guid));
    for (node=be->blist; node; node=node->next)
    {
-      GNCBook *book = node->data;
+      QofBook *book = node->data;
       acc = xaccAccountLookup (acct_guid, book);
       if (acc) { LEAVE("acc = %p", acc); return acc; }
    }
@@ -169,7 +157,7 @@ pgendTransLookup (PGBackend *be, const GUID *txn_guid)
    ENTER("guid = %s", guid_to_string(txn_guid));
    for (node=be->blist; node; node=node->next)
    {
-      GNCBook *book = node->data;
+      QofBook *book = node->data;
       txn = xaccTransLookup (txn_guid, book);
       if (txn) { LEAVE("txt = %p", txn); return txn; }
    }
@@ -187,7 +175,7 @@ pgendSplitLookup (PGBackend *be, const GUID *split_guid)
    ENTER("guid = %s", guid_to_string(split_guid));
    for (node=be->blist; node; node=node->next)
    {
-      GNCBook *book = node->data;
+      QofBook *book = node->data;
       split = xaccSplitLookup (split_guid, book);
       if (split) { LEAVE("split = %p", split); return split; }
    }
@@ -196,35 +184,38 @@ pgendSplitLookup (PGBackend *be, const GUID *split_guid)
    return NULL;
 }
 
-GNCPrice *
-pgendPriceLookup (PGBackend *be, const GUID *price_guid)
+struct _iter {
+  const GUID *guid;
+  QofEntity *ent;
+};
+
+static void
+cforeach (QofCollection *col, gpointer data)
 {
-   GList *node;
-   GNCPrice * price = NULL;
-
-   ENTER("guid = %s", guid_to_string(price_guid));
-   for (node=be->blist; node; node=node->next)
-   {
-      GNCBook *book = node->data;
-      price = gnc_price_lookup (price_guid, book);
-      if (price) { LEAVE("price = %p", price); return price; }
-   }
-
-   return NULL;
+  struct _iter *iter = data;
+  if (iter->ent) return;
+  iter->ent = qof_collection_lookup_entity (col, iter->guid);
 }
 
-GNCIdType
+QofIdType
 pgendGUIDType (PGBackend *be, const GUID *guid)
 {
    GList *node;
-   GNCIdType tip = GNC_ID_NONE;
+   struct _iter iter;
+
+   iter.guid = guid;
+   iter.ent = NULL;
 
    ENTER("guid = %s", guid_to_string(guid));
    for (node=be->blist; node; node=node->next)
    {
-      GNCBook *book = node->data;
-      tip = xaccGUIDType (guid, book);
-      if (GNC_ID_NONE != tip) { LEAVE("tip = %s", tip); return tip; }
+      QofBook *book = node->data;
+      qof_book_foreach_collection (book, cforeach, &iter);
+      if (iter.ent) 
+      { 
+         LEAVE("tip = %s", iter.ent->e_type); 
+         return iter.ent->e_type; 
+      }
    }
 
    LEAVE("tip = NULL");
@@ -233,19 +224,19 @@ pgendGUIDType (PGBackend *be, const GUID *guid)
 
 /* ============================================================= */
 
-GNCBook *
+QofBook *
 pgendGetBook(PGBackend *pbe) {
-    GNCBook *book;
+    QofBook *book;
     
     ENTER(" ");
-    book = gnc_session_get_book(pbe->session);
+    book = qof_session_get_book(pbe->session);
 
     LEAVE("book = %p", book);
     return book;
 }
 
 static void 
-pgend_set_book (PGBackend *be, GNCBook *book)
+pgend_set_book (PGBackend *be, QofBook *book)
 {
    GList *node;
    be->book = book;
@@ -262,9 +253,9 @@ pgend_set_book (PGBackend *be, GNCBook *book)
 /* This routine finds the commodity by parsing a string
  * of the form NAMESPACE::MNEMONIC
  */
-
+/* FIXME: replace w/ gnc_commodity_table_lookup_unique */
 gnc_commodity *
-gnc_string_to_commodity (const char *str, GNCBook *book)
+gnc_string_to_commodity (const char *str, QofBook *book)
 {
    gnc_commodity_table *comtab;
    gnc_commodity *com;
@@ -273,7 +264,7 @@ gnc_string_to_commodity (const char *str, GNCBook *book)
    comtab = gnc_book_get_commodity_table (book);
 
    space = g_strdup(str);
-   name = strchr (space, ':');
+   name = strchr (space, ':'); /* BUG */
 
    if (!name)
    {
@@ -472,14 +463,14 @@ query_cb (PGBackend *be, PGresult *result, int j, gpointer data)
 
    xaccTransSetNum (trans, DB_GET_VAL("num",j));
    xaccTransSetDescription (trans, DB_GET_VAL("description",j));
-   ts = gnc_iso8601_to_timespec_local (DB_GET_VAL("date_posted",j));
+   ts = gnc_iso8601_to_timespec_gmt (DB_GET_VAL("date_posted",j));
    xaccTransSetDatePostedTS (trans, &ts);
-   ts = gnc_iso8601_to_timespec_local (DB_GET_VAL("date_entered",j));
+   ts = gnc_iso8601_to_timespec_gmt (DB_GET_VAL("date_entered",j));
    xaccTransSetDateEnteredTS (trans, &ts);
    xaccTransSetVersion (trans, atoi(DB_GET_VAL("version",j)));
    trans->idata = atoi(DB_GET_VAL("iguid",j));
 
-   currency = gnc_string_to_commodity (DB_GET_VAL("currency",j),
+   currency = gnc_string_to_commodity (DB_GET_VAL("currency",j), 
                                        pgendGetBook(be));
    if (currency)
      xaccTransSetCurrency (trans, currency);
@@ -575,13 +566,13 @@ pgendFillOutToCheckpoint (PGBackend *be, const char *query_string)
       Transaction *trans = (Transaction *) node->data;
       GList *engine_splits, *snode;
 
-      trans->kvp_data = pgendKVPFetch (be, trans->idata, trans->kvp_data);
+      trans->inst.kvp_data = pgendKVPFetch (be, trans->idata, trans->inst.kvp_data);
    
       engine_splits = xaccTransGetSplitList(trans);
       for (snode = engine_splits; snode; snode=snode->next)
       {
          Split *s = snode->data;
-         s->kvp_data = pgendKVPFetch (be, s->idata, s->kvp_data);
+         s->inst.kvp_data = pgendKVPFetch (be, s->idata, s->inst.kvp_data);
       }
 
       xaccTransCommitEdit (trans);
@@ -706,18 +697,18 @@ pgendFillOutToCheckpoint (PGBackend *be, const char *query_string)
 }
 
 static gpointer
-pgendCompileQuery (Backend *bend, Query *q)
+pgendCompileQuery (QofBackend *bend, Query *q)
 {
   return q;
 }
 
 static void
-pgendFreeQuery (Backend *bend, gpointer q)
+pgendFreeQuery (QofBackend *bend, gpointer q)
 {
 }
 
 static void 
-pgendRunQuery (Backend *bend, gpointer q_p)
+pgendRunQuery (QofBackend *bend, gpointer q_p)
 {
    PGBackend *be = (PGBackend *)bend;
    Query *q = (Query*) q_p;
@@ -729,7 +720,7 @@ pgendRunQuery (Backend *bend, gpointer q_p)
    if (!be || !q) { LEAVE("(null) args"); return; }
    be->version_check = (guint32) time(0);
 
-   gnc_engine_suspend_events();
+   qof_event_suspend();
    pgendDisable(be);
 
    /* first thing we do is convert the gnc-engine query into
@@ -767,7 +758,7 @@ pgendRunQuery (Backend *bend, gpointer q_p)
    xaccGroupMarkSaved (topgroup);
 
    pgendEnable(be);
-   gnc_engine_resume_events();
+   qof_event_resume();
 
    LEAVE (" ");
 }
@@ -792,7 +783,7 @@ get_all_trans_cb (PGBackend *be, PGresult *result, int j, gpointer data)
    GUID *trans_guid;
 
    /* find the transaction this goes into */
-   trans_guid = xaccGUIDMalloc();
+   trans_guid = guid_malloc();
    *trans_guid = nullguid;  /* just in case the read fails ... */
    string_to_guid (DB_GET_VAL("transGUID",j), trans_guid);
 
@@ -806,7 +797,7 @@ pgendGetAllTransactions (PGBackend *be, AccountGroup *grp)
 {
    GList *node, *xaction_list = NULL;
 
-   gnc_engine_suspend_events();
+   qof_event_suspend();
    pgendDisable(be);
 
    SEND_QUERY (be, "SELECT transGuid FROM gncTransaction;", );
@@ -817,13 +808,13 @@ pgendGetAllTransactions (PGBackend *be, AccountGroup *grp)
    for (node=xaction_list; node; node=node->next)
    {
       xxxpgendCopyTransactionToEngine (be, (GUID *)node->data);
-      xaccGUIDFree (node->data);
+      guid_free (node->data);
    }
    g_list_free(xaction_list);
    xaccAccountGroupCommitEdit (grp);
 
    pgendEnable(be);
-   gnc_engine_resume_events();
+   qof_event_resume();
 }
 #endif
 
@@ -877,7 +868,7 @@ pgendGetAllTransactions (PGBackend *be, AccountGroup *grp)
 
 
 static void
-pgendSync (Backend *bend, GNCBook *book)
+pgendSync (QofBackend *bend, QofBook *book)
 {
    PGBackend *be = (PGBackend *)bend;
    AccountGroup *grp = gnc_book_get_group (book);
@@ -907,7 +898,7 @@ pgendSync (Backend *bend, GNCBook *book)
    pgendStoreAllTransactions (be, grp);
 
    /* don't send events  to GUI, don't accept callbacks to backend */
-   gnc_engine_suspend_events();
+   qof_event_suspend();
    pgendDisable(be);
 
    pgendKVPInit(be);
@@ -915,7 +906,7 @@ pgendSync (Backend *bend, GNCBook *book)
    if ((MODE_SINGLE_FILE != be->session_mode) &&
        (MODE_SINGLE_UPDATE != be->session_mode))
    {
-      Timespec ts = gnc_iso8601_to_timespec_local (CK_BEFORE_LAST_DATE);
+      Timespec ts = gnc_iso8601_to_timespec_gmt (CK_BEFORE_LAST_DATE);
       pgendGroupGetAllBalances (be, grp, ts);
    } 
    else
@@ -927,12 +918,12 @@ pgendSync (Backend *bend, GNCBook *book)
    /* hack alert -- In some deranged theory, we should be
     * syncing prices here, as well as syncing any/all other
     * engine structures that need to be stored.  But instead,
-    * price syn is handled as a separate routine ...
+    * price sync is handled as a separate routine ...
     */
 
    /* re-enable events */
    pgendEnable(be);
-   gnc_engine_resume_events();
+   qof_event_resume();
 
    LEAVE(" ");
 }
@@ -961,7 +952,7 @@ trans_traverse_cb (Transaction *trans, void *cb_data)
 }
 
 static void
-pgendSyncSingleFile (Backend *bend, GNCBook *book)
+pgendSyncSingleFile (QofBackend *bend, QofBook *book)
 {
    char book_guid[40];
    char buff[4000];
@@ -985,7 +976,7 @@ pgendSyncSingleFile (Backend *bend, GNCBook *book)
    SEND_QUERY (be,p, );
    FINISH_QUERY(be->connection);
 
-   guid_to_string_buff (gnc_book_get_guid(book), book_guid);
+   guid_to_string_buff (qof_book_get_guid(book), book_guid);
 
    /* First, we delete all of the accounts, splits and transactions
     * associated with this book.  Its very tempting to just delete
@@ -1055,7 +1046,7 @@ pgendSyncSingleFile (Backend *bend, GNCBook *book)
  */
 
 static void
-pgendSyncPriceDB (Backend *bend, GNCBook *book)
+pgendSyncPriceDB (QofBackend *bend, QofBook *book)
 {
    PGBackend *be = (PGBackend *)bend;
    ENTER ("be=%p", be);
@@ -1078,14 +1069,14 @@ pgendSyncPriceDB (Backend *bend, GNCBook *book)
    pgendStorePriceDB (be, book);
 
    /* don't send events  to GUI, don't accept callbacks to backend */
-   gnc_engine_suspend_events();
+   qof_event_suspend();
    pgendDisable(be);
 
    pgendGetAllPricesInBook (be, book);
 
    /* re-enable events */
    pgendEnable(be);
-   gnc_engine_resume_events();
+   qof_event_resume();
 
    LEAVE(" ");
 }
@@ -1108,7 +1099,7 @@ pgendSyncPriceDB (Backend *bend, GNCBook *book)
  */
 
 static void
-pgendSyncPriceDBSingleFile (Backend *bend, GNCBook *book)
+pgendSyncPriceDBSingleFile (QofBackend *bend, QofBook *book)
 {
    char buff[400], *p;
    PGBackend *be = (PGBackend *)bend;
@@ -1120,7 +1111,7 @@ pgendSyncPriceDBSingleFile (Backend *bend, GNCBook *book)
    p = stpcpy (p, "BEGIN;\n"
                   "LOCK TABLE gncPrice IN EXCLUSIVE MODE;\n"
                   "DELETE FROM gncPrice WHERE bookGuid='\n");
-   p = guid_to_string_buff (gnc_book_get_guid(book), p);
+   p = guid_to_string_buff (qof_book_get_guid(book), p);
    p = stpcpy (p, "';");
    SEND_QUERY (be,buff, );
    FINISH_QUERY(be->connection);
@@ -1161,13 +1152,13 @@ pgendSessionGetMode (PGBackend *be)
 /* Instead of loading the book, just set the lock error */
 
 static void
-pgend_book_load_single_lockerr (Backend *bend, GNCBook *book)
+pgend_book_load_single_lockerr (QofBackend *bend, QofBook *book)
 {
    PGBackend *be = (PGBackend *)bend;
 
    if (!be) return;
 
-   xaccBackendSetError (&be->be, ERR_BACKEND_LOCKED);
+   qof_backend_set_error (&be->be, ERR_BACKEND_LOCKED);
 }
 
 /* ============================================================= */
@@ -1296,7 +1287,7 @@ pgendSessionValidate (PGBackend *be, int break_lock)
                                               is_gnucash_cb,
                                               GINT_TO_POINTER (FALSE)));
    if (FALSE == retval) {
-      xaccBackendSetError (&be->be, ERR_BACKEND_DATA_CORRUPT);
+      qof_backend_set_error (&be->be, ERR_BACKEND_DATA_CORRUPT);
       return FALSE;
    }
    
@@ -1334,7 +1325,7 @@ pgendSessionValidate (PGBackend *be, int break_lock)
    
    if (FALSE == retval)
    {
-      xaccBackendSetError (&be->be, ERR_BACKEND_PERM);
+      qof_backend_set_error (&be->be, ERR_BACKEND_PERM);
       p = "ROLLBACK;";
       SEND_QUERY (be,p, FALSE);
       FINISH_QUERY(be->connection);
@@ -1350,12 +1341,12 @@ pgendSessionValidate (PGBackend *be, int break_lock)
        * (The GUI allows users to break the lock, if desired).
        */
       be->be.load = pgend_book_load_single_lockerr;
-      xaccBackendSetError (&be->be, ERR_BACKEND_LOCKED);
+      qof_backend_set_error (&be->be, ERR_BACKEND_LOCKED);
       retval = FALSE;
    } else {
 
       /* make note of the session. */
-      be->sessionGuid = xaccGUIDMalloc();
+      be->sessionGuid = guid_malloc();
       guid_new (be->sessionGuid);
       guid_to_string_buff (be->sessionGuid, be->session_guid_str);
       pgendStoreOneSessionOnly (be, (void *)-1, SQL_INSERT);
@@ -1404,7 +1395,7 @@ pgendSessionEnd (PGBackend *be)
    SEND_QUERY (be,be->buff, );
    FINISH_QUERY(be->connection);
 
-   xaccGUIDFree (be->sessionGuid); be->sessionGuid = NULL;
+   guid_free (be->sessionGuid); be->sessionGuid = NULL;
    guid_to_string_buff (&nullguid, be->session_guid_str);
 }
 
@@ -1416,7 +1407,7 @@ pgendSessionEnd (PGBackend *be)
  */
 
 static void
-pgend_session_end (Backend *bend)
+pgend_session_end (QofBackend *bend)
 {
    int i;
    PGBackend *be = (PGBackend *)bend;
@@ -1487,18 +1478,16 @@ pgend_session_end (Backend *bend)
  *    and never the transactions, need to be loaded. 
  */
 static void
-pgend_book_load_poll (Backend *bend, GNCBook *book)
+pgend_book_load_poll (QofBackend *bend, QofBook *book)
 {
-   Timespec ts = gnc_iso8601_to_timespec_local (CK_BEFORE_LAST_DATE);
+   Timespec ts = gnc_iso8601_to_timespec_gmt (CK_BEFORE_LAST_DATE);
    AccountGroup *grp;
    PGBackend *be = (PGBackend *)bend;
 
-   ENTER("be = %p", bend);
-   
-   if (!be || !book) { LEAVE("(null) args"); return; }
+   if (!be) return;
 
    /* don't send events  to GUI, don't accept callbacks to backend */
-   gnc_engine_suspend_events();
+   qof_event_suspend();
    pgendDisable(be);
    be->version_check = (guint32) time(0);
 
@@ -1506,12 +1495,8 @@ pgend_book_load_poll (Backend *bend, GNCBook *book)
 
    if (be->blist) 
    {
+      PWARN ("old book list not empty--clearing it out ");
       /* XXX not clear what this means ... should we free old books ?? */
-      /* The old book list is set by the session when the session is 
-       * created.  It is an empty book, and should be discarded in favor
-       * of the Book retrieved from the database.
-       * PWARN ("old book list not empty ");
-       */
       g_list_free (be->blist);
       be->blist = NULL;
    }
@@ -1519,7 +1504,7 @@ pgend_book_load_poll (Backend *bend, GNCBook *book)
    pgend_set_book (be, book);
                         
    PINFO("Book GUID = %s\n",
-           guid_to_string(gnc_book_get_guid(book)));
+           guid_to_string(qof_book_get_guid(book)));
 
    pgendGetAllAccountsInBook (be, book);
 
@@ -1530,8 +1515,7 @@ pgend_book_load_poll (Backend *bend, GNCBook *book)
 
    /* re-enable events */
    pgendEnable(be);
-   gnc_engine_resume_events();
-   LEAVE(" ");
+   qof_event_resume();
 }
 
 /* ============================================================= */
@@ -1543,16 +1527,15 @@ pgend_book_load_poll (Backend *bend, GNCBook *book)
  */
 
 static void
-pgend_book_load_single (Backend *bend, GNCBook *book)
+pgend_book_load_single (QofBackend *bend, QofBook *book)
 {
    PGBackend *be = (PGBackend *)bend;
 
-   ENTER("be = %p", bend);
-   
-   if (!be || !book) { LEAVE("(null) args"); return; }
+   if (!be) return;
+
 
    /* don't send events  to GUI, don't accept callbacks to backend */
-   gnc_engine_suspend_events();
+   qof_event_suspend();
    pgendDisable(be);
    be->version_check = (guint32) time(0);
 
@@ -1560,9 +1543,10 @@ pgend_book_load_single (Backend *bend, GNCBook *book)
 
    if (be->blist) 
    {
+      PWARN ("old book list not empty--clearing it out ");
       /* XXX not clear what this means ... should we free old books ?? */
-      PWARN ("old book list not empty ");
       g_list_free (be->blist);
+      be->blist = NULL;
    }
    pgendBookRestore (be, book);
    pgend_set_book (be, book);
@@ -1573,8 +1557,7 @@ pgend_book_load_single (Backend *bend, GNCBook *book)
 
    /* re-enable events */
    pgendEnable(be);
-   gnc_engine_resume_events();
-   LEAVE(" ");
+   qof_event_resume();
 }
 
 /* ============================================================= */
@@ -1583,16 +1566,18 @@ pgend_book_load_single (Backend *bend, GNCBook *book)
  */
 
 static void
-pgend_price_load_single (Backend *bend, GNCBook *book)
+pgend_price_load_single (QofBackend *bend, QofBook *book)
 {
    PGBackend *be = (PGBackend *)bend;
 
-   if (!be || !book) return;
+   ENTER("be = %p", bend);
+   
+   if (!be || !book) { LEAVE("(null) args"); return; }
 
    pgend_set_book (be, book);
 
    /* don't send events  to GUI, don't accept callbacks to backend */
-   gnc_engine_suspend_events();
+   qof_event_suspend();
    pgendDisable(be);
    be->version_check = (guint32) time(0);
 
@@ -1600,18 +1585,19 @@ pgend_price_load_single (Backend *bend, GNCBook *book)
 
    /* re-enable events */
    pgendEnable(be);
-   gnc_engine_resume_events();
+   qof_event_resume();
+   LEAVE(" ");
 }
 
 /* ============================================================= */
 /*
- * These are the Backend Entry Points, which call into the various
+ * These are the QofBackend Entry Points, which call into the various
  * functions herein..  These are generic, and (eventually) pluggable.
  *
  */
 
 static void
-pgend_do_load_single (Backend *bend, GNCBook *book)
+pgend_do_load_single (QofBackend *bend, QofBook *book)
 {
   ENTER ("be=%p", bend);
   pgend_book_load_single (bend, book);
@@ -1622,7 +1608,7 @@ pgend_do_load_single (Backend *bend, GNCBook *book)
 }
 
 static void
-pgendDoSyncSingleFile (Backend *bend, GNCBook *book)
+pgendDoSyncSingleFile (QofBackend *bend, QofBook *book)
 {
   ENTER ("be=%p", bend);
   pgendSyncSingleFile (bend, book);
@@ -1633,7 +1619,7 @@ pgendDoSyncSingleFile (Backend *bend, GNCBook *book)
 }
 
 static void
-pgendDoSync (Backend *bend, GNCBook *book)
+pgendDoSync (QofBackend *bend, QofBook *book)
 {
   ENTER ("be=%p", bend);
   pgendSync (bend, book);
@@ -1644,20 +1630,21 @@ pgendDoSync (Backend *bend, GNCBook *book)
 }
 
 static void
-pgend_do_begin (Backend *bend, GNCIdTypeConst type, gpointer object)
+pgend_do_begin (QofBackend *bend, QofInstance *inst)
 {
   PGBackend *be = (PGBackend*)bend;
+  QofIdTypeConst type = inst->entity.e_type;
 
   ENTER ("be=%p, type=%s", bend, type);
-  if (!safe_strcmp (type, GNC_ID_PERIOD))
-    return pgend_book_transfer_begin (bend, object);
+  // if (!safe_strcmp (type, GNC_ID_PERIOD))
+  //   return pgend_book_transfer_begin (bend, object);
 
   switch (be->session_mode) {
       case MODE_EVENT:
       case MODE_POLL:
       case MODE_SINGLE_UPDATE:
           if (!safe_strcmp (type, GNC_ID_PRICE))
-              return pgend_price_begin_edit (bend, object);
+              return pgend_price_begin_edit (bend, (GNCPrice *) inst);
 
       case MODE_SINGLE_FILE:
       case MODE_NONE:
@@ -1669,13 +1656,14 @@ pgend_do_begin (Backend *bend, GNCIdTypeConst type, gpointer object)
 }
 
 static void
-pgend_do_commit (Backend *bend, GNCIdTypeConst type, gpointer object)
+pgend_do_commit (QofBackend *bend, QofInstance *inst)
 {
   PGBackend *be = (PGBackend*)bend;
+  QofIdTypeConst type = inst->entity.e_type;
 
   ENTER ("be=%p, type=%s", bend, type);
-  if (!safe_strcmp (type, GNC_ID_PERIOD))
-    return pgend_book_transfer_commit (bend, object);
+  // if (!safe_strcmp (type, GNC_ID_PERIOD))
+  //  return pgend_book_transfer_commit (bend, object);
 
   switch (be->session_mode) {
   case MODE_EVENT:
@@ -1683,15 +1671,15 @@ pgend_do_commit (Backend *bend, GNCIdTypeConst type, gpointer object)
   case MODE_SINGLE_UPDATE:
 
     if (!safe_strcmp (type, GNC_ID_TRANS)) {
-      Transaction *txn = (Transaction*) object;
+      Transaction *txn = (Transaction*) inst;
       return pgend_trans_commit_edit (bend, txn, txn->orig);
     }
 
     if (!safe_strcmp (type, GNC_ID_PRICE))
-      return pgend_price_commit_edit (bend, object);
+      return pgend_price_commit_edit (bend, (GNCPrice *) inst);
 
     if (!safe_strcmp (type, GNC_ID_ACCOUNT))
-      return pgend_account_commit_edit (bend, object);
+      return pgend_account_commit_edit (bend, (Account *) inst);
 
   case MODE_SINGLE_FILE:
   case MODE_NONE:
@@ -1704,9 +1692,10 @@ pgend_do_commit (Backend *bend, GNCIdTypeConst type, gpointer object)
 }
 
 static void
-pgend_do_rollback (Backend *bend, GNCIdTypeConst type, gpointer object)
+pgend_do_rollback (QofBackend *bend, QofInstance *inst)
 {
   PGBackend *be = (PGBackend*)bend;
+  QofIdTypeConst type = inst->entity.e_type;
 
   ENTER ("be=%p, type=%s", bend, type);
   switch (be->session_mode) {
@@ -1714,7 +1703,7 @@ pgend_do_rollback (Backend *bend, GNCIdTypeConst type, gpointer object)
   case MODE_POLL:
 
     if (!safe_strcmp (type, GNC_ID_TRANS))
-      return pgend_trans_rollback_edit (bend, object);
+      return pgend_trans_rollback_edit (bend, (Transaction *)inst);
 
   case MODE_SINGLE_UPDATE:
   case MODE_SINGLE_FILE:
@@ -1764,8 +1753,8 @@ pgend_create_db (PGBackend *be)
 }
 
 static void
-pgend_session_begin (Backend *backend,
-                     GNCSession *session,
+pgend_session_begin (QofBackend *backend,
+                     QofSession *session,
                      const char * sessionid, 
                      gboolean ignore_lock,
                      gboolean create_new_db)
@@ -1786,7 +1775,7 @@ pgend_session_begin (Backend *backend,
          sessionid ? sessionid : "(null)");
 
    /* close any dangling sessions from before; reinitialize */
-   pgend_session_end ((Backend *) be);
+   pgend_session_end ((QofBackend *) be);
    pgendInit (be);
 
    be->session = session;
@@ -1814,7 +1803,7 @@ pgend_session_begin (Backend *backend,
 
    if (strncmp (sessionid, "postgres://", 11)) 
    {
-      xaccBackendSetError (&be->be, ERR_BACKEND_BAD_URL);
+      qof_backend_set_error (&be->be, ERR_BACKEND_BAD_URL);
       return;
    }
    url = g_strdup(sessionid);
@@ -1841,7 +1830,7 @@ pgend_session_begin (Backend *backend,
    start = end+1;
    if (0x0 == *start) 
    { 
-      xaccBackendSetError (&be->be, ERR_BACKEND_BAD_URL);
+      qof_backend_set_error (&be->be, ERR_BACKEND_BAD_URL);
       g_free(url); 
       return; 
    }
@@ -1878,7 +1867,7 @@ pgend_session_begin (Backend *backend,
              PWARN ("the following message should be shown in a gui");
              PWARN ("unknown mode %s, will use multi-user mode",
                     start ? start : "(null)");
-             xaccBackendSetMessage((Backend *)be, _("Unknown database access mode '%s'. Using default mode: multi-user."),
+             qof_backend_set_message((QofBackend *)be, _("Unknown database access mode '%s'. Using default mode: multi-user."),
 			           start ? start : "(null)");
              be->session_mode = MODE_EVENT;
          } 
@@ -1995,8 +1984,8 @@ pgend_session_begin (Backend *backend,
             /* I wish that postgres returned usable error codes. 
              * Alas, it does not, so we just bomb out.
              */
-            xaccBackendSetError (&be->be, ERR_BACKEND_CANT_CONNECT);
-            xaccBackendSetMessage(&be->be, "From the Postgresql Server: %s", msg);
+            qof_backend_set_error (&be->be, ERR_BACKEND_CANT_CONNECT);
+            qof_backend_set_message(&be->be, _("From the Postgresql Server: %s"), msg);
             return;
          }
 
@@ -2020,12 +2009,12 @@ pgend_session_begin (Backend *backend,
             /* Weird.  We couldn't connect to the database, but it 
              * does seem to exist.  I presume that this is some 
              * sort of access control problem. */
-            xaccBackendSetError (&be->be, ERR_BACKEND_PERM);
+            qof_backend_set_error (&be->be, ERR_BACKEND_PERM);
             return;
          }
 
          /* Let GUI know that we connected, but we couldn't find it. */
-         xaccBackendSetError (&be->be, ERR_BACKEND_NO_SUCH_DB);
+         qof_backend_set_error (&be->be, ERR_BACKEND_NO_SUCH_DB);
          return;
       }
 
@@ -2046,7 +2035,7 @@ pgend_session_begin (Backend *backend,
           * upgrade the database contents. */
          PQfinish (be->connection);
          be->connection = NULL;
-         xaccBackendSetError (&be->be, ERR_SQL_DB_TOO_OLD);
+         qof_backend_set_error (&be->be, ERR_SQL_DB_TOO_OLD);
          return;
       }
    }
@@ -2078,7 +2067,7 @@ pgend_session_begin (Backend *backend,
          /* I wish that postgres returned usable error codes. 
           * Alas, it does not, so we just bomb out.
           */
-         xaccBackendSetError (&be->be, ERR_BACKEND_CANT_CONNECT);
+         qof_backend_set_error (&be->be, ERR_BACKEND_CANT_CONNECT);
          return;
       }
       
@@ -2135,7 +2124,7 @@ pgend_session_begin (Backend *backend,
             be->connection = NULL;
             /* We just created the database! If we can't connect now, 
              * the server is insane! */
-            xaccBackendSetError (&be->be, ERR_BACKEND_SERVER_ERR);
+            qof_backend_set_error (&be->be, ERR_BACKEND_SERVER_ERR);
             return;
          }
 
@@ -2177,7 +2166,7 @@ pgend_session_begin (Backend *backend,
              * just not to this database. Therefore, it must be a 
              * permission problem.
              */
-            xaccBackendSetError (&be->be, ERR_BACKEND_PERM);
+            qof_backend_set_error (&be->be, ERR_BACKEND_PERM);
             return;
          }
 
@@ -2212,7 +2201,7 @@ pgend_session_begin (Backend *backend,
              /* But first, make sure all users are logged off ... */
              p = "BEGIN;\n"
                "LOCK TABLE gncSession IN ACCESS EXCLUSIVE MODE;\n"
-               "SELECT time_off FROM gncSession WHERE time_off ='infinity';";
+               "SELECT time_off FROM gncSession WHERE time_off ='infinity';\n";
              SEND_QUERY (be,p, );
              someones_still_on =
                GPOINTER_TO_INT (pgendGetResults (be, has_results_cb,
@@ -2222,7 +2211,7 @@ pgend_session_begin (Backend *backend,
                p = "COMMIT;\n";
                SEND_QUERY (be,p, );
                FINISH_QUERY(be->connection);
-               xaccBackendSetError (&be->be, ERR_SQL_DB_BUSY);
+               qof_backend_set_error (&be->be, ERR_SQL_DB_BUSY);
                return;
              }
              p = "COMMIT;\n";
@@ -2466,7 +2455,7 @@ pgendInit (PGBackend *be)
    ENTER(" ");
    
    /* initialize global variable */
-   nullguid = *(xaccGUIDNULL());
+   nullguid = *(guid_null());
 
    /* access mode */
    be->session_mode = MODE_EVENT;
@@ -2474,7 +2463,7 @@ pgendInit (PGBackend *be)
    guid_to_string_buff (&nullguid, be->session_guid_str);
 
    /* generic backend handlers */
-   xaccInitBackend((Backend*)be);
+   qof_backend_init((QofBackend*)be);
 
    be->be.session_begin = pgend_session_begin;
    be->be.session_end = pgend_session_end;
@@ -2532,7 +2521,7 @@ pgendInit (PGBackend *be)
 
 /* ============================================================= */
 
-Backend * 
+QofBackend * 
 pgendNew (void)
 {
    PGBackend *be;
@@ -2541,8 +2530,32 @@ pgendNew (void)
    be = g_new0 (PGBackend, 1);
    pgendInit (be);
 
-   LEAVE(" ")
-   return (Backend *) be;
+   LEAVE(" ");
+   return (QofBackend *) be;
+}
+
+static void
+pg_provider_free (QofBackendProvider *prov)
+{
+        prov->provider_name = NULL;
+        prov->access_method = NULL;
+        g_free (prov);
+}
+
+G_MODULE_EXPORT const gchar *
+g_module_check_init(GModule *module)
+{
+	QofBackendProvider *prov;
+
+	prov = g_new0(QofBackendProvider, 1);
+	prov->provider_name = "The Postgres backend for GnuCash";
+	prov->access_method = "postgres";
+	prov->partial_book_supported = FALSE;
+	prov->backend_new = pgendNew;
+	prov->provider_free = pg_provider_free;
+	prov->check_data_type = NULL;
+	qof_backend_register_provider (prov);
+	return NULL;
 }
 
 /* ======================== END OF FILE ======================== */
