@@ -16,31 +16,24 @@
  * along with this program; if not, contact:                        *
  *                                                                  *
  * Free Software Foundation           Voice:  +1-617-542-5942       *
- * 59 Temple Place - Suite 330        Fax:    +1-617-542-2652       *
- * Boston, MA  02111-1307,  USA       gnu@gnu.org                   *
+ * 51 Franklin Street, Fifth Floor    Fax:    +1-617-542-2652       *
+ * Boston, MA  02110-1301,  USA       gnu@gnu.org                   *
 \********************************************************************/
 
-
-#define _GNU_SOURCE
-
 #include "config.h"
 
 #include <glib.h>
+#include <stdlib.h>  
 #include <string.h>  
 
 #include <libpq-fe.h>  
  
 #include "AccountP.h"
-#include "Backend.h"
-#include "BackendP.h"
+#include "qof.h"
 #include "Group.h"
 #include "GroupP.h"
-#include "gnc-book.h"
-#include "gnc-book-p.h"
 #include "gnc-commodity.h"
-#include "gnc-engine-util.h"
-#include "gnc-event.h"
-#include "guid.h"
+#include "gnc-pricedb.h"
 
 #include "account.h"
 #include "book.h"
@@ -49,7 +42,7 @@
 #include "PostgresBackend.h"
 #include "price.h"
 
-static short module = MOD_BACKEND; 
+static QofLogModule log_module = GNC_MOD_BACKEND; 
 
 #include "putil.h"
 
@@ -90,7 +83,7 @@ pgendStoreAccountNoLock (PGBackend *be, Account *acct,
    const gnc_commodity *com;
 
    if (!be || !acct) return;
-   if ((FALSE == do_mark) && (FALSE == acct->core_dirty)) return;
+   if ((FALSE == do_mark) && (FALSE == acct->inst.dirty)) return;
 
    ENTER ("acct=%p, mark=%d", acct, do_mark);
 
@@ -137,7 +130,7 @@ pgendStoreAccountNoLock (PGBackend *be, Account *acct,
    if (acct->idata)
    {
       pgendKVPDelete (be, acct->idata);
-      pgendKVPStore (be, acct->idata, acct->kvp_data);
+      pgendKVPStore (be, acct->idata, acct->inst.kvp_data);
    }
    LEAVE(" ");
 }
@@ -225,7 +218,7 @@ restore_cb (Account *acc, void * cb_data)
 {
    PGBackend *be = (PGBackend *) cb_data;
    if (0 == acc->idata) return NULL;
-   acc->kvp_data = pgendKVPFetch (be, acc->idata, acc->kvp_data);
+   acc->inst.kvp_data = pgendKVPFetch (be, acc->idata, acc->inst.kvp_data);
    return NULL;
 }
 
@@ -257,7 +250,7 @@ typedef struct
 
 typedef struct
 {
-  GNCBook * book;
+  QofBook * book;
   GList * resolve_info;
 } GetAccountData;
 
@@ -272,7 +265,7 @@ static gpointer
 get_account_cb (PGBackend *be, PGresult *result, int j, gpointer data)
 {
    GetAccountData * gad = data;
-   GNCBook * book = gad->book;
+   QofBook * book = gad->book;
    Account *parent;
    Account *acc;
    GUID acct_guid;
@@ -323,7 +316,7 @@ get_account_cb (PGBackend *be, PGresult *result, int j, gpointer data)
    PINFO ("parent GUID=%s", DB_GET_VAL("parentGUID",j));
    acct_guid = nullguid;  /* just in case the read fails ... */
    string_to_guid (DB_GET_VAL("parentGUID",j), &acct_guid);
-   if (guid_equal(xaccGUIDNULL(), &acct_guid)) 
+   if (guid_equal(guid_null(), &acct_guid)) 
    {
       /* if the parent guid is null, then this
        * account belongs in the top group */
@@ -359,7 +352,7 @@ get_account_cb (PGBackend *be, PGresult *result, int j, gpointer data)
 }
 
 static void
-pgendGetAccounts (PGBackend *be, GNCBook *book)
+pgendGetAccounts (PGBackend *be, QofBook *book)
 {
   GetAccountData gad;
 
@@ -428,7 +421,7 @@ pgendGetAccounts (PGBackend *be, GNCBook *book)
 void
 pgendGetAllAccounts (PGBackend *be)
 {
-   BookList *node;
+   QofBookList *node;
    char * bufp;
 
    ENTER ("be=%p", be);
@@ -447,7 +440,7 @@ pgendGetAllAccounts (PGBackend *be)
 
    for (node=be->blist; node; node=node->next)
    {
-      GNCBook *book = node->data;
+      QofBook *book = node->data;
       AccountGroup *topgrp = gnc_book_get_group (book);
       pgendGetAllAccountKVP (be, topgrp);
 
@@ -461,7 +454,7 @@ pgendGetAllAccounts (PGBackend *be)
 }
 
 void
-pgendGetAllAccountsInBook (PGBackend *be, GNCBook *book)
+pgendGetAllAccountsInBook (PGBackend *be, QofBook *book)
 {
    char *p, buff[400];
    AccountGroup *topgrp;
@@ -476,7 +469,7 @@ pgendGetAllAccountsInBook (PGBackend *be, GNCBook *book)
 
    p = buff;
    p = stpcpy (p, "SELECT * FROM gncAccount WHERE bookGuid='");
-   p = guid_to_string_buff (gnc_book_get_guid(book), p);
+   p = guid_to_string_buff (qof_book_get_guid(book), p);
    p = stpcpy (p, "';");
    SEND_QUERY (be, buff, );
    pgendGetAccounts (be, book);
@@ -506,7 +499,7 @@ pgendCopyAccountToEngine (PGBackend *be, const GUID *acct_guid)
    if (!be || !acct_guid) return 0;
 
    /* disable callbacks into the backend, and events to GUI */
-   gnc_engine_suspend_events();
+   qof_event_suspend();
    pgendDisable(be);
 
    /* First, see if we already have such an account */
@@ -550,7 +543,7 @@ pgendCopyAccountToEngine (PGBackend *be, const GUID *acct_guid)
       {
          if (acc->idata)
          {
-            acc->kvp_data = pgendKVPFetch (be, acc->idata, acc->kvp_data);
+            acc->inst.kvp_data = pgendKVPFetch (be, acc->idata, acc->inst.kvp_data);
          }
 
          acc->version_check = be->version_check;
@@ -559,7 +552,7 @@ pgendCopyAccountToEngine (PGBackend *be, const GUID *acct_guid)
 
    /* re-enable events to the backend and GUI */
    pgendEnable(be);
-   gnc_engine_resume_events();
+   qof_event_resume();
 
    LEAVE (" ");
    return acc;
@@ -572,18 +565,18 @@ pgendCopyAccountToEngine (PGBackend *be, const GUID *acct_guid)
 /* ============================================================= */
 
 void
-pgend_account_commit_edit (Backend * bend, 
+pgend_account_commit_edit (QofBackend * bend, 
                            Account * acct)
 {
    AccountGroup *parent;
    char *p;
-   GNCBackendError err;
+   QofBackendError err;
    PGBackend *be = (PGBackend *)bend;
 
    ENTER ("be=%p, acct=%p", be, acct);
    if (!be || !acct) return;
 
-   if (FALSE == acct->core_dirty)
+   if (FALSE == acct->inst.dirty)
    {
       parent = xaccAccountGetParent(acct);
       if (parent) parent->saved = 1;
@@ -605,7 +598,7 @@ pgend_account_commit_edit (Backend * bend,
     * made changes, and we must roll back. */
    if (0 < pgendAccountCompareVersion (be, acct))
    {
-      acct->do_free = FALSE;
+      acct->inst.do_free = FALSE;
       p = "ROLLBACK;";
       SEND_QUERY (be,p,);
       FINISH_QUERY(be->connection);
@@ -615,14 +608,14 @@ pgend_account_commit_edit (Backend * bend,
       PWARN(" account data in engine is newer\n"
             " account must be rolled back.  This function\n"
             " is not completely implemented !! \n");
-      xaccBackendSetError (&be->be, ERR_BACKEND_MODIFIED);
+      qof_backend_set_error (&be->be, ERR_BACKEND_MODIFIED);
       LEAVE ("rolled back");
       return;
    }
    acct->version ++;   /* be sure to update the version !! */
    acct->version_check = be->version_check;
 
-   if (acct->do_free)
+   if (acct->inst.do_free)
    {
       const GUID *guid = xaccAccountGetGUID(acct);
       pgendKVPDelete (be, acct->idata);
