@@ -17,8 +17,8 @@
  * along with this program; if not, contact:                        *
  *                                                                  *
  * Free Software Foundation           Voice:  +1-617-542-5942       *
- * 59 Temple Place - Suite 330        Fax:    +1-617-542-2652       *
- * Boston, MA  02111-1307,  USA       gnu@gnu.org                   *
+ * 51 Franklin Street, Fifth Floor    Fax:    +1-617-542-2652       *
+ * Boston, MA  02110-1301,  USA       gnu@gnu.org                   *
  *                                                                  *
 \********************************************************************/
 
@@ -47,13 +47,10 @@
 #include "gncTaxTableP.h"
 #include "gnc-entry-xml-v2.h"
 #include "gnc-owner-xml-v2.h"
-#include "gnc-engine-util.h"
 
-#include "gncObject.h"
+#define _GNC_MOD_NAME	GNC_ID_ENTRY
 
-#define _GNC_MOD_NAME	GNC_ENTRY_MODULE_NAME
-
-static short module = MOD_IO;
+static QofLogModule log_module = GNC_MOD_IO;
 
 const gchar *entry_version_string = "2.0.0";
 
@@ -119,11 +116,11 @@ entry_dom_tree_create (GncEntry *entry)
     GncOrder *order;
     GncInvoice *invoice;
 
-    ret = xmlNewNode(NULL, gnc_entry_string);
-    xmlSetProp(ret, "version", entry_version_string);
+    ret = xmlNewNode(NULL, BAD_CAST gnc_entry_string);
+    xmlSetProp(ret, BAD_CAST "version", BAD_CAST entry_version_string);
 
     xmlAddChild(ret, guid_to_dom_tree(entry_guid_string,
-				      gncEntryGetGUID (entry)));
+				      qof_instance_get_guid(QOF_INSTANCE(entry))));
 
     ts = gncEntryGetDate (entry);
     xmlAddChild(ret, timespec_to_dom_tree (entry_date_string, &ts));
@@ -143,7 +140,7 @@ entry_dom_tree_create (GncEntry *entry)
     acc = gncEntryGetInvAccount (entry);
     if (acc)
       xmlAddChild (ret, guid_to_dom_tree (entry_invacct_string,
-					  xaccAccountGetGUID (acc)));
+					  qof_instance_get_guid(QOF_INSTANCE(acc))));
 
     maybe_add_numeric (ret, entry_iprice_string, gncEntryGetInvPrice (entry));
 
@@ -152,7 +149,7 @@ entry_dom_tree_create (GncEntry *entry)
     invoice = gncEntryGetInvoice (entry);
     if (invoice) {
       xmlAddChild (ret, guid_to_dom_tree (entry_invoice_string,
-					  gncInvoiceGetGUID (invoice)));
+					  qof_instance_get_guid(QOF_INSTANCE(invoice))));
 
       xmlAddChild(ret, text_to_dom_tree(entry_idisctype_string,
 					gncAmountTypeToString (
@@ -170,14 +167,14 @@ entry_dom_tree_create (GncEntry *entry)
     taxtable = gncEntryGetInvTaxTable (entry);
     if (taxtable)
       xmlAddChild (ret, guid_to_dom_tree (entry_itaxtable_string,
-					  gncTaxTableGetGUID (taxtable)));
+					  qof_instance_get_guid (QOF_INSTANCE(taxtable))));
 
     /* vendor bills */
 
     acc = gncEntryGetBillAccount (entry);
     if (acc)
       xmlAddChild (ret, guid_to_dom_tree (entry_billacct_string,
-					  xaccAccountGetGUID (acc)));
+					  qof_instance_get_guid (QOF_INSTANCE(acc))));
 
     maybe_add_numeric (ret, entry_bprice_string, gncEntryGetBillPrice (entry));
 
@@ -185,7 +182,7 @@ entry_dom_tree_create (GncEntry *entry)
     if (invoice) {
       GncOwner *owner;
       xmlAddChild (ret, guid_to_dom_tree (entry_bill_string,
-					  gncInvoiceGetGUID (invoice)));
+					  qof_instance_get_guid(QOF_INSTANCE(invoice))));
       xmlAddChild(ret, int_to_dom_tree(entry_billable_string,
 				       gncEntryGetBillable (entry)));
       owner = gncEntryGetBillTo (entry);
@@ -203,14 +200,14 @@ entry_dom_tree_create (GncEntry *entry)
     taxtable = gncEntryGetBillTaxTable (entry);
     if (taxtable)
       xmlAddChild (ret, guid_to_dom_tree (entry_btaxtable_string,
-					  gncTaxTableGetGUID (taxtable)));
+					  qof_instance_get_guid (QOF_INSTANCE(taxtable))));
 
     /* Other stuff */
 
     order = gncEntryGetOrder (entry);
     if (order)
       xmlAddChild (ret, guid_to_dom_tree (entry_order_string,
-					  gncOrderGetGUID (order)));
+					  qof_instance_get_guid(QOF_INSTANCE (order))));
 
     return ret;
 }
@@ -220,11 +217,11 @@ entry_dom_tree_create (GncEntry *entry)
 struct entry_pdata
 {
   GncEntry *entry;
-  GNCBook *book;
+  QofBook *book;
   Account *acc;
 };
 
-static gboolean
+static inline gboolean
 set_string(xmlNodePtr node, GncEntry* entry,
            void (*func)(GncEntry *entry, const char *txt))
 {
@@ -236,19 +233,18 @@ set_string(xmlNodePtr node, GncEntry* entry,
   return TRUE;
 }
 
-static gboolean
+static inline gboolean
 set_timespec(xmlNodePtr node, GncEntry* entry,
            void (*func)(GncEntry *entry, Timespec ts))
 {
-  Timespec *ts = dom_tree_to_timespec (node);
-  g_return_val_if_fail(ts, FALSE);
+  Timespec ts = dom_tree_to_timespec (node);
+  if (!dom_tree_valid_timespec(&ts, node->name)) return FALSE;
     
-  func(entry, *ts);
-  g_free(ts);
+  func(entry, ts);
   return TRUE;
 }
 
-static gboolean
+static inline gboolean
 set_numeric(xmlNodePtr node, GncEntry* entry,
            void (*func)(GncEntry *entry, gnc_numeric num))
 {
@@ -260,7 +256,7 @@ set_numeric(xmlNodePtr node, GncEntry* entry,
   return TRUE;
 }
 
-static gboolean
+static inline gboolean
 set_boolean(xmlNodePtr node, GncEntry* entry,
 	    void (*func)(GncEntry *entry, gboolean val))
 {
@@ -272,7 +268,7 @@ set_boolean(xmlNodePtr node, GncEntry* entry,
     return TRUE;
 }
 
-static gboolean
+static inline gboolean
 set_account(xmlNodePtr node, struct entry_pdata *pdata,
 	    void (*func)(GncEntry *entry, Account *acc))
 {
@@ -292,7 +288,7 @@ set_account(xmlNodePtr node, struct entry_pdata *pdata,
     return TRUE;
 }
 
-static gboolean
+static inline gboolean
 set_taxtable (xmlNodePtr node, struct entry_pdata *pdata,
 	      void (*func)(GncEntry *entry, GncTaxTable *taxtable))
 {
@@ -708,7 +704,7 @@ static struct dom_tree_handler entry_handlers_v2[] = {
 };
 
 static GncEntry*
-dom_tree_to_entry (xmlNodePtr node, GNCBook *book)
+dom_tree_to_entry (xmlNodePtr node, QofBook *book)
 {
     struct entry_pdata entry_pdata;
     gboolean successful;
@@ -749,7 +745,7 @@ gnc_entry_end_handler(gpointer data_for_children,
     GncEntry *entry;
     xmlNodePtr tree = (xmlNodePtr)data_for_children;
     gxpf_data *gdata = (gxpf_data*)global_data;
-    GNCBook *book = gdata->bookdata;
+    QofBook *book = gdata->bookdata;
 
     successful = TRUE;
 
@@ -785,25 +781,25 @@ entry_sixtp_parser_create(void)
 }
 
 static void
-do_count (gpointer entry_p, gpointer count_p)
+do_count (QofEntity * entry_p, gpointer count_p)
 {
   int *count = count_p;
   (*count)++;
 }
 
 static int
-entry_get_count (GNCBook *book)
+entry_get_count (QofBook *book)
 {
   int count = 0;
-  gncObjectForeach (_GNC_MOD_NAME, book, do_count, (gpointer) &count);
+  qof_object_foreach (_GNC_MOD_NAME, book, do_count, (gpointer) &count);
   return count;
 }
 
 static void
-xml_add_entry (gpointer entry_p, gpointer out_p)
+xml_add_entry (QofEntity * entry_p, gpointer out_p)
 {
   xmlNodePtr node;
-  GncEntry *entry = entry_p;
+  GncEntry *entry = (GncEntry *) entry_p;
   FILE *out = out_p;
 
   /* Don't save non-attached entries! */
@@ -818,9 +814,16 @@ xml_add_entry (gpointer entry_p, gpointer out_p)
 }
 
 static void
-entry_write (FILE *out, GNCBook *book)
+entry_write (FILE *out, QofBook *book)
 {
-  gncObjectForeach (_GNC_MOD_NAME, book, xml_add_entry, (gpointer) out);
+  qof_object_foreach (_GNC_MOD_NAME, book, xml_add_entry, (gpointer) out);
+}
+
+static void
+entry_ns(FILE *out)
+{
+  g_return_if_fail(out);
+  gnc_xml2_write_namespace_decl(out, "entry");
 }
 
 void
@@ -834,9 +837,10 @@ gnc_entry_xml_initialize (void)
     entry_get_count,
     entry_write,
     NULL,			/* scrub */
+    entry_ns,
   };
 
-  gncObjectRegisterBackend (_GNC_MOD_NAME,
+  qof_object_register_backend (_GNC_MOD_NAME,
 			    GNC_FILE_BACKEND,
 			    &be_data);
 }
