@@ -1,5 +1,26 @@
+/********************************************************************\
+ * gncEntry.c -- the Core Business Entry Interface                  *
+ *                                                                  *
+ * This program is free software; you can redistribute it and/or    *
+ * modify it under the terms of the GNU General Public License as   *
+ * published by the Free Software Foundation; either version 2 of   *
+ * the License, or (at your option) any later version.              *
+ *                                                                  *
+ * This program is distributed in the hope that it will be useful,  *
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of   *
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the    *
+ * GNU General Public License for more details.                     *
+ *                                                                  *
+ * You should have received a copy of the GNU General Public License*
+ * along with this program; if not, contact:                        *
+ *                                                                  *
+ * Free Software Foundation           Voice:  +1-617-542-5942       *
+ * 51 Franklin Street, Fifth Floor    Fax:    +1-617-542-2652       *
+ * Boston, MA  02110-1301,  USA       gnu@gnu.org                   *
+ *                                                                  *
+\********************************************************************/
+
 /*
- * gncEntry.c -- the Core Business Entry Interface
  * Copyright (C) 2001,2002 Derek Atkins
  * Author: Derek Atkins <warlord@MIT.EDU>
  */
@@ -8,25 +29,17 @@
 
 #include <glib.h>
 
-#include "messages.h"
-#include "gnc-numeric.h"
-#include "gnc-engine-util.h"
-#include "gnc-book-p.h"
-#include "GNCIdP.h"
-#include "QueryObject.h"
-#include "gnc-event-p.h"
-#include "gnc-be-utils.h"
+#include "gnc-commodity.h"
 
-#include "gncBusiness.h"
 #include "gncEntry.h"
 #include "gncEntryP.h"
 #include "gncInvoice.h"
 #include "gncOrder.h"
 
-struct _gncEntry {
-  GNCBook *	book;
+struct _gncEntry 
+{
+  QofInstance inst;
 
-  GUID		guid;
   Timespec	date;
   Timespec	date_entered;
   char *	desc;
@@ -61,12 +74,7 @@ struct _gncEntry {
   GncInvoice *	invoice;
   GncInvoice *	bill;
 
-  int		editlevel;
-  gboolean	do_free;
-  gboolean	dirty;
-
   /* CACHED VALUES */
-
   gboolean	values_dirty;
 
   /* customer invoice */
@@ -88,7 +96,7 @@ struct _gncEntry {
   Timespec	b_taxtable_modtime;
 };
 
-static short	module = MOD_BUSINESS;
+static QofLogModule log_module = GNC_MOD_BUSINESS;
 
 /* You must edit the functions in this block in tandem.  KEEP THEM IN
    SYNC! */
@@ -145,10 +153,7 @@ gboolean gncEntryPaymentStringToType (const char *str, GncEntryPaymentType *type
 }
 #undef GNC_RETURN_ON_MATCH
 
-#define _GNC_MOD_NAME	GNC_ENTRY_MODULE_NAME
-
-#define CACHE_INSERT(str) g_cache_insert(gnc_engine_get_string_cache(), (gpointer)(str));
-#define CACHE_REMOVE(str) g_cache_remove(gnc_engine_get_string_cache(), (str));
+#define _GNC_MOD_NAME	GNC_ID_ENTRY
 
 #define SET_STR(obj, member, str) { \
 	char * tmp; \
@@ -160,22 +165,17 @@ gboolean gncEntryPaymentStringToType (const char *str, GncEntryPaymentType *type
 	member = tmp; \
 	}
 
-static void addObj (GncEntry *entry);
-static void remObj (GncEntry *entry);
-
 G_INLINE_FUNC void mark_entry (GncEntry *entry);
-G_INLINE_FUNC void
-mark_entry (GncEntry *entry)
+void mark_entry (GncEntry *entry)
 {
-  entry->dirty = TRUE;
-  gncBusinessSetDirtyFlag (entry->book, _GNC_MOD_NAME, TRUE);
-
-  gnc_engine_generate_event (&entry->guid, GNC_EVENT_MODIFY);
+  qof_instance_set_dirty(&entry->inst);
+  qof_event_gen (&entry->inst.entity, QOF_EVENT_MODIFY, NULL);
 }
 
+/* ================================================================ */
 /* Create/Destroy Functions */
 
-GncEntry *gncEntryCreate (GNCBook *book)
+GncEntry *gncEntryCreate (QofBook *book)
 {
   GncEntry *entry;
   gnc_numeric zero = gnc_numeric_zero ();
@@ -183,7 +183,7 @@ GncEntry *gncEntryCreate (GNCBook *book)
   if (!book) return NULL;
 
   entry = g_new0 (GncEntry, 1);
-  entry->book = book;
+  qof_instance_init (&entry->inst, _GNC_MOD_NAME, book);
 
   entry->desc = CACHE_INSERT ("");
   entry->action = CACHE_INSERT ("");
@@ -203,10 +203,7 @@ GncEntry *gncEntryCreate (GNCBook *book)
 
   entry->values_dirty = TRUE;
 
-  xaccGUIDNew (&entry->guid, book);
-  addObj (entry);
-
-  gnc_engine_generate_event (&entry->guid, GNC_EVENT_CREATE);
+  qof_event_gen (&entry->inst.entity, QOF_EVENT_CREATE, NULL);
 
   return entry;
 }
@@ -214,7 +211,7 @@ GncEntry *gncEntryCreate (GNCBook *book)
 void gncEntryDestroy (GncEntry *entry)
 {
   if (!entry) return;
-  entry->do_free = TRUE;
+  entry->inst.do_free = TRUE;
   gncEntryCommitEdit(entry);
 }
 
@@ -222,7 +219,7 @@ static void gncEntryFree (GncEntry *entry)
 {
   if (!entry) return;
 
-  gnc_engine_generate_event (&entry->guid, GNC_EVENT_DESTROY);
+  qof_event_gen (&entry->inst.entity, QOF_EVENT_DESTROY, NULL);
 
   CACHE_REMOVE (entry->desc);
   CACHE_REMOVE (entry->action);
@@ -235,33 +232,58 @@ static void gncEntryFree (GncEntry *entry)
     gncTaxTableDecRef (entry->i_tax_table);
   if (entry->b_tax_table)
     gncTaxTableDecRef (entry->b_tax_table);
-  remObj (entry);
 
+  qof_instance_release (&entry->inst);
   g_free (entry);
 }
 
-/* Set Functions */
-
-void gncEntrySetGUID (GncEntry *entry, const GUID *guid)
+GncEntry *
+gncCloneEntry (GncEntry *from, QofBook *book)
 {
-  if (!entry || !guid) return;
-  if (guid_equal (guid, &entry->guid)) return;
-
-  gncEntryBeginEdit (entry);
-  remObj (entry);
-  entry->guid = *guid;
-  addObj (entry);
-  gncEntryCommitEdit (entry);
+  /* XXX unfinished */
+  return NULL;
 }
 
+GncEntry *
+gncEntryObtainTwin (GncEntry *from, QofBook *book)
+{
+  GncEntry *entry;
+  if (!book) return NULL;
+
+  entry = (GncEntry *) qof_instance_lookup_twin (QOF_INSTANCE(from), book);
+  if (!entry)
+  {
+    entry = gncCloneEntry (from, book);
+  }
+
+  return entry;
+}
+
+
+/* ================================================================ */
+/* Set Functions */
+
 void gncEntrySetDate (GncEntry *entry, Timespec date)
 {
+  gboolean first_date = FALSE;
+  Timespec zero_time = { 0, 0 };
+
   if (!entry) return;
   if (timespec_equal (&entry->date, &date)) return;
+  if (timespec_equal (&entry->date, &zero_time))
+    first_date = TRUE;
   gncEntryBeginEdit (entry);
   entry->date = date;
   mark_entry (entry);
   gncEntryCommitEdit (entry);
+
+  /* Don't re-sort the first time we set the date on this entry */
+  if (!first_date) {
+    if (entry->invoice)
+      gncInvoiceSortEntries(entry->invoice);
+    if (entry->bill)
+      gncInvoiceSortEntries(entry->bill);
+  }
 }
 
 void gncEntrySetDateEntered (GncEntry *entry, Timespec date)
@@ -404,6 +426,35 @@ void gncEntrySetInvDiscountHow (GncEntry *entry, GncDiscountHow how)
   gncEntryCommitEdit (entry);
 }
 
+void qofEntrySetInvDiscType (GncEntry *entry, const char *type_string)
+{
+	GncAmountType type;
+
+	if (!entry) return;
+	gncAmountStringToType(type_string, &type);
+	if (entry->i_disc_type == type) return;
+	gncEntryBeginEdit (entry);
+	entry->i_disc_type = type;
+	entry->values_dirty = TRUE;
+	mark_entry (entry);
+	gncEntryCommitEdit (entry);
+
+}
+
+void qofEntrySetInvDiscHow  (GncEntry *entry, const char *type)
+{
+	GncDiscountHow how;
+
+	if (!entry) return;
+	gncEntryBeginEdit (entry);
+	gncEntryDiscountStringToHow(type, &how);
+	if (entry->i_disc_how == how) return;
+	entry->i_disc_how = how;
+	entry->values_dirty = TRUE;
+	mark_entry (entry);
+	gncEntryCommitEdit (entry);
+}
+
 /* Vendor Bills */
 
 void gncEntrySetBillAccount (GncEntry *entry, Account *acc)
@@ -507,8 +558,10 @@ void gncEntrySetOrder (GncEntry *entry, GncOrder *order)
   gncEntryCommitEdit (entry);
 
   /* Generate an event modifying the Order's end-owner */
-  gnc_engine_generate_event (gncOwnerGetEndGUID (gncOrderGetOwner (order)),
-			     GNC_EVENT_MODIFY);
+#if 0  
+  qof_event_gen (gncOwnerGetEndGUID (gncOrderGetOwner (order)),
+		 QOF_EVENT_MODIFY, NULL);
+#endif
 }
 
 /* called from gncInvoice when we're added to the Invoice */
@@ -533,12 +586,6 @@ void gncEntrySetBill (GncEntry *entry, GncInvoice *bill)
   gncEntryCommitEdit (entry);
 }
 
-void gncEntrySetDirty (GncEntry *entry, gboolean dirty)
-{
-  if (!entry) return;
-  entry->dirty = dirty;
-}
-
 void gncEntryCopy (const GncEntry *src, GncEntry *dest)
 {
   if (!src || !dest) return;
@@ -586,20 +633,9 @@ void gncEntryCopy (const GncEntry *src, GncEntry *dest)
   gncEntryCommitEdit (dest);
 }
 
+/* ================================================================ */
 /* Get Functions */
 
-GNCBook * gncEntryGetBook (GncEntry *entry)
-{
-  if (!entry) return NULL;
-  return entry->book;
-}
-
-const GUID * gncEntryGetGUID (GncEntry *entry)
-{
-  if (!entry) return NULL;
-  return &(entry->guid);
-}
-
 Timespec gncEntryGetDate (GncEntry *entry)
 {
   Timespec ts; ts.tv_sec = 0; ts.tv_nsec = 0;
@@ -670,6 +706,24 @@ GncDiscountHow gncEntryGetInvDiscountHow (GncEntry *entry)
   return entry->i_disc_how;
 }
 
+char* qofEntryGetInvDiscType (GncEntry *entry)
+{
+	char *type_string;
+
+	if (!entry) return 0;
+	type_string = g_strdup(gncAmountTypeToString(entry->i_disc_type));
+	return type_string;
+}
+
+char* qofEntryGetInvDiscHow (GncEntry *entry)
+{
+	char *type_string;
+
+	if (!entry) return 0;
+	type_string = g_strdup(gncEntryDiscountHowToString(entry->i_disc_how));
+	return type_string;
+}
+
 gboolean gncEntryGetInvTaxable (GncEntry *entry)
 {
   if (!entry) return FALSE;
@@ -756,13 +810,7 @@ GncOrder * gncEntryGetOrder (GncEntry *entry)
   return entry->order;
 }
 
-GncEntry * gncEntryLookup (GNCBook *book, const GUID *guid)
-{
-  if (!book || !guid) return NULL;
-  return xaccLookupEntity (gnc_book_get_entity_table (book),
-			   guid, _GNC_MOD_NAME);
-}
-
+/* ================================================================ */
 /*
  * This is the logic of computing the total for an Entry, so you know
  * what values to put into various Splits or to display in the ledger.
@@ -791,13 +839,15 @@ GncEntry * gncEntryLookup (GNCBook *book, const GUID *guid)
  * the amount the merchant gets; the taxes are the amount the gov't
  * gets, and the customer pays the sum or value + taxes.
  *
+ * The SCU is the denominator to convert the value.
+ *
  * The discount return value is just for entertainment -- you may way
  * to let a consumer know how much they saved.
  */
 void gncEntryComputeValue (gnc_numeric qty, gnc_numeric price,
 			   GncTaxTable *tax_table, gboolean tax_included,
 			   gnc_numeric discount, GncAmountType discount_type,
-			   GncDiscountHow discount_how,
+			   GncDiscountHow discount_how, int SCU,
 			   gnc_numeric *value, gnc_numeric *discount_value,
 			   GList **tax_value)
 {
@@ -927,11 +977,15 @@ void gncEntryComputeValue (gnc_numeric qty, gnc_numeric price,
    * need to compute taxes (based on 'pretax') if the caller wants it.
    */
 
-  if (discount_value != NULL)
+  if (discount_value != NULL) {
+    if (SCU) discount = gnc_numeric_convert(discount, SCU, GNC_RND_ROUND);
     *discount_value = discount;
+  }
 
-  if (value != NULL)
+  if (value != NULL) {
+    if (SCU) result = gnc_numeric_convert(result, SCU, GNC_RND_ROUND);
     *value = result;
+  }
 
   /* Now... Compute the list of tax values (if the caller wants it) */
 
@@ -947,12 +1001,14 @@ void gncEntryComputeValue (gnc_numeric qty, gnc_numeric price,
 
       switch (gncTaxTableEntryGetType (entry)) {
       case GNC_AMT_TYPE_VALUE:
+	if (SCU) amount = gnc_numeric_convert(amount, SCU, GNC_RND_ROUND);
 	taxes = gncAccountValueAdd (taxes, acc, amount);
 	break;
       case GNC_AMT_TYPE_PERCENT:
 	amount = gnc_numeric_div (amount, percent, GNC_DENOM_AUTO,
 				  GNC_DENOM_LCD);
 	tax = gnc_numeric_mul (pretax, amount, GNC_DENOM_AUTO, GNC_DENOM_LCD);
+	if (SCU) tax = gnc_numeric_convert(tax, SCU, GNC_RND_ROUND);
 	taxes = gncAccountValueAdd (taxes, acc, tax);
 	break;
       default:
@@ -966,7 +1022,7 @@ void gncEntryComputeValue (gnc_numeric qty, gnc_numeric price,
 }
 
 static int
-get_commodity_denom (GncEntry *entry)
+get_entry_commodity_denom (GncEntry *entry)
 {
   gnc_commodity *c;
   if (!entry)
@@ -1018,12 +1074,16 @@ gncEntryRecomputeValues (GncEntry *entry)
     entry->b_tax_values = NULL;
   }
 
+  /* Determine the commodity denominator */
+  denom = get_entry_commodity_denom (entry);
+
   /* Compute the invoice values */
   gncEntryComputeValue (entry->quantity, entry->i_price,
 			(entry->i_taxable ? entry->i_tax_table : NULL),
 			entry->i_taxincluded,
 			entry->i_discount, entry->i_disc_type,
 			entry->i_disc_how,
+			denom,
 			&(entry->i_value), &(entry->i_disc_value),
 			&(entry->i_tax_values));
 
@@ -1032,9 +1092,9 @@ gncEntryRecomputeValues (GncEntry *entry)
 			(entry->b_taxable ? entry->b_tax_table : NULL),
 			entry->b_taxincluded,
 			gnc_numeric_zero(), GNC_AMT_TYPE_VALUE, GNC_DISC_PRETAX,
+			denom,
 			&(entry->b_value), NULL, &(entry->b_tax_values));
 
-  denom = get_commodity_denom (entry);
   entry->i_value_rounded = gnc_numeric_convert (entry->i_value, denom,
 						GNC_RND_ROUND);
   entry->i_disc_value_rounded = gnc_numeric_convert (entry->i_disc_value, denom,
@@ -1095,32 +1155,38 @@ gnc_numeric gncEntryReturnDiscountValue (GncEntry *entry, gboolean is_inv)
   return (is_inv ? entry->i_disc_value_rounded : gnc_numeric_zero());
 }
 
+/* XXXX this exsitnace of this routine is just wrong */
 gboolean gncEntryIsOpen (GncEntry *entry)
 {
   if (!entry) return FALSE;
-  return (entry->editlevel > 0);
+  return (entry->inst.editlevel > 0);
 }
 
+/* ================================================================ */
+
 void gncEntryBeginEdit (GncEntry *entry)
 {
-  GNC_BEGIN_EDIT (entry, _GNC_MOD_NAME);
+  QOF_BEGIN_EDIT (&entry->inst);
 }
 
-static void gncEntryOnError (GncEntry *entry, GNCBackendError errcode)
+static inline void gncEntryOnError (QofInstance *entry, QofBackendError errcode)
 {
-  PERR("Entry Backend Failure: %d", errcode);
+  PERR("Entry QofBackend Failure: %d", errcode);
 }
 
-static void gncEntryOnDone (GncEntry *entry)
+static inline void gncEntryOnDone (QofInstance *inst) {}
+
+static inline void entry_free (QofInstance *inst)
 {
-  entry->dirty = FALSE;
+  GncEntry *entry = (GncEntry *)inst;
+  gncEntryFree (entry);
 }
 
 void gncEntryCommitEdit (GncEntry *entry)
 {
-  GNC_COMMIT_EDIT_PART1 (entry);
-  GNC_COMMIT_EDIT_PART2 (entry, _GNC_MOD_NAME, gncEntryOnError,
-			 gncEntryOnDone, gncEntryFree);
+  if (!qof_commit_edit (QOF_INSTANCE(entry))) return;
+  qof_commit_edit_part2 (&entry->inst, gncEntryOnError,
+			 gncEntryOnDone, entry_free);
 }
 
 int gncEntryCompare (GncEntry *a, GncEntry *b)
@@ -1143,81 +1209,63 @@ int gncEntryCompare (GncEntry *a, GncEntry *b)
   compare = safe_strcmp (a->action, b->action);
   if (compare) return compare;
 
-  return guid_compare (&(a->guid), &(b->guid));
+  return guid_compare (&(a->inst.entity.guid), &(b->inst.entity.guid));
 }
 
-/* Package-Private functions */
+/* ============================================================= */
+/* Object declaration */
 
-static void addObj (GncEntry *entry)
+static QofObject gncEntryDesc = 
 {
-  gncBusinessAddObject (entry->book, _GNC_MOD_NAME, entry, &entry->guid);
-}
-
-static void remObj (GncEntry *entry)
-{
-  gncBusinessRemoveObject (entry->book, _GNC_MOD_NAME, &entry->guid);
-}
-
-static void _gncEntryCreate (GNCBook *book)
-{
-  gncBusinessCreate (book, _GNC_MOD_NAME);
-}
-
-static void _gncEntryDestroy (GNCBook *book)
-{
-  gncBusinessDestroy (book, _GNC_MOD_NAME);
-}
-
-static gboolean _gncEntryIsDirty (GNCBook *book)
-{
-  return gncBusinessIsDirty (book, _GNC_MOD_NAME);
-}
-
-static void _gncEntryMarkClean (GNCBook *book)
-{
-  gncBusinessSetDirtyFlag (book, _GNC_MOD_NAME, FALSE);
-}
-
-static void _gncEntryForeach (GNCBook *book, foreachObjectCB cb,
-			      gpointer user_data)
-{
-  gncBusinessForeach (book, _GNC_MOD_NAME, cb, user_data);
-}
-
-static GncObject_t gncEntryDesc = {
-  GNC_OBJECT_VERSION,
-  _GNC_MOD_NAME,
-  "Order/Invoice/Bill Entry",
-  _gncEntryCreate,
-  _gncEntryDestroy,
-  _gncEntryIsDirty,
-  _gncEntryMarkClean,
-  _gncEntryForeach,
-  NULL				/* printable */
+  interface_version:  QOF_OBJECT_VERSION,
+  e_type:             _GNC_MOD_NAME,
+  type_label:         "Order/Invoice/Bill Entry",
+  create:             (gpointer)gncEntryCreate,
+  book_begin:         NULL,
+  book_end:           NULL,
+  is_dirty:           qof_collection_is_dirty,
+  mark_clean:         qof_collection_mark_clean,
+  foreach:            qof_collection_foreach,
+  printable:          NULL,
+  version_cmp:        (int (*)(gpointer, gpointer)) qof_instance_version_cmp,
 };
 
 gboolean gncEntryRegister (void)
 {
-  static QueryObjectDef params[] = {
-    { ENTRY_DATE, QUERYCORE_DATE, (QueryAccess)gncEntryGetDate },
-    { ENTRY_DATE_ENTERED, QUERYCORE_DATE, (QueryAccess)gncEntryGetDateEntered },
-    { ENTRY_DESC, QUERYCORE_STRING, (QueryAccess)gncEntryGetDescription },
-    { ENTRY_ACTION, QUERYCORE_STRING, (QueryAccess)gncEntryGetAction },
-    { ENTRY_NOTES, QUERYCORE_STRING, (QueryAccess)gncEntryGetNotes },
-    { ENTRY_QTY, QUERYCORE_NUMERIC, (QueryAccess)gncEntryGetQuantity },
-    { ENTRY_IPRICE, QUERYCORE_NUMERIC, (QueryAccess)gncEntryGetInvPrice },
-    { ENTRY_BPRICE, QUERYCORE_NUMERIC, (QueryAccess)gncEntryGetBillPrice },
-    { ENTRY_INVOICE, GNC_INVOICE_MODULE_NAME, (QueryAccess)gncEntryGetInvoice },
-    { ENTRY_BILL, GNC_INVOICE_MODULE_NAME, (QueryAccess)gncEntryGetBill },
-    { ENTRY_BILLABLE, QUERYCORE_BOOLEAN, (QueryAccess)gncEntryGetBillable },
-    { ENTRY_BILLTO, GNC_OWNER_MODULE_NAME, (QueryAccess)gncEntryGetBillTo },
-    { ENTRY_ORDER, GNC_ORDER_MODULE_NAME, (QueryAccess)gncEntryGetOrder },
-    { QUERY_PARAM_BOOK, GNC_ID_BOOK, (QueryAccess)gncEntryGetBook },
-    { QUERY_PARAM_GUID, QUERYCORE_GUID, (QueryAccess)gncEntryGetGUID },
+  static QofParam params[] = {
+    { ENTRY_DATE, QOF_TYPE_DATE, (QofAccessFunc)gncEntryGetDate, (QofSetterFunc)gncEntrySetDate },
+    { ENTRY_DATE_ENTERED, QOF_TYPE_DATE, (QofAccessFunc)gncEntryGetDateEntered, (QofSetterFunc)gncEntrySetDateEntered },
+    { ENTRY_DESC, QOF_TYPE_STRING, (QofAccessFunc)gncEntryGetDescription, (QofSetterFunc)gncEntrySetDescription },
+    { ENTRY_ACTION, QOF_TYPE_STRING, (QofAccessFunc)gncEntryGetAction, (QofSetterFunc)gncEntrySetAction },
+    { ENTRY_NOTES, QOF_TYPE_STRING, (QofAccessFunc)gncEntryGetNotes, (QofSetterFunc)gncEntrySetNotes },
+    { ENTRY_QTY, QOF_TYPE_NUMERIC, (QofAccessFunc)gncEntryGetQuantity, (QofSetterFunc)gncEntrySetQuantity },
+    { ENTRY_IPRICE, QOF_TYPE_NUMERIC, (QofAccessFunc)gncEntryGetInvPrice, (QofSetterFunc)gncEntrySetInvPrice },
+    { ENTRY_BPRICE, QOF_TYPE_NUMERIC, (QofAccessFunc)gncEntryGetBillPrice, (QofSetterFunc)gncEntrySetBillPrice },
+    { ENTRY_INVOICE, GNC_ID_INVOICE, (QofAccessFunc)gncEntryGetInvoice, NULL },
+    { ENTRY_IACCT, GNC_ID_ACCOUNT,  (QofAccessFunc)gncEntryGetInvAccount,  (QofSetterFunc)gncEntrySetInvAccount  },
+    { ENTRY_BACCT, GNC_ID_ACCOUNT,  (QofAccessFunc)gncEntryGetBillAccount, (QofSetterFunc)gncEntrySetBillAccount },
+    { ENTRY_BILL, GNC_ID_INVOICE, (QofAccessFunc)gncEntryGetBill, NULL },
+    { ENTRY_INV_DISC_TYPE, QOF_TYPE_STRING, (QofAccessFunc)qofEntryGetInvDiscType,
+       (QofSetterFunc)qofEntrySetInvDiscType },
+    { ENTRY_INV_DISC_HOW, QOF_TYPE_STRING, (QofAccessFunc)qofEntryGetInvDiscHow,
+       (QofSetterFunc)qofEntrySetInvDiscHow },
+    { ENTRY_INV_TAXABLE, QOF_TYPE_BOOLEAN, (QofAccessFunc)gncEntryGetInvTaxable,
+       (QofSetterFunc)gncEntrySetInvTaxable },
+    { ENTRY_INV_TAX_INC, QOF_TYPE_BOOLEAN, (QofAccessFunc)gncEntryGetInvTaxIncluded,
+       (QofSetterFunc)gncEntrySetInvTaxIncluded },
+    { ENTRY_BILL_TAXABLE, QOF_TYPE_BOOLEAN, (QofAccessFunc)gncEntryGetInvTaxable,
+       (QofSetterFunc)gncEntrySetInvTaxable },
+    { ENTRY_BILL_TAX_INC, QOF_TYPE_BOOLEAN, (QofAccessFunc)gncEntryGetBillTaxIncluded,
+       (QofSetterFunc)gncEntrySetBillTaxIncluded },
+    { ENTRY_BILLABLE, QOF_TYPE_BOOLEAN, (QofAccessFunc)gncEntryGetBillable, (QofSetterFunc)gncEntrySetBillable },
+    { ENTRY_BILLTO, GNC_ID_OWNER, (QofAccessFunc)gncEntryGetBillTo, (QofSetterFunc)gncEntrySetBillTo },
+    { ENTRY_ORDER, GNC_ID_ORDER, (QofAccessFunc)gncEntryGetOrder, NULL },
+    { QOF_PARAM_BOOK, QOF_ID_BOOK, (QofAccessFunc)qof_instance_get_book, NULL },
+    { QOF_PARAM_GUID, QOF_TYPE_GUID, (QofAccessFunc)qof_instance_get_guid, NULL },
     { NULL },
   };
 
-  gncQueryObjectRegister (_GNC_MOD_NAME, (QuerySort)gncEntryCompare, params);
+  qof_class_register (_GNC_MOD_NAME, (QofSortFunc)gncEntryCompare, params);
 
-  return gncObjectRegister (&gncEntryDesc);
+  return qof_object_register (&gncEntryDesc);
 }
