@@ -1,6 +1,28 @@
+/********************************************************************\
+ * gncTaxTable.c -- the Gnucash Tax Table interface                 *
+ *                                                                  *
+ * This program is free software; you can redistribute it and/or    *
+ * modify it under the terms of the GNU General Public License as   *
+ * published by the Free Software Foundation; either version 2 of   *
+ * the License, or (at your option) any later version.              *
+ *                                                                  *
+ * This program is distributed in the hope that it will be useful,  *
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of   *
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the    *
+ * GNU General Public License for more details.                     *
+ *                                                                  *
+ * You should have received a copy of the GNU General Public License*
+ * along with this program; if not, contact:                        *
+ *                                                                  *
+ * Free Software Foundation           Voice:  +1-617-542-5942       *
+ * 51 Franklin Street, Fifth Floor    Fax:    +1-617-542-2652       *
+ * Boston, MA  02110-1301,  USA       gnu@gnu.org                   *
+ *                                                                  *
+\********************************************************************/
+
 /*
- * gncTaxTable.c -- the Gnucash Tax Table interface
  * Copyright (C) 2002 Derek Atkins
+ * Copyright (C) 2003 Linas Vepstas <linas@linas.org>
  * Author: Derek Atkins <warlord@MIT.EDU>
  */
 
@@ -8,52 +30,42 @@
 
 #include <glib.h>
 
-#include "messages.h"
-#include "gnc-numeric.h"
-#include "gnc-engine-util.h"
-#include "gnc-book-p.h"
-#include "GNCIdP.h"
-#include "QueryObject.h"
-#include "gnc-event-p.h"
-#include "gnc-be-utils.h"
-
-#include "gncBusiness.h"
 #include "gncTaxTableP.h"
 
+struct _gncTaxTable 
+{
+  QofInstance     inst;
+  char *          name;
+  GList *         entries;
+  Timespec        modtime;      /* internal date of last modtime */
 
-struct _gncTaxTable {
-  GUID		guid;
-  char *	name;
-  GList *	entries;
-
-  Timespec	modtime;	/* internal date of last modtime */
-  gint64	refcount;
-  GNCBook *	book;
-  GncTaxTable *	parent;		/* if non-null, we are an immutable child */
-  GncTaxTable *	child;		/* if non-null, we have not changed */
-  gboolean	invisible;
-
-  GList *	children;	/* A list of children */
-
-  int		editlevel;
-  gboolean	do_free;
-  gboolean	dirty;
+  /* See src/doc/business.txt for an explanation of the following */
+  /* Code that handles this is *identical* to that in gncBillTerm */
+  gint64          refcount;
+  GncTaxTable *   parent;       /* if non-null, we are an immutable child */
+  GncTaxTable *   child;        /* if non-null, we have not changed */
+  gboolean        invisible;
+  GList *         children;     /* list of children for disconnection */
 };
 
-struct _gncTaxTableEntry {
-  GncTaxTable *	table;
-  Account *	account;
-  GncAmountType	type;
-  gnc_numeric	amount;
+struct _gncTaxTableEntry 
+{
+  GncTaxTable *   table;
+  Account *       account;
+  GncAmountType   type;
+  gnc_numeric     amount;
 };
 
-struct _book_info {
-  GncBookInfo	bi;
-  GList *	tables;		/* visible tables */
+struct _book_info 
+{
+  GList *         tables;          /* visible tables */
 };
 
-static short	module = MOD_BUSINESS;
+static GncTaxTableEntry * CloneTaxEntry (GncTaxTableEntry*, QofBook *);
 
+static QofLogModule log_module = GNC_MOD_BUSINESS;
+
+/* =============================================================== */
 /* You must edit the functions in this block in tandem.  KEEP THEM IN
    SYNC! */
 
@@ -113,76 +125,182 @@ gncTaxIncludedStringToType (const char *str, GncTaxIncluded *type)
 }
 #undef GNC_RETURN_ON_MATCH
 
-#define _GNC_MOD_NAME	GNC_TAXTABLE_MODULE_NAME
+/* =============================================================== */
+/* Misc inline functions */
 
-#define CACHE_INSERT(str) g_cache_insert(gnc_engine_get_string_cache(), (gpointer)(str));
-#define CACHE_REMOVE(str) g_cache_remove(gnc_engine_get_string_cache(), (str));
+#define _GNC_MOD_NAME        GNC_ID_TAXTABLE
 
 #define SET_STR(obj, member, str) { \
-	char * tmp; \
-	\
-	if (!safe_strcmp (member, str)) return; \
-	gncTaxTableBeginEdit (obj); \
-	tmp = CACHE_INSERT (str); \
-	CACHE_REMOVE (member); \
-	member = tmp; \
-	}
+        char * tmp; \
+        \
+        if (!safe_strcmp (member, str)) return; \
+        gncTaxTableBeginEdit (obj); \
+        tmp = CACHE_INSERT (str); \
+        CACHE_REMOVE (member); \
+        member = tmp; \
+        }
 
-static void add_or_rem_object (GncTaxTable *table, gboolean add);
-static void addObj (GncTaxTable *table);
-static void remObj (GncTaxTable *table);
-static void maybe_resort_list (GncTaxTable *table);
-
-static void gncTaxTableRemoveChild (GncTaxTable *table, GncTaxTable *child);
-
-G_INLINE_FUNC void mark_table (GncTaxTable *table);
-G_INLINE_FUNC void
+static inline void
 mark_table (GncTaxTable *table)
 {
-  table->dirty = TRUE;
-  gncBusinessSetDirtyFlag (table->book, _GNC_MOD_NAME, TRUE);
-
-  gnc_engine_generate_event (&table->guid, GNC_EVENT_MODIFY);
+  qof_instance_set_dirty(&table->inst);
+  qof_event_gen (&table->inst.entity, QOF_EVENT_MODIFY, NULL);
 }
 
-G_INLINE_FUNC void mod_table (GncTaxTable *table);
-G_INLINE_FUNC void
+static inline void 
+maybe_resort_list (GncTaxTable *table)
+{
+  struct _book_info *bi;
+
+  if (table->parent || table->invisible) return;
+  bi = qof_book_get_data (table->inst.book, _GNC_MOD_NAME);
+  bi->tables = g_list_sort (bi->tables, (GCompareFunc)gncTaxTableCompare);
+}
+
+static inline void
 mod_table (GncTaxTable *table)
 {
   timespecFromTime_t (&table->modtime, time(NULL));
 }
 
+static inline void addObj (GncTaxTable *table)
+{
+  struct _book_info *bi;
+  bi = qof_book_get_data (table->inst.book, _GNC_MOD_NAME);
+  bi->tables = g_list_insert_sorted (bi->tables, table,
+                                       (GCompareFunc)gncTaxTableCompare);
+}
+
+static inline void remObj (GncTaxTable *table)
+{
+  struct _book_info *bi;
+  bi = qof_book_get_data (table->inst.book, _GNC_MOD_NAME);
+  bi->tables = g_list_remove (bi->tables, table);
+}
+
+static inline void
+gncTaxTableAddChild (GncTaxTable *table, GncTaxTable *child)
+{
+  g_return_if_fail(table);
+  g_return_if_fail(child);
+  g_return_if_fail(table->inst.do_free == FALSE);
+
+  table->children = g_list_prepend(table->children, child);
+}
+
+static inline void
+gncTaxTableRemoveChild (GncTaxTable *table, GncTaxTable *child)
+{
+  g_return_if_fail(table);
+  g_return_if_fail(child);
+
+  if (table->inst.do_free) return;
+
+  table->children = g_list_remove(table->children, child);
+}
+
+/* =============================================================== */
 /* Create/Destroy Functions */
-GncTaxTable * gncTaxTableCreate (GNCBook *book)
+
+GncTaxTable * 
+gncTaxTableCreate (QofBook *book)
 {
   GncTaxTable *table;
   if (!book) return NULL;
 
   table = g_new0 (GncTaxTable, 1);
-  table->book = book;
+  qof_instance_init (&table->inst, _GNC_MOD_NAME, book);
   table->name = CACHE_INSERT ("");
-  xaccGUIDNew (&table->guid, book);
   addObj (table);
-  gnc_engine_generate_event (&table->guid, GNC_EVENT_CREATE);
+  qof_event_gen (&table->inst.entity, QOF_EVENT_CREATE, NULL);
   return table;
 }
 
-void gncTaxTableDestroy (GncTaxTable *table)
+
+GncTaxTable * 
+gncCloneTaxTable (GncTaxTable *from, QofBook *book)
+{
+  GList *node;
+  GncTaxTable *table;
+  if (!book) return NULL;
+
+  table = g_new0 (GncTaxTable, 1);
+  qof_instance_init (&table->inst, _GNC_MOD_NAME, book);
+  qof_instance_gemini (&table->inst, &from->inst);
+  
+  table->name = CACHE_INSERT (from->name);
+  table->modtime = from->modtime;
+  table->invisible = from->invisible;
+
+  table->refcount = 0;
+
+  /* Make copies of parents and children. Note that this can be
+   * a recursive copy ... treat as doubly-linked list. */
+  if (from->child)
+  {
+    table->child = gncTaxTableObtainTwin (from->child, book);
+    table->child->parent = table;
+  }
+  if (from->parent)
+  {
+    table->parent = gncTaxTableObtainTwin (from->parent, book);
+    table->parent->child = table;
+  }
+  for (node=g_list_last(from->children); node; node=node->next)
+  {
+    GncTaxTable *tbl = node->data;
+    tbl = gncTaxTableObtainTwin (tbl, book);
+    tbl->parent = table;
+    table->children = g_list_prepend(table->children, tbl);
+  }
+
+  /* Copy tax entries, preserving the order in the list */
+  table->entries = NULL;
+  for (node=g_list_last(from->entries); node; node=node->prev)
+  {
+    GncTaxTableEntry *ent = node->data;
+    ent = CloneTaxEntry (ent, book);
+    table->entries = g_list_prepend (table->entries, ent);
+  }
+
+  addObj (table);
+  qof_event_gen (&table->inst.entity, QOF_EVENT_CREATE, NULL);
+  return table;
+}
+
+GncTaxTable *
+gncTaxTableObtainTwin (GncTaxTable *from, QofBook *book)
+{
+  GncTaxTable *table;
+  if (!from) return NULL;
+
+  table = (GncTaxTable *) qof_instance_lookup_twin (QOF_INSTANCE(from), book);
+  if (!table)
+  {
+    table = gncCloneTaxTable (table, book);
+  }
+  return table;
+}
+
+
+void 
+gncTaxTableDestroy (GncTaxTable *table)
 {
   if (!table) return;
-  table->do_free = TRUE;
-  gncBusinessSetDirtyFlag (table->book, _GNC_MOD_NAME, TRUE);
+  table->inst.do_free = TRUE;
+  qof_instance_set_dirty (&table->inst);
   gncTaxTableCommitEdit (table);
 }
 
-static void gncTaxTableFree (GncTaxTable *table)
+static void 
+gncTaxTableFree (GncTaxTable *table)
 {
   GList *list;
   GncTaxTable *child;
 
   if (!table) return;
 
-  gnc_engine_generate_event (&table->guid, GNC_EVENT_DESTROY);
+  qof_event_gen (&table->inst.entity, QOF_EVENT_DESTROY, NULL);
   CACHE_REMOVE (table->name);
   remObj (table);
 
@@ -191,7 +309,7 @@ static void gncTaxTableFree (GncTaxTable *table)
     gncTaxTableEntryDestroy (list->data);
   g_list_free (table->entries);
 
-  if (!table->do_free)
+  if (!table->inst.do_free)
     PERR("free a taxtable without do_free set!");
 
   /* disconnect from parent */
@@ -205,30 +323,11 @@ static void gncTaxTableFree (GncTaxTable *table)
   }
   g_list_free(table->children);
 
+  qof_instance_release (&table->inst);
   g_free (table);
 }
 
-static void
-gncTaxTableAddChild (GncTaxTable *table, GncTaxTable *child)
-{
-  g_return_if_fail(table);
-  g_return_if_fail(child);
-  g_return_if_fail(table->do_free == FALSE);
-
-  table->children = g_list_prepend(table->children, child);
-}
-
-static void
-gncTaxTableRemoveChild (GncTaxTable *table, GncTaxTable *child)
-{
-  g_return_if_fail(table);
-  g_return_if_fail(child);
-
-  if (table->do_free)
-    return;
-
-  table->children = g_list_remove(table->children, child);
-}
+/* =============================================================== */
 
 GncTaxTableEntry * gncTaxTableEntryCreate (void)
 {
@@ -244,20 +343,28 @@ void gncTaxTableEntryDestroy (GncTaxTableEntry *entry)
   g_free (entry);
 }
 
-
-/* Set Functions */
-void gncTaxTableSetGUID (GncTaxTable *table, const GUID *guid)
+/** Makes a clone. The account is from the appriate book. 
+ *  Note that the table is left blank (for performance reasons
+ *  we set it above, when cloning the table).
+ */
+static GncTaxTableEntry * 
+CloneTaxEntry (GncTaxTableEntry*from, QofBook *book)
 {
-  if (!table || !guid) return;
-  if (guid_equal (guid, &table->guid)) return;
+  QofInstance *acc;
+  GncTaxTableEntry *entry;
+  entry = g_new0 (GncTaxTableEntry, 1);
 
-  gncTaxTableBeginEdit (table);
-  remObj (table);
-  table->guid = *guid;
-  addObj (table);
-  gncTaxTableCommitEdit (table);
+  entry->type = from->type;
+  entry->amount = from->amount;
+
+  acc = qof_instance_lookup_twin (QOF_INSTANCE(from->account), book);
+  entry->account = (Account *) acc;
+  return entry;
 }
 
+/* =============================================================== */
+/* Set Functions */
+
 void gncTaxTableSetName (GncTaxTable *table, const char *name)
 {
   if (!table || !name) return;
@@ -292,7 +399,7 @@ void gncTaxTableSetChild (GncTaxTable *table, GncTaxTable *child)
 void gncTaxTableIncRef (GncTaxTable *table)
 {
   if (!table) return;
-  if (table->parent || table->invisible) return;	/* children dont need refcounts */
+  if (table->parent || table->invisible) return;        /* children dont need refcounts */
   gncTaxTableBeginEdit (table);
   table->refcount++;
   gncTaxTableCommitEdit (table);
@@ -301,7 +408,7 @@ void gncTaxTableIncRef (GncTaxTable *table)
 void gncTaxTableDecRef (GncTaxTable *table)
 {
   if (!table) return;
-  if (table->parent || table->invisible) return;	/* children dont need refcounts */
+  if (table->parent || table->invisible) return;        /* children dont need refcounts */
   gncTaxTableBeginEdit (table);
   table->refcount--;
   g_return_if_fail (table->refcount >= 0);
@@ -316,10 +423,12 @@ void gncTaxTableSetRefcount (GncTaxTable *table, gint64 refcount)
 
 void gncTaxTableMakeInvisible (GncTaxTable *table)
 {
+  struct _book_info *bi;
   if (!table) return;
   gncTaxTableBeginEdit (table);
   table->invisible = TRUE;
-  add_or_rem_object (table, FALSE);
+  bi = qof_book_get_data (table->inst.book, _GNC_MOD_NAME);
+  bi->tables = g_list_remove (bi->tables, table);
   gncTaxTableCommitEdit (table);
 }
 
@@ -367,7 +476,7 @@ void gncTaxTableAddEntry (GncTaxTable *table, GncTaxTableEntry *entry)
 
   entry->table = table;
   table->entries = g_list_insert_sorted (table->entries, entry,
-					 (GCompareFunc)gncTaxTableEntryCompare);
+                                         (GCompareFunc)gncTaxTableEntryCompare);
   mark_table (table);
   mod_table (table);
   gncTaxTableCommitEdit (table);
@@ -392,38 +501,38 @@ void gncTaxTableChanged (GncTaxTable *table)
   gncTaxTableCommitEdit (table);
 }
 
+/* =============================================================== */
+
 void gncTaxTableBeginEdit (GncTaxTable *table)
 {
-  GNC_BEGIN_EDIT (table, _GNC_MOD_NAME);
+  QOF_BEGIN_EDIT (&table->inst);
 }
 
-static void gncTaxTableOnError (GncTaxTable *table, GNCBackendError errcode)
+static inline void gncTaxTableOnError (QofInstance *inst, QofBackendError errcode)
 {
-  PERR("TaxTable Backend Failure: %d", errcode);
+  PERR("TaxTable QofBackend Failure: %d", errcode);
 }
 
-static void gncTaxTableOnDone (GncTaxTable *table)
+static inline void gncTaxTableOnDone (QofInstance *inst) {}
+
+static inline void table_free (QofInstance *inst)
 {
-  table->dirty = FALSE;
+  GncTaxTable *table = (GncTaxTable *) inst;
+  gncTaxTableFree (table);
 }
 
 void gncTaxTableCommitEdit (GncTaxTable *table)
 {
-  GNC_COMMIT_EDIT_PART1 (table);
-  GNC_COMMIT_EDIT_PART2 (table, _GNC_MOD_NAME, gncTaxTableOnError,
-			 gncTaxTableOnDone, gncTaxTableFree);
+  if (!qof_commit_edit (QOF_INSTANCE(table))) return;
+  qof_commit_edit_part2 (&table->inst, gncTaxTableOnError,
+                         gncTaxTableOnDone, table_free);
 }
 
 
+/* =============================================================== */
 /* Get Functions */
-GncTaxTable * gncTaxTableLookup (GNCBook *book, const GUID *guid)
-{
-  if (!book || !guid) return NULL;
-  return xaccLookupEntity (gnc_book_get_entity_table (book),
-			   guid, _GNC_MOD_NAME);
-}
 
-GncTaxTable *gncTaxTableLookupByName (GNCBook *book, const char *name)
+GncTaxTable *gncTaxTableLookupByName (QofBook *book, const char *name)
 {
   GList *list = gncTaxTableGetTables (book);
 
@@ -435,28 +544,15 @@ GncTaxTable *gncTaxTableLookupByName (GNCBook *book, const char *name)
   return NULL;
 }
 
-GList * gncTaxTableGetTables (GNCBook *book)
+GList * gncTaxTableGetTables (QofBook *book)
 {
   struct _book_info *bi;
   if (!book) return NULL;
 
-  bi = gnc_book_get_data (book, _GNC_MOD_NAME);
+  bi = qof_book_get_data (book, _GNC_MOD_NAME);
   return bi->tables;
 }
 
-
-const GUID *gncTaxTableGetGUID (GncTaxTable *table)
-{
-  if (!table) return NULL;
-  return &table->guid;
-}
-
-GNCBook *gncTaxTableGetBook (GncTaxTable *table)
-{
-  if (!table) return NULL;
-  return table->book;
-}
-
 const char *gncTaxTableGetName (GncTaxTable *table)
 {
   if (!table) return NULL;
@@ -482,7 +578,7 @@ static GncTaxTable *gncTaxTableCopy (GncTaxTable *table)
   GList *list;
 
   if (!table) return NULL;
-  t = gncTaxTableCreate (table->book);
+  t = gncTaxTableCreate (table->inst.book);
   gncTaxTableSetName (t, table->name);
   for (list = table->entries; list; list=list->next) {
     GncTaxTableEntry *entry, *e;
@@ -566,9 +662,8 @@ int gncTaxTableEntryCompare (GncTaxTableEntry *a, GncTaxTableEntry *b)
   if (!a) return -1;
   if (!b) return 1;
     
-  name_a = xaccAccountGetFullName (a->account, ':');
-  name_b = xaccAccountGetFullName (b->account, ':');
-  /* for comparison purposes it doesn't matter what we use as a separator */
+  name_a = xaccAccountGetFullName (a->account);
+  name_b = xaccAccountGetFullName (b->account);
   retval = safe_strcmp(name_a, name_b);
   g_free(name_a);
   g_free(name_b);
@@ -605,7 +700,7 @@ GList *gncAccountValueAdd (GList *list, Account *acc, gnc_numeric value)
     res = li->data;
     if (res->account == acc) {
       res->value = gnc_numeric_add (res->value, value, GNC_DENOM_AUTO,
-				    GNC_DENOM_LCD);
+                                    GNC_DENOM_LCD);
       return list;
     }
   }
@@ -652,119 +747,60 @@ void gncAccountValueDestroy (GList *list)
   g_list_free (list);
 }
 
-GUID gncTaxTableRetGUID (GncTaxTable *table)
-{
-  if (!table)
-    return *xaccGUIDNULL();
-
-  return table->guid;
-}
-
-GncTaxTable *gncTaxTableLookupDirect (GUID guid, GNCBook *book)
-{
-  if (!book) return NULL;
-  return gncTaxTableLookup (book, &guid);
-}
-
 /* Package-Private functions */
 
-static void maybe_resort_list (GncTaxTable *table)
-{
-  struct _book_info *bi;
-
-  if (table->parent || table->invisible) return;
-  bi = gnc_book_get_data (table->book, _GNC_MOD_NAME);
-  bi->tables = g_list_sort (bi->tables, (GCompareFunc)gncTaxTableCompare);
-}
-
-static void add_or_rem_object (GncTaxTable *table, gboolean add)
-{
-  struct _book_info *bi;
-
-  if (!table) return;
-  bi = gnc_book_get_data (table->book, _GNC_MOD_NAME);
-
-  if (add)
-    bi->tables = g_list_insert_sorted (bi->tables, table,
-				       (GCompareFunc)gncTaxTableCompare);
-  else
-    bi->tables = g_list_remove (bi->tables, table);
-}
-
-static void addObj (GncTaxTable *table)
-{
-  gncBusinessAddObject (table->book, _GNC_MOD_NAME, table, &table->guid);
-  add_or_rem_object (table, TRUE);
-}
-
-static void remObj (GncTaxTable *table)
-{
-  gncBusinessRemoveObject (table->book, _GNC_MOD_NAME, &table->guid);
-  add_or_rem_object (table, FALSE);
-}
-
-static void _gncTaxTableCreate (GNCBook *book)
+static void _gncTaxTableCreate (QofBook *book)
 {
   struct _book_info *bi;
 
   if (!book) return;
 
   bi = g_new0 (struct _book_info, 1);
-  bi->bi.ht = guid_hash_table_new ();
-  gnc_book_set_data (book, _GNC_MOD_NAME, bi);
+  qof_book_set_data (book, _GNC_MOD_NAME, bi);
 }
 
-static void _gncTaxTableDestroy (GNCBook *book)
+static void _gncTaxTableDestroy (QofBook *book)
 {
   struct _book_info *bi;
 
   if (!book) return;
 
-  bi = gnc_book_get_data (book, _GNC_MOD_NAME);
+  bi = qof_book_get_data (book, _GNC_MOD_NAME);
 
-  /* XXX : Destroy the objects? */
-  g_hash_table_destroy (bi->bi.ht);
   g_list_free (bi->tables);
   g_free (bi);
 }
 
-static gboolean _gncTaxTableIsDirty (GNCBook *book)
+static QofObject gncTaxTableDesc = 
 {
-  return gncBusinessIsDirty (book, _GNC_MOD_NAME);
-}
-
-static void _gncTaxTableMarkClean (GNCBook *book)
-{
-  gncBusinessSetDirtyFlag (book, _GNC_MOD_NAME, FALSE);
-}
-
-static void _gncTaxTableForeach (GNCBook *book, foreachObjectCB cb,
-			      gpointer user_data)
-{
-  gncBusinessForeach (book, _GNC_MOD_NAME, cb, user_data);
-}
-
-static GncObject_t gncTaxTableDesc = {
-  GNC_OBJECT_VERSION,
-  _GNC_MOD_NAME,
-  "Tax Table",
-  _gncTaxTableCreate,
-  _gncTaxTableDestroy,
-  _gncTaxTableIsDirty,
-  _gncTaxTableMarkClean,
-  _gncTaxTableForeach,
-  NULL				/* printable */
+  interface_version:  QOF_OBJECT_VERSION,
+  e_type:             _GNC_MOD_NAME,
+  type_label:         "Tax Table",
+  create:             (gpointer)gncTaxTableCreate,
+  book_begin:         _gncTaxTableCreate,
+  book_end:           _gncTaxTableDestroy,
+  is_dirty:           qof_collection_is_dirty,
+  mark_clean:         qof_collection_mark_clean,
+  foreach:            qof_collection_foreach,
+  printable:          NULL,
+  version_cmp:        (int (*)(gpointer, gpointer)) qof_instance_version_cmp,
 };
 
 gboolean gncTaxTableRegister (void)
 {
-  static QueryObjectDef params[] = {
-    { QUERY_PARAM_BOOK, GNC_ID_BOOK, (QueryAccess)gncTaxTableGetBook },
-    { QUERY_PARAM_GUID, QUERYCORE_GUID, (QueryAccess)gncTaxTableGetGUID },
+  static QofParam params[] = {
+	{ GNC_TT_NAME, 		QOF_TYPE_STRING, 	(QofAccessFunc)gncTaxTableGetName, 		(QofSetterFunc)gncTaxTableSetName },
+	{ GNC_TT_REFCOUNT, 	QOF_TYPE_INT64, 	(QofAccessFunc)gncTaxTableGetRefcount, 	(QofSetterFunc)gncTaxTableSetRefcount },
+    { QOF_PARAM_BOOK, 	QOF_ID_BOOK, 		(QofAccessFunc)qof_instance_get_book, 	NULL },
+    { QOF_PARAM_GUID, 	QOF_TYPE_GUID, 		(QofAccessFunc)qof_instance_get_guid, 	NULL },
     { NULL },
   };
 
-  gncQueryObjectRegister (_GNC_MOD_NAME, (QuerySort)gncTaxTableCompare, params);
+  qof_class_register (_GNC_MOD_NAME, (QofSortFunc)gncTaxTableCompare, params);
 
-  return gncObjectRegister (&gncTaxTableDesc);
+  return qof_object_register (&gncTaxTableDesc);
 }
+
+/* need a QOF tax table entry object */
+//gncTaxTableEntrySetType_q int32
+//gint gncTaxTableEntryGetType_q (GncTaxTableEntry *entry);
