@@ -2,7 +2,25 @@
 ;; invoice.scm -- an Invoice Report, used to print a GncInvoice
 ;;
 ;; Created by:  Derek Atkins <warlord@MIT.EDU>
+;; Copyright (c) 2002, 2003 Derek Atkins <warlord@MIT.EDU>
 ;;
+;; This program is free software; you can redistribute it and/or    
+;; modify it under the terms of the GNU General Public License as   
+;; published by the Free Software Foundation; either version 2 of   
+;; the License, or (at your option) any later version.              
+;;                                                                  
+;; This program is distributed in the hope that it will be useful,  
+;; but WITHOUT ANY WARRANTY; without even the implied warranty of   
+;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the    
+;; GNU General Public License for more details.                     
+;;                                                                  
+;; You should have received a copy of the GNU General Public License
+;; along with this program; if not, contact:
+;;
+;; Free Software Foundation           Voice:  +1-617-542-5942
+;; 51 Franklin Street, Fifth Floor    Fax:    +1-617-542-2652
+;; Boston, MA  02110-1301,  USA       gnu@gnu.org
+
 
 (define-module (gnucash report invoice))
 
@@ -244,7 +262,7 @@
   (gnc:register-inv-option
    (gnc:make-simple-boolean-option
     (N_ "Display Columns") (N_ "Price")
-    "hb" "Display the price per item?" #t))
+    "hb" (N_ "Display the price per item?") #t))
 
   (gnc:register-inv-option
    (gnc:make-simple-boolean-option
@@ -305,7 +323,7 @@
    (gnc:make-text-option
     (N_ "Display") (N_ "Extra Notes")
      "u" (N_ "Extra notes to put on the invoice")
-     "Thank you for your patronage"))
+     (_ "Thank you for your patronage")))
 
   (gnc:register-inv-option
    (gnc:make-string-option
@@ -531,7 +549,7 @@
     (gnc:html-table-append-row!
      table
      (list
-      (string-expand (gnc:owner-get-address-dep owner) #\newline "<br>")))
+      (string-expand (gnc:owner-get-name-and-address-dep owner) #\newline "<br>")))
     (gnc:html-table-append-row!
      table
      (list "<br>"))
@@ -631,8 +649,8 @@
 	       (set! title (_ "Bill")))
 	      ((gnc-owner-employee)
 	       (set! title (_ "Expense Voucher")))))
-	  (set! title (string-append title " #"
-				     (gnc:invoice-get-id invoice)))))
+	  (set! title (sprintf #f (_"%s #%d") title
+			       (gnc:invoice-get-id invoice)))))
 
     (gnc:html-document-set-title! document title)
 
@@ -727,7 +745,7 @@
 	(gnc:html-document-add-object!
 	 document
 	 (gnc:make-html-text
-	  (N_ "No Valid Invoice Selected"))))
+	  (_ "No Valid Invoice Selected"))))
 
     document))
 
