@@ -2,27 +2,59 @@
  * dialog-tax-table.c -- Dialog to create and edit tax-tables
  * Copyright (C) 2002 Derek Atkins
  * Author: Derek Atkins <warlord@MIT.EDU>
+ *
+ * Copyright (c) 2006 David Hampton <hampton@employees.org>
+ *
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License as
+ * published by the Free Software Foundation; either version 2 of
+ * the License, or (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, contact:
+ *
+ * Free Software Foundation           Voice:  +1-617-542-5942
+ * 51 Franklin Street, Fifth Floor    Fax:    +1-617-542-2652
+ * Boston, MA  02110-1301,  USA       gnu@gnu.org
  */
 
 #include "config.h"
 
-#include <gnome.h>
+#include <gtk/gtk.h>
+#include <glib/gi18n.h>
 
 #include "dialog-utils.h"
-#include "global-options.h"
 #include "gnc-component-manager.h"
 #include "gnc-ui.h"
 #include "gnc-gui-query.h"
 #include "gnc-ui-util.h"
-#include "gnc-engine-util.h"
+#include "qof.h"
 #include "gnc-amount-edit.h"
-#include "gnc-account-tree.h"
-#include "gnc-numeric.h"
+#include "gnc-tree-view-account.h"
 
 #include "gncTaxTable.h"
 #include "dialog-tax-table.h"
 
 #define DIALOG_TAX_TABLE_CM_CLASS "tax-table-dialog"
+#define GCONF_SECTION "dialogs/business/tax_tables"
+
+void tax_table_new_table_cb (GtkButton *button, TaxTableWindow *ttw);
+void tax_table_delete_table_cb (GtkButton *button, TaxTableWindow *ttw);
+void tax_table_new_entry_cb (GtkButton *button, TaxTableWindow *ttw);
+void tax_table_edit_entry_cb (GtkButton *button, TaxTableWindow *ttw);
+void tax_table_delete_entry_cb (GtkButton *button, TaxTableWindow *ttw);
+void tax_table_row_selected (GtkCList *clist, gint row, gint column,
+			     GdkEventButton *event, gpointer user_data);
+void tax_table_entry_row_selected (GtkCList *clist, gint row, gint column,
+				   GdkEventButton *event, gpointer user_data);
+void tax_table_window_close (GtkWidget *widget, gpointer data);
+void tax_table_window_destroy_cb (GtkWidget *widget, gpointer data);
+
 
 struct _taxtable_window {
   GtkWidget *	dialog;
@@ -49,17 +81,16 @@ typedef struct _new_taxtable {
 } NewTaxTable;
 
 
-static void
-new_tax_table_ok_cb (GtkWidget *widget, gpointer data)
+static gboolean
+new_tax_table_ok_cb (NewTaxTable *ntt)
 {
-  NewTaxTable *ntt = data;
   TaxTableWindow *ttw;
-  char *name = NULL;
+  const char *name = NULL;
   char *message;
   Account *acc;
   gnc_numeric amount;
 
-  g_return_if_fail (ntt);
+  g_return_val_if_fail (ntt, FALSE);
   ttw = ntt->ttw;
 
   /* Verify that we've got real, valid data */
@@ -69,16 +100,16 @@ new_tax_table_ok_cb (GtkWidget *widget, gpointer data)
     name = gtk_entry_get_text (GTK_ENTRY (ntt->name_entry));
     if (name == NULL || *name == '\0') {
       message = _("You must provide a name for this Tax Table.");
-      gnc_error_dialog_parented (GTK_WINDOW (ntt->dialog), message);
-      return;
+      gnc_error_dialog (ntt->dialog, message);
+      return FALSE;
     }
     if (gncTaxTableLookupByName (ttw->book, name)) {
       message = g_strdup_printf(_(
-			 "You must provide a unique name for this Tax Table.\n"
+			 "You must provide a unique name for this Tax Table. "
 			 "Your choice \"%s\" is already in use."), name);
-      gnc_error_dialog_parented (GTK_WINDOW (ntt->dialog), "%s", message);
+      gnc_error_dialog (ntt->dialog, "%s", message);
       g_free (message);
-      return;
+      return FALSE;
     }
   }
 
@@ -86,24 +117,23 @@ new_tax_table_ok_cb (GtkWidget *widget, gpointer data)
   amount = gnc_amount_edit_get_amount (GNC_AMOUNT_EDIT (ntt->amount_entry));
   if (gnc_numeric_negative_p (amount)) {
     message = _("Negative amounts are not allowed.");
-    gnc_error_dialog_parented (GTK_WINDOW (ntt->dialog), message);
-    return;
+    gnc_error_dialog (ntt->dialog, message);
+    return FALSE;
   }
   if (ntt->type == GNC_AMT_TYPE_PERCENT &&
       gnc_numeric_compare (amount,
 			   gnc_numeric_create (100, 1)) > 0) {
     message = _("Percentage amount must be between 0 and 100.");
-    gnc_error_dialog_parented (GTK_WINDOW (ntt->dialog), message);
-    return;
+    gnc_error_dialog (ntt->dialog, message);
+    return FALSE;
   }							   
 
   /* verify the account */
-  acc =
-    gnc_account_tree_get_current_account (GNC_ACCOUNT_TREE (ntt->acct_tree));
+  acc = gnc_tree_view_account_get_selected_account (GNC_TREE_VIEW_ACCOUNT(ntt->acct_tree));
   if (acc == NULL) {
     message = _("You must choose a Tax Account.");
-    gnc_error_dialog_parented (GTK_WINDOW (ntt->dialog), message);
-    return;
+    gnc_error_dialog (ntt->dialog, message);
+    return FALSE;
   }
 
   gnc_suspend_gui_refresh ();
@@ -141,34 +171,14 @@ new_tax_table_ok_cb (GtkWidget *widget, gpointer data)
   gncTaxTableCommitEdit (ttw->current_table);
 
   gnc_resume_gui_refresh();
-
-  /* Then close/refresh the dialog/window */
-  gnome_dialog_close (GNOME_DIALOG (ntt->dialog));
-}
-
-static void
-new_tax_table_cancel_cb (GtkWidget *widget, gpointer data)
-{
-  NewTaxTable *ntt = data;
-  g_return_if_fail (ntt);
-  gnome_dialog_close (GNOME_DIALOG (ntt->dialog));
-}
-
-static void
-new_tax_table_dialog_destroy_cb (GtkWidget *widget, gpointer data)
-{
-  NewTaxTable *ntt = data;
-
-  if (!ntt) return;
-  g_free (ntt);
+  return TRUE;
 }
 
 static void
 optionmenu_changed (GtkWidget *widget, NewTaxTable *ntt)
 {
   g_return_if_fail (ntt);
-  ntt->type = GPOINTER_TO_INT (gtk_object_get_data (GTK_OBJECT (widget),
-						    "option"));
+  ntt->type = GPOINTER_TO_INT (g_object_get_data (G_OBJECT(widget), "option"));
 }
 
 static GtkWidget *
@@ -178,9 +188,9 @@ add_menu_item (GtkWidget *menu, NewTaxTable *ntt, char *label, gint type)
   GtkWidget *item;
 
   item = gtk_menu_item_new_with_label (label);
-  gtk_object_set_data (GTK_OBJECT (item), "option", GINT_TO_POINTER (type));
-  gtk_signal_connect (GTK_OBJECT (item), "activate", optionmenu_changed, ntt);
-  gtk_menu_append (GTK_MENU (menu), item);
+  g_object_set_data (G_OBJECT (item), "option", GINT_TO_POINTER (type));
+  g_signal_connect (G_OBJECT (item), "activate", G_CALLBACK (optionmenu_changed), ntt);
+  gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
   gtk_widget_show (item);
   return item;
 }
@@ -198,19 +208,22 @@ make_menu (GtkWidget *omenu, NewTaxTable *ntt)
 
   gtk_option_menu_set_menu (GTK_OPTION_MENU (omenu), menu);
 
-  gtk_signal_emit_by_name (GTK_OBJECT ((current == GNC_AMT_TYPE_VALUE-1 ?
+  g_signal_emit_by_name (G_OBJECT ((current == GNC_AMT_TYPE_VALUE-1 ?
 					value : percent)), "activate", ntt);
   gtk_option_menu_set_history (GTK_OPTION_MENU (omenu), current);
   return menu;
 }
 
-static NewTaxTable *
+static GncTaxTable *
 new_tax_table_dialog (TaxTableWindow *ttw, gboolean new_table,
-		      GncTaxTableEntry *entry)
+		      GncTaxTableEntry *entry, const char *name)
 {
+  GncTaxTable *created_table = NULL;
   NewTaxTable *ntt;
   GladeXML *xml;
   GtkWidget *box, *widget;
+  gboolean done;
+  gint response;
 
   if (!ttw) return NULL;
   if (new_table && entry) return NULL;
@@ -229,6 +242,8 @@ new_tax_table_dialog (TaxTableWindow *ttw, gboolean new_table,
   xml = gnc_glade_xml_new ("tax-tables.glade", "New Tax Table Dialog");
   ntt->dialog = glade_xml_get_widget (xml, "New Tax Table Dialog");
   ntt->name_entry = glade_xml_get_widget (xml, "name_entry");
+  if (name)
+    gtk_entry_set_text (GTK_ENTRY (ntt->name_entry), name);
 
   /* Create the menu */
   make_menu (glade_xml_get_widget (xml, "type_menu"), ntt);
@@ -241,54 +256,77 @@ new_tax_table_dialog (TaxTableWindow *ttw, gboolean new_table,
   gtk_box_pack_start (GTK_BOX (box), widget, TRUE, TRUE, 0);
 
   box = glade_xml_get_widget (xml, "acct_window");
-  ntt->acct_tree = widget = gnc_account_tree_new ();
-  gtk_clist_column_titles_hide (GTK_CLIST (widget));
-  gnc_account_tree_hide_all_but_name (GNC_ACCOUNT_TREE (widget));
-  gtk_container_add (GTK_CONTAINER (box), widget);
-  gnc_account_tree_refresh(GNC_ACCOUNT_TREE(ntt->acct_tree));
+  ntt->acct_tree = GTK_WIDGET(gnc_tree_view_account_new (FALSE));
+  gtk_container_add (GTK_CONTAINER (box), ntt->acct_tree);
+  gtk_tree_view_set_headers_visible (GTK_TREE_VIEW(ntt->acct_tree), FALSE);
 
   /* Make 'enter' do the right thing */
-  gnome_dialog_set_default (GNOME_DIALOG (ntt->dialog), 0);
-  gnome_dialog_editable_enters (GNOME_DIALOG (ntt->dialog),
-				GTK_EDITABLE (ntt->name_entry));
-  gnome_dialog_editable_enters (GNOME_DIALOG (ntt->dialog),
-				GTK_EDITABLE (gnc_amount_edit_gtk_entry
-					      (GNC_AMOUNT_EDIT (ntt->amount_entry))));
+  gtk_entry_set_activates_default(GTK_ENTRY (gnc_amount_edit_gtk_entry
+					     (GNC_AMOUNT_EDIT (ntt->amount_entry))),
+				  TRUE);
 
-  /* Connect the dialog buttons */
-  gnome_dialog_button_connect (GNOME_DIALOG (ntt->dialog), 0,
-			       new_tax_table_ok_cb, ntt);
-
-  gnome_dialog_button_connect (GNOME_DIALOG (ntt->dialog), 1,
-			       new_tax_table_cancel_cb, ntt);
+  /* Fix mnemonics for generated target widgets */
+  widget = glade_xml_get_widget (xml, "value_label");
+  gtk_label_set_mnemonic_widget (GTK_LABEL (widget), ntt->amount_entry);
+  widget = glade_xml_get_widget (xml, "account_label");
+  gtk_label_set_mnemonic_widget (GTK_LABEL (widget), ntt->acct_tree);
 
   /* Fill in the widgets appropriately */
   if (entry) {
     gnc_amount_edit_set_amount (GNC_AMOUNT_EDIT (ntt->amount_entry),
 				gncTaxTableEntryGetAmount (entry));
-    gnc_account_tree_select_account (GNC_ACCOUNT_TREE (ntt->acct_tree),
-				     gncTaxTableEntryGetAccount (entry), TRUE);
+    gnc_tree_view_account_set_selected_account (GNC_TREE_VIEW_ACCOUNT (ntt->acct_tree),
+						gncTaxTableEntryGetAccount (entry));
   }
 
-  /* Set our modality */
-  gnome_dialog_set_parent (GNOME_DIALOG (ntt->dialog),
-			   GTK_WINDOW (ttw->dialog));
-  gtk_window_set_modal (GTK_WINDOW (ntt->dialog), TRUE);
+  /* Set our parent */
+  gtk_window_set_transient_for (GTK_WINDOW(ntt->dialog), GTK_WINDOW(ttw->dialog));
 
-  gtk_signal_connect (GTK_OBJECT (ntt->dialog), "destroy",
-		      new_tax_table_dialog_destroy_cb, ntt);
+  /* Setup signals */
+  glade_xml_signal_autoconnect_full( xml,
+                                     gnc_glade_autoconnect_full_func,
+                                     ntt);
 
   /* Show what we should */
   gtk_widget_show_all (ntt->dialog);
   if (new_table == FALSE) {
-    widget = glade_xml_get_widget (xml, "table_frame");
-    gtk_widget_hide_all (widget);
+    gtk_widget_hide (glade_xml_get_widget (xml, "table_title"));
+    gtk_widget_hide (glade_xml_get_widget (xml, "table_name"));
+    gtk_widget_hide (glade_xml_get_widget (xml, "spacer"));
+    gtk_widget_hide (ntt->name_entry);
+    /* Tables are great for layout, but a pain when you hide widgets */
+    widget = glade_xml_get_widget (xml, "ttd_table");
+    gtk_table_set_row_spacing (GTK_TABLE(widget), 0, 0);
+    gtk_table_set_row_spacing (GTK_TABLE(widget), 1, 0);
+    gtk_table_set_row_spacing (GTK_TABLE(widget), 2, 0);
     gtk_widget_grab_focus (gnc_amount_edit_gtk_entry
 			   (GNC_AMOUNT_EDIT (ntt->amount_entry)));
   } else
     gtk_widget_grab_focus (ntt->name_entry);
 
-  return ntt;
+  /* Display the dialog now that we're done manipulating it */
+  gtk_widget_show (ntt->dialog);
+
+  done = FALSE;
+  while (!done) {
+    response = gtk_dialog_run (GTK_DIALOG (ntt->dialog));
+    switch (response) {
+     case GTK_RESPONSE_OK:
+      if (new_tax_table_ok_cb (ntt)) {
+	created_table = ntt->created_table;
+	done = TRUE;
+      }
+      break;
+     default:
+      done = TRUE;
+      break;
+    }
+  }
+
+  gtk_widget_destroy(ntt->dialog);
+  g_free(ntt);
+
+  return created_table;
 }
 
 /***********************************************************************/
@@ -327,7 +365,7 @@ tax_table_entries_refresh (TaxTableWindow *ttw, gboolean new_table)
     Account *acc = gncTaxTableEntryGetAccount (entry);
     gnc_numeric amount = gncTaxTableEntryGetAmount (entry);
 
-    row_text[0] = xaccAccountGetFullName (acc, gnc_get_account_separator ());
+    row_text[0] = xaccAccountGetFullName (acc);
     switch (gncTaxTableEntryGetType (entry)) {
     case GNC_AMT_TYPE_PERCENT:
      row_text[1] =
@@ -415,7 +453,7 @@ tax_table_window_refresh (TaxTableWindow *ttw)
 
     gnc_gui_component_watch_entity (ttw->component_id,
 				    gncTaxTableGetGUID (table),
-				    GNC_EVENT_MODIFY);
+				    QOF_EVENT_MODIFY);
 
     row_text[0] = (char *)gncTaxTableGetName (table);
     row_text[1] = NULL;
@@ -429,7 +467,7 @@ tax_table_window_refresh (TaxTableWindow *ttw)
 
   gnc_gui_component_watch_entity_type (ttw->component_id,
 				       GNC_TAXTABLE_MODULE_NAME,
-				       GNC_EVENT_CREATE | GNC_EVENT_DESTROY);
+				       QOF_EVENT_CREATE | QOF_EVENT_DESTROY);
 
   if (vadjustment) {
     save_value = CLAMP (save_value, vadjustment->lower,
@@ -454,7 +492,7 @@ tax_table_window_refresh (TaxTableWindow *ttw)
   /* select_row() above will refresh the entries window */
 }
 
-static void
+void
 tax_table_row_selected (GtkCList *clist, gint row, gint column,
 			GdkEventButton *event, gpointer user_data)
 {
@@ -473,7 +511,7 @@ tax_table_row_selected (GtkCList *clist, gint row, gint column,
   tax_table_entries_refresh (ttw, TRUE);
 }
 
-static void
+void
 tax_table_entry_row_selected (GtkCList *clist, gint row, gint column,
 			      GdkEventButton *event, gpointer user_data)
 {
@@ -487,17 +525,17 @@ tax_table_entry_row_selected (GtkCList *clist, gint row, gint column,
 
   /* If we double-click an item, then pop up an 'edit' window */
   if (event && event->type == GDK_2BUTTON_PRESS)
-    new_tax_table_dialog (ttw, FALSE, entry);
+    new_tax_table_dialog (ttw, FALSE, entry, NULL);
 }
 
-static void
+void
 tax_table_new_table_cb (GtkButton *button, TaxTableWindow *ttw)
 {
   g_return_if_fail (ttw);
-  new_tax_table_dialog (ttw, TRUE, NULL);
+  new_tax_table_dialog (ttw, TRUE, NULL, NULL);
 }
 
-static void
+void
 tax_table_delete_table_cb (GtkButton *button, TaxTableWindow *ttw)
 {
   g_return_if_fail (ttw);
@@ -509,42 +547,43 @@ tax_table_delete_table_cb (GtkButton *button, TaxTableWindow *ttw)
     char *message =
       g_strdup_printf (_("Tax table \"%s\" is in use.  You cannot delete it."),
 		       gncTaxTableGetName (ttw->current_table));
-    gnc_error_dialog_parented (GTK_WINDOW (ttw->dialog), "%s", message);
+    gnc_error_dialog (ttw->dialog, "%s", message);
     g_free (message);
     return;
   }
 
-  if (gnc_verify_dialog_parented (ttw->dialog, FALSE,
-				  _("Are you sure you want to delete \"%s\"?"),
-				  gncTaxTableGetName (ttw->current_table))) {
+  if (gnc_verify_dialog (ttw->dialog, FALSE,
+			 _("Are you sure you want to delete \"%s\"?"),
+			 gncTaxTableGetName (ttw->current_table))) {
     /* Ok, let's remove it */
     gnc_suspend_gui_refresh ();
     gncTaxTableBeginEdit (ttw->current_table);
     gncTaxTableDestroy (ttw->current_table);
     ttw->current_table = NULL;
+    ttw->current_entry = NULL;
     gnc_resume_gui_refresh ();
   }
 }
 
-static void
+void
 tax_table_new_entry_cb (GtkButton *button, TaxTableWindow *ttw)
 {
   g_return_if_fail (ttw);
   if (!ttw->current_table)
     return;
-  new_tax_table_dialog (ttw, FALSE, NULL);
+  new_tax_table_dialog (ttw, FALSE, NULL, NULL);
 }
 
-static void
+void
 tax_table_edit_entry_cb (GtkButton *button, TaxTableWindow *ttw)
 {
   g_return_if_fail (ttw);
   if (!ttw->current_entry)
     return;
-  new_tax_table_dialog (ttw, FALSE, ttw->current_entry);
+  new_tax_table_dialog (ttw, FALSE, ttw->current_entry, NULL);
 }
 
-static void
+void
 tax_table_delete_entry_cb (GtkButton *button, TaxTableWindow *ttw)
 {
   g_return_if_fail (ttw);
@@ -552,14 +591,14 @@ tax_table_delete_entry_cb (GtkButton *button, TaxTableWindow *ttw)
     return;
 
   if (g_list_length (gncTaxTableGetEntries (ttw->current_table)) <= 1) {
-    char *message = _("You cannot remove the last entry from the tax table.\n"
+    char *message = _("You cannot remove the last entry from the tax table. "
 		      "Try deleting the tax table if you want to do that.");
-    gnc_error_dialog_parented (GTK_WINDOW (ttw->dialog), message);
+    gnc_error_dialog (ttw->dialog, message);
     return;
   }
 
-  if (gnc_verify_dialog_parented (ttw->dialog, FALSE,
-			  _("Are you sure you want to delete this entry?"))) {
+  if (gnc_verify_dialog (ttw->dialog, FALSE,
+			 _("Are you sure you want to delete this entry?"))) {
     /* Ok, let's remove it */
     gnc_suspend_gui_refresh ();
     gncTaxTableBeginEdit (ttw->current_table);
@@ -586,18 +625,20 @@ tax_table_window_close_handler (gpointer data)
 {
   TaxTableWindow *ttw = data;
   g_return_if_fail (ttw);
-  gnome_dialog_close (GNOME_DIALOG (ttw->dialog));
+
+  gtk_widget_destroy (ttw->dialog);
 }
 
-static void
+void
 tax_table_window_close (GtkWidget *widget, gpointer data)
 {
   TaxTableWindow *ttw = data;
 
+  gnc_save_window_size (GCONF_SECTION, GTK_WINDOW (ttw->dialog));
   gnc_ui_tax_table_window_destroy (ttw);
 }
 
-static void
+void
 tax_table_window_destroy_cb (GtkWidget *widget, gpointer data)
 {
   TaxTableWindow *ttw = data;
@@ -624,12 +665,11 @@ gnc_ui_tax_table_window_new (GNCBook *book)
 {
   TaxTableWindow *ttw;
   GladeXML *xml;
-  GtkWidget *button;
 
   if (!book) return NULL;
 
   /*
-   * Find an existing tax-tab;e window.  If found, bring it to
+   * Find an existing tax-table window.  If found, bring it to
    * the front.  If we have an actual owner, then set it in
    * the window.
    */
@@ -650,35 +690,10 @@ gnc_ui_tax_table_window_new (GNCBook *book)
   ttw->names_clist = glade_xml_get_widget (xml, "tax_tables_clist");
   ttw->entries_clist = glade_xml_get_widget (xml, "tax_table_entries");
 
-  /* Connect all the buttons */
-  button = glade_xml_get_widget (xml, "new_table_button");
-  gtk_signal_connect (GTK_OBJECT (button), "clicked",
-		      tax_table_new_table_cb, ttw);
-  button = glade_xml_get_widget (xml, "delete_table_button");
-  gtk_signal_connect (GTK_OBJECT (button), "clicked",
-		      tax_table_delete_table_cb, ttw);
-  button = glade_xml_get_widget (xml, "new_entry_button");
-  gtk_signal_connect (GTK_OBJECT (button), "clicked",
-		      tax_table_new_entry_cb, ttw);
-  button = glade_xml_get_widget (xml, "edit_entry_button");
-  gtk_signal_connect (GTK_OBJECT (button), "clicked",
-		      tax_table_edit_entry_cb, ttw);
-  button = glade_xml_get_widget (xml, "delete_entry_button");
-  gtk_signal_connect (GTK_OBJECT (button), "clicked",
-		      tax_table_delete_entry_cb, ttw);
-
-  /* Set the row-select callbacks */
-  gtk_signal_connect (GTK_OBJECT (ttw->names_clist), "select-row",
-		      tax_table_row_selected, ttw);
-  gtk_signal_connect (GTK_OBJECT (ttw->entries_clist), "select-row",
-		      tax_table_entry_row_selected, ttw);
-
-  /* Connect the dialog buttons */
-  gnome_dialog_button_connect (GNOME_DIALOG (ttw->dialog), 0,
-			       tax_table_window_close, ttw);
-
-  gtk_signal_connect (GTK_OBJECT (ttw->dialog), "destroy",
-		      tax_table_window_destroy_cb, ttw);
+  /* Setup signals */
+  glade_xml_signal_autoconnect_full( xml,
+                                     gnc_glade_autoconnect_full_func,
+                                     ttw);
 
   /* register with component manager */
   ttw->component_id =
@@ -688,6 +703,7 @@ gnc_ui_tax_table_window_new (GNCBook *book)
 				ttw);
 
   tax_table_window_refresh (ttw);
+  gnc_restore_window_size (GCONF_SECTION, GTK_WINDOW (ttw->dialog));
   gtk_widget_show_all (ttw->dialog);
   return ttw;
 }
@@ -702,49 +718,16 @@ gnc_ui_tax_table_window_destroy (TaxTableWindow *ttw)
   gnc_close_gui_component (ttw->component_id);
 }
 
-static int
-from_name_close_cb (GnomeDialog *dialog, gpointer data)
-{
-  NewTaxTable *ntt;
-  GncTaxTable **created_table = data;
-
-  ntt = gtk_object_get_data (GTK_OBJECT (dialog), "dialog_info");
-
-  *created_table = ntt->created_table;
-
-  gtk_main_quit ();
-
-  return FALSE;
-}
-
 /* Create a new tax-table by name */
 GncTaxTable *
 gnc_ui_tax_table_new_from_name (GNCBook *book, const char *name)
 {
-  GncTaxTable *created_table = NULL;
   TaxTableWindow *ttw;
-  NewTaxTable *ntt;
 
   if (!book) return NULL;
 
   ttw = gnc_ui_tax_table_window_new (book);
   if (!ttw) return NULL;
 
-  ntt = new_tax_table_dialog (ttw, TRUE, NULL);
-  if (!ntt) return NULL;
-
-  gtk_object_set_data (GTK_OBJECT (ntt->dialog), "dialog_info", ntt);
-  gtk_signal_connect (GTK_OBJECT (ntt->dialog), "close",
-		      GTK_SIGNAL_FUNC (from_name_close_cb), &created_table);
-
-  /* Preset the name in the new dialog */
-  if (name)
-    gtk_entry_set_text (GTK_ENTRY (ntt->name_entry), name);
-
-  /* I know that NTT is already modal, no need to reset it here */
-
-  /* Now run the dialog -- wait for it to close */
-  gtk_main ();
-
-  return created_table;
+  return new_tax_table_dialog (ttw, TRUE, NULL, name);
 }
