@@ -16,12 +16,11 @@
  * along with this program; if not, contact:                        *
  *                                                                  *
  * Free Software Foundation           Voice:  +1-617-542-5942       *
- * 59 Temple Place - Suite 330        Fax:    +1-617-542-2652       *
- * Boston, MA  02111-1307,  USA       gnu@gnu.org                   *
+ * 51 Franklin Street, Fifth Floor    Fax:    +1-617-542-2652       *
+ * Boston, MA  02110-1301,  USA       gnu@gnu.org                   *
  *                                                                  *
 \********************************************************************/
 
-#define _GNU_SOURCE
 #include "config.h"
 
 #include <errno.h>
@@ -34,8 +33,7 @@
 #include "Transaction.h"
 #include "TransactionP.h"
 #include "TransLog.h"
-#include "date.h"
-#include "gnc-engine-util.h"
+#include "qof.h"
 
 /* 
  * Some design philosphy that I think would be good to keep in mind:
@@ -81,7 +79,8 @@
 
 
 static int gen_logs = 1;
-static FILE * trans_log = NULL;
+static FILE * trans_log = NULL; /**< current log file handle */
+static char * trans_log_name = NULL; /**< current log file name */
 static char * log_base_name = NULL;
 
 /********************************************************************\
@@ -93,6 +92,16 @@ void xaccLogEnable  (void) { gen_logs = 1; }
 /********************************************************************\
 \********************************************************************/
 
+void 
+xaccReopenLog (void)
+{
+   if (trans_log) {
+      xaccCloseLog();
+      xaccOpenLog();
+   }
+}
+
+
 void 
 xaccLogSetBaseName (const char *basepath)
 {
@@ -107,6 +116,28 @@ xaccLogSetBaseName (const char *basepath)
    }
 }
 
+
+/*
+ * See if the provided file name is that of the current log file.
+ * Since the filename is generated with a time-stamp we can ignore the
+ * directory path and avoid problems with worrying about any ".."
+ * components in the path.
+ */
+gboolean
+xaccFileIsCurrentLog (const gchar *name)
+{
+  gchar *base;
+  gint result;
+
+  if (!name || !trans_log_name)
+    return FALSE;
+
+  base = g_path_get_basename(name);
+  result = (strcmp(base, trans_log_name) == 0);
+  g_free(base);
+  return result;
+}
+
 /********************************************************************\
 \********************************************************************/
 
@@ -137,15 +168,20 @@ xaccOpenLog (void)
       return;
    }
 
+   /* Save the log file name */
+   if (trans_log_name)
+     g_free (trans_log_name);
+   trans_log_name = g_path_get_basename(filename);
+
    g_free (filename);
    g_free (timestamp);
 
-   /* use tab-separated fields */
-   fprintf (trans_log, "mod	trans_guid	split_guid	time_now	" \
-                       "date_entered	date_posted	" \
-                       "acc_guid	acc_name	num	description	" \
-                       "notes	memo	action	reconciled	" \
-                       "amount	value	date_reconciled\n");
+   /*  Note: this must match src/import-export/log-replay/gnc-log-replay.c */
+   fprintf (trans_log, "mod\ttrans_guid\tsplit_guid\ttime_now\t"
+                       "date_entered\tdate_posted\t"
+                       "acc_guid\tacc_name\tnum\tdescription\t"
+                       "notes\tmemo\taction\treconciled\t"
+                       "amount\tvalue\tdate_reconciled\n");
    fprintf (trans_log, "-----------------\n");
 }
 
@@ -190,33 +226,41 @@ xaccTransWriteLog (Transaction *trans, char flag)
    trans_notes = xaccTransGetNotes(trans);
    fprintf (trans_log, "===== START\n");
 
-   for (node = trans->splits; node; node = node->next) {
+   for (node = trans->splits; node; node = node->next) 
+   {
       Split *split = node->data;
       const char * accname = "";
       char acc_guid_str[GUID_ENCODING_LENGTH+1];
+      gnc_numeric amt,val;
 
-      if (xaccSplitGetAccount(split)){
+      if (xaccSplitGetAccount(split))
+      {
         accname = xaccAccountGetName (xaccSplitGetAccount(split));
-	guid_to_string_buff(xaccAccountGetGUID(xaccSplitGetAccount(split)),
-			    acc_guid_str);
-      } else {
-	acc_guid_str[0] = '\0';
+        guid_to_string_buff(xaccAccountGetGUID(xaccSplitGetAccount(split)),
+                            acc_guid_str);
+      } 
+      else 
+      {
+        acc_guid_str[0] = '\0';
       }
       
-         timespecFromTime_t(&ts,split->date_reconciled.tv_sec);
-	 gnc_timespec_to_iso8601_buff (ts, drecn);
+      timespecFromTime_t(&ts,split->date_reconciled.tv_sec);
+      gnc_timespec_to_iso8601_buff (ts, drecn);
 
       guid_to_string_buff (xaccSplitGetGUID(split), split_guid_str);
+      amt = xaccSplitGetAmount (split);
+      val = xaccSplitGetValue (split);
+
       /* use tab-separated fields */
       fprintf (trans_log,
                "%c\t%s\t%s\t%s\t%s\t%s\t%s\t%s\t%s\t"
-               "%s\t%s\t%s\t%s\t%c\t%lld/%lld\t%lld/%lld\t%s\n",
+               "%s\t%s\t%s\t%s\t%c\t%" G_GINT64_FORMAT "/%" G_GINT64_FORMAT "\t%" G_GINT64_FORMAT "/%" G_GINT64_FORMAT "\t%s\n",
                flag,
                trans_guid_str, split_guid_str,  /* trans+split make up unique id */
                dnow ? dnow : "",
                dent ? dent : "", 
                dpost ? dpost : "", 
-	       acc_guid_str,
+               acc_guid_str,
                accname ? accname : "",
                trans->num ? trans->num : "", 
                trans->description ? trans->description : "",
@@ -224,10 +268,10 @@ xaccTransWriteLog (Transaction *trans, char flag)
                split->memo ? split->memo : "",
                split->action ? split->action : "",
                split->reconciled,
-               (long long int) gnc_numeric_num(split->amount), 
-               (long long int) gnc_numeric_denom(split->amount),
-               (long long int) gnc_numeric_num(split->value), 
-               (long long int) gnc_numeric_denom(split->value),
+               gnc_numeric_num(amt), 
+               gnc_numeric_denom(amt),
+               gnc_numeric_num(val), 
+               gnc_numeric_denom(val),
                drecn ? drecn : "");
    }
 
@@ -248,12 +292,13 @@ xaccTransWriteLog (Transaction *trans, char flag)
  */
 
 char *
-xaccSplitAsString(Split *split, const char prefix[]) {
+xaccSplitAsString(Split *split, const char prefix[]) 
+{
   char *result = NULL;
   size_t result_size;
   FILE *stream = open_memstream(&result, &result_size); 
   const char *split_memo = xaccSplitGetMemo(split);
-  const double split_value = DxaccSplitGetValue(split);
+  const double split_value = gnc_numeric_to_double(xaccSplitGetValue(split));
   Account *split_dest = xaccSplitGetAccount(split);
   const char *dest_name =
     split_dest ? xaccAccountGetName(split_dest) : NULL;
@@ -287,7 +332,8 @@ xaccTransGetDateStr (Transaction *trans)
 }
 
 char *
-xaccTransAsString(Transaction *txn, const char prefix[]) {
+xaccTransAsString(Transaction *txn, const char prefix[]) 
+{
   char *result = NULL;
   size_t result_size;
   FILE *stream = open_memstream(&result, &result_size); 
@@ -295,7 +341,7 @@ xaccTransAsString(Transaction *txn, const char prefix[]) {
   const char *num = xaccTransGetNum(txn);
   const char *desc = xaccTransGetDescription(txn);
   const char *memo = xaccSplitGetMemo(xaccTransGetSplit(txn, 0));
-  const double total = DxaccSplitGetValue(xaccTransGetSplit(txn, 0));
+  const double total = gnc_numeric_to_double(xaccSplitGetValue(xaccTransGetSplit(txn, 0)));
   
   g_return_val_if_fail (stream, NULL);
 
