@@ -1,7 +1,7 @@
 /********************************************************************\
  * TransactionP.h -- private header for transaction & splits        *
  * Copyright (C) 1997 Robin D. Clark                                *
- * Copyright (C) 1997, 1998, 1999, 2000 Linas Vepstas               *
+ * Copyright (C) 1997-2000 Linas Vepstas <linas@linas.org>          *
  * Copyright (C) 2000 Bill Gribble                                  *
  *                                                                  *
  * This program is free software; you can redistribute it and/or    *
@@ -18,8 +18,8 @@
  * along with this program; if not, contact:                        *
  *                                                                  *
  * Free Software Foundation           Voice:  +1-617-542-5942       *
- * 59 Temple Place - Suite 330        Fax:    +1-617-542-2652       *
- * Boston, MA  02111-1307,  USA       gnu@gnu.org                   *
+ * 51 Franklin Street, Fifth Floor    Fax:    +1-617-542-2652       *
+ * Boston, MA  02110-1301,  USA       gnu@gnu.org                   *
  *                                                                  *
 \********************************************************************/
 
@@ -48,11 +48,9 @@
 #include <time.h>
 #include <glib.h>
 
-#include "config.h"
 #include "gnc-engine.h"   /* for typedefs */
-#include "gnc-numeric.h"
-#include "GNCIdP.h"
-#include "kvp_frame.h"
+#include "SplitP.h"
+#include "qof.h"
 
 
 /** STRUCTS *********************************************************/
@@ -70,75 +68,12 @@
  * accounts.  Thus, a single credit-card transaction might be split
  * between "dining", "tips" and "taxes" categories.
  *
- * A "split" is more commonly refered to as a "entry" in a "transaction".
+ * A "split" is more commonly referred to as an "entry" in a "transaction".
  */
 
-
-struct split_s
-{
-  GUID guid;  /* globally unique id */
-
-  GNCBook *book;             /* The enitity table where this split is stored. */
-
-  Account *acc;              /* back-pointer to debited/credited account  */
-
-  GNCLot *lot;               /* back-pointer to debited/credited lot */
-
-  Transaction *parent;       /* parent of split                           */
-
-  /* The memo field is an arbitrary user-assiged value. 
-   * It is intended to hold a short (zero to forty character) string 
-   * that is displayed by the GUI along with this split. 
-   */
-  char  * memo;
-
-  /* The action field is an arbitrary user-assigned value.
-   * It is meant to be a very short (one to ten cahracter) string that
-   * signifies the "type" of this split, such as e.g. Buy, Sell, Div,
-   * Withdraw, Deposit, ATM, Check, etc. The idea is that this field
-   * can be used to create custom reports or graphs of data.
-   */ 
-  char  * action;            /* Buy, Sell, Div, etc.                      */
-
-  /* kvp_data is a key-value pair database for storing simple 
-   * "extra" information in splits, transactions, and accounts. 
-   * it's NULL until accessed. */
-  kvp_frame * kvp_data;
-
-  char    reconciled;        /* The reconciled field                      */
-  Timespec date_reconciled;  /* date split was reconciled                 */
-
-  /* 'value' is the amount of the transaction balancing commodity
-   * (i.e. currency) involved, 'amount' is the amount of the account's
-   * commodity (formerly known as 'security') involved. */
-  gnc_numeric  value;
-  gnc_numeric  amount;
-
-  /* -------------------------------------------------------------- */
-  /* Below follow some 'temporary' fields */
-
-  /* The various "balances" are the sum of all of the values of 
-   * all the splits in the account, up to and including this split.
-   * These balances apply to a sorting order by date posted
-   * (not by date entered). */
-  gnc_numeric  balance;
-  gnc_numeric  cleared_balance;
-  gnc_numeric  reconciled_balance;
-
-  /* -------------------------------------------------------------- */
-  /* Backend private expansion data */
-  guint32  idata;     /* used by the sql backend for kvp management */
-};
-
-
 struct transaction_s
 {
-  /* guid is a globally unique identifier which can be used to
-   * reference the transaction.
-   */
-  GUID guid;
-
-  GNCBook *book;         /* The entity_table where the transaction is stored */
+  QofInstance inst;     /* glbally unique id */
 
   Timespec date_entered;     /* date register entry was made              */
   Timespec date_posted;      /* date transaction was posted at bank       */
@@ -154,23 +89,10 @@ struct transaction_s
    */
   char * description;        
 
-  /* kvp_data is a key-value pair database for storing simple 
-   * "extra" information in splits, transactions, and accounts. 
-   * it's NULL until accessed. */
-  kvp_frame * kvp_data;
-
-
   /* The common_currency field is the balancing common currency for
-   * all the splits in the transaction. 
-   *
-   * This field is going to replace the currency field in the account
-   * structures.  However, right now we are in a transition period: we
-   * store it here an in the account, and test its value dynamically
-   * for correctness.  If we can run for a few months without errors,
-   * then we'll make the conversion permanent.
-   *
-   * Alternate, better(?) name: "valuation currency": it is the
-   * currency in which all of the splits can be valued.  */
+   * all the splits in the transaction.  Alternate, better(?) name: 
+   * "valuation currency": it is the currency in which all of the 
+   * splits can be valued.  */
   gnc_commodity *common_currency;
 
   /* version number, used for tracking multiuser updates */
@@ -187,10 +109,7 @@ struct transaction_s
    * corresponding to the current traversal. */
   unsigned char  marker;      
 
-  gint32 editlevel; /* nestcount of begin/end edit calls */
-  gboolean do_free; /* transaction in process of being destroyed */
-
-  /* the orig pointer points at a copy of the original transaction,
+  /* The orig pointer points at a copy of the original transaction,
    * before editing was started.  This orig copy is used to rollback 
    * any changes made if/when the edit is abandoned.
    */
@@ -204,20 +123,7 @@ struct transaction_s
 /* Set the transaction's GUID. This should only be done when reading
  * a transaction from a datafile, or some other external source. Never
  * call this on an existing transaction! */
-void xaccTransSetGUID (Transaction *trans, const GUID *guid);
-
-/* Set the split's GUID. This should only be done when reading
- * a split from a datafile, or some other external source. Never
- * call this on an existing split! */
-void xaccSplitSetGUID (Split *split, const GUID *guid);
-
-/* The xaccFreeSplit() method simply frees all memory associated
- * with the split.  It does not verify that the split isn't
- * referenced in some account.  If the split is referenced by an 
- * account, then calling this method will leave the system in an 
- * inconsistent state.  This *will* lead to crashes and hangs.
- */
-void  xaccFreeSplit (Split *split);    /* frees memory */
+#define xaccTransSetGUID(t,g) qof_entity_set_guid(QOF_ENTITY(t),g)
 
 /* This routine makes a 'duplicate' of the indicated transaction.
  * This routine cannot be exposed publically since the duplicate
@@ -228,12 +134,7 @@ void  xaccFreeSplit (Split *split);    /* frees memory */
  * Another 'feature': the splits point at the old transaction
  * as the parent, not the new transaction.  
  */
-Transaction * xaccDupeTransaction (Transaction *t);
-
-/* compute the value of a list of splits in the given currency,
- * excluding the skip_me split. */
-gnc_numeric xaccSplitsComputeValue (GList *splits, Split * skip_me,
-                                    const gnc_commodity * base_currency);
+Transaction * xaccDupeTransaction (const Transaction *t);
 
 /* The xaccTransSet/GetVersion() routines set & get the version
  *    numbers on this transaction.  The version number is used to manage
@@ -243,16 +144,40 @@ gnc_numeric xaccSplitsComputeValue (GList *splits, Split * skip_me,
 void xaccTransSetVersion (Transaction*, gint32);
 gint32 xaccTransGetVersion (const Transaction*);
 
-/* The xaccTransFindCommonCurrency () method returns a gnc_commodity
- *    indicating a currency denomination that all of the splits in this
- *    transaction have in common, using the old currency/security fields
- *    of the split accounts. */
-gnc_commodity * xaccTransFindOldCommonCurrency (Transaction *trans,
-                                                GNCBook *book);
-
-/* Code to register Split and Transaction types with the engine */
-gboolean xaccSplitRegister (void);
+/* Code to register Transaction type with the engine */
 gboolean xaccTransRegister (void);
 
+/* The xaccTransactionGetBackend() subroutine will find the
+ *    persistent-data storage backend associated with this 
+ *    transaction.
+ */
+QofBackend * xaccTransactionGetBackend (Transaction *trans);
+
+/* The xaccEnable/DisableDataScrubbing() routines affect what
+ *   happens during transaction commit.  When scrubbing is enabled,
+ *   then transactions are fixed up during transaction commit, 
+ *   so that only consistent transactions are commited to the engine.
+ *   However, when data is being loaded from a backend (in particular,
+ *   from the file backend), the data might not be consistent until
+ *   its completely loaded.   In particular, gains transactions might
+ *   be loaded at a different time than the transactions that casued
+ *   the gains.  Thus, scrubbing needs do be disabled during file
+ *   load.  These routines enable that.
+ */
+void xaccEnableDataScrubbing(void);
+void xaccDisableDataScrubbing(void);
+
+/** Set the KvpFrame slots of this transaction to the given frm by
+ *  * directly using the frm pointer (i.e. non-copying).
+ *   * XXX this is wrong, nedds to be replaced with a transactional thingy
+ *   in kvp + qofinstance. for now, this is a quasi-unctional placeholder.
+ *    */
+#define xaccTransSetSlots_nc(T,F) qof_instance_set_slots(QOF_INSTANCE(T),F)
+
+void xaccTransRemoveSplit (Transaction *trans, const Split *split);
+G_INLINE_FUNC void check_open (const Transaction *trans);
+
+/*@}*/
+
 
 #endif /* XACC_TRANSACTION_P_H */
