@@ -22,20 +22,18 @@
  * along with this program; if not, contact:                        *
  *                                                                  *
  * Free Software Foundation           Voice:  +1-617-542-5942       *
- * 59 Temple Place - Suite 330        Fax:    +1-617-542-2652       *
- * Boston, MA  02111-1307,  USA       gnu@gnu.org                   *
+ * 51 Franklin Street, Fifth Floor    Fax:    +1-617-542-2652       *
+ * Boston, MA  02110-1301,  USA       gnu@gnu.org                   *
 \********************************************************************/
 
 #include "config.h"
 
 #include "AccountP.h"
-#include "GNCIdP.h"
 #include "gnc-associate-account.h"
-#include "gnc-book-p.h"
 #include "gnc-engine.h"
-#include "gnc-engine-util.h"
+#include "qof.h"
 
-static short module = MOD_ENGINE;
+static QofLogModule log_module = GNC_MOD_ENGINE;
 
 /* Maps GNCTrackingIncomeCategory to string keys.  If this enum
    changes, update */
@@ -52,11 +50,11 @@ static char * income_to_key[] = {"income-miscellaneous",
 static char * expense_to_key[] = {"expense-miscellaneous",
 				  "expense-commission"};
 
-static kvp_frame *
-get_assoc_acc_frame(kvp_frame *account_frame)
+static KvpFrame *
+get_assoc_acc_frame(KvpFrame *account_frame)
 {
-  kvp_frame *assoc_acc_frame;
-  kvp_value *assoc_acc_frame_kvpvalue =
+  KvpFrame *assoc_acc_frame;
+  KvpValue *assoc_acc_frame_kvpvalue =
     kvp_frame_get_slot(account_frame, "associated-accounts");
 
   assoc_acc_frame = kvp_value_get_frame(assoc_acc_frame_kvpvalue);
@@ -76,8 +74,8 @@ back_associate_expense_accounts(Account *stock_account,
 				GList *accounts,
 				GNCTrackingExpenseCategory category)
 {
-  kvp_frame *acc_frame;
-  kvp_value *val, *stock_acc_guid_kvpval, *stock_acc_category_kvpval;
+  KvpFrame *acc_frame;
+  KvpValue *val, *stock_acc_guid_kvpval, *stock_acc_category_kvpval;
   const GUID *stock_acc_guid;
   const GUID *existing_acc_guid;
 
@@ -89,13 +87,11 @@ back_associate_expense_accounts(Account *stock_account,
   for(; accounts; accounts = g_list_next(accounts))
   {
     acc_frame = xaccAccountGetSlots(accounts->data);
-    g_return_if_fail(val = kvp_frame_get_slot(acc_frame, 
-					      "associated-stock-account"));
+    g_return_if_fail((val = kvp_frame_get_slot(acc_frame, 
+					      "associated-stock-account")));
     g_return_if_fail(kvp_value_get_type(val) == KVP_TYPE_GUID);
     existing_acc_guid = kvp_value_get_guid(val);
 
-    g_return_if_fail(GNC_ID_NONE == xaccGUIDType (existing_acc_guid, stock_account->book));
-    
     kvp_frame_set_slot_nc(acc_frame, "associated-stock-account",
                           stock_acc_guid_kvpval);
     
@@ -111,8 +107,8 @@ back_associate_income_accounts(Account *stock_account,
 			       GList *accounts,
 			       GNCTrackingIncomeCategory category)
 {
-  kvp_frame *acc_frame;
-  kvp_value *val, *stock_acc_guid_kvpval, *stock_acc_category_kvpval;
+  KvpFrame *acc_frame;
+  KvpValue *val, *stock_acc_guid_kvpval, *stock_acc_category_kvpval;
   const GUID *stock_acc_guid;
   const GUID *existing_acc_guid;
 
@@ -124,15 +120,11 @@ back_associate_income_accounts(Account *stock_account,
   for(; accounts; accounts = g_list_next(accounts))
   {
     acc_frame = xaccAccountGetSlots(accounts->data);
-    g_return_if_fail(val = kvp_frame_get_slot(acc_frame, 
-					      "associated-stock-account"));
+    g_return_if_fail((val = kvp_frame_get_slot(acc_frame, 
+					      "associated-stock-account")));
     g_return_if_fail(kvp_value_get_type(val) == KVP_TYPE_GUID);
     existing_acc_guid = kvp_value_get_guid(val);
 
-    g_return_if_fail(xaccGUIDType(existing_acc_guid,
-                                             stock_account->book) ==
-                     GNC_ID_NONE);
-    
     kvp_frame_set_slot_nc(acc_frame, "associated-stock-account",
 		       stock_acc_guid_kvpval);
     kvp_frame_set_slot_nc(acc_frame, "associated-stock-account-category",
@@ -142,12 +134,12 @@ back_associate_income_accounts(Account *stock_account,
   return;
 }  
     
-static kvp_value *
+static KvpValue *
 make_kvpd_on_list(GList *account_list)
 {
   GList *iter;
-  kvp_value *retval;
-  kvp_value *guid_kvp;
+  KvpValue *retval;
+  KvpValue *guid_kvp;
   GList  *kvp_acc_list = NULL;
   const GUID *acc_id;
 
@@ -172,7 +164,7 @@ make_kvpd_on_list(GList *account_list)
 }
 
 static GList *
-de_kvp_account_list(kvp_value *kvpd_list, GNCBook *book)
+de_kvp_account_list(KvpValue *kvpd_list, QofBook *book)
 { 
   GList *guid_account_list = kvp_value_get_glist(kvpd_list);
   if (guid_account_list)
@@ -180,7 +172,7 @@ de_kvp_account_list(kvp_value *kvpd_list, GNCBook *book)
     GList *expense_acc_list= NULL;
     for(; guid_account_list; guid_account_list=g_list_next(guid_account_list))
     {
-      g_list_prepend(expense_acc_list,
+      expense_acc_list = g_list_prepend(expense_acc_list,
                      xaccAccountLookup(guid_account_list->data, book));
     }
     
@@ -214,13 +206,11 @@ gnc_tracking_associate_income_accounts(Account *stock_account,
                                        GNCTrackingIncomeCategory category, 
                                        GList *account_list)
 {
-  kvp_frame *account_frame, *inc_account_frame;
-  kvp_value *kvpd_on_account_list;
-  GNCAccountType type;
+  KvpFrame *account_frame, *inc_account_frame;
+  KvpValue *kvpd_on_account_list;
 
   g_return_if_fail(stock_account);
-  type = xaccAccountGetType(stock_account);
-  g_return_if_fail(type == STOCK || type == MUTUAL);
+  g_return_if_fail(xaccAccountIsPriced(stock_account));
   account_frame = xaccAccountGetSlots(stock_account);
   g_return_if_fail(account_frame);
   g_return_if_fail(category >= 0);
@@ -256,13 +246,11 @@ gnc_tracking_asssociate_expense_account(Account *stock_account,
                                         GNCTrackingExpenseCategory category,
                                         GList *account_list)
 {
-  kvp_frame *account_frame, *expense_acc_frame;
-  kvp_value *kvpd_on_account_list;
-  GNCAccountType type;
+  KvpFrame *account_frame, *expense_acc_frame;
+  KvpValue *kvpd_on_account_list;
 
   g_return_if_fail(stock_account);
-  type = xaccAccountGetType(stock_account);
-  g_return_if_fail(type == STOCK || type == MUTUAL);
+  g_return_if_fail(xaccAccountIsPriced(stock_account));
   account_frame = xaccAccountGetSlots(stock_account);
   g_return_if_fail(account_frame);
   g_return_if_fail(category >= 0);
@@ -294,15 +282,12 @@ GList *
 gnc_tracking_find_expense_accounts(Account *stock_account, 
                                    GNCTrackingExpenseCategory category)
 {
-  
-  GNCAccountType type;
-  kvp_frame *account_frame, *expense_acc_frame;
-  kvp_value *kvpd_on_account_list;
+  KvpFrame *account_frame, *expense_acc_frame;
+  KvpValue *kvpd_on_account_list;
 
-  type = xaccAccountGetType(stock_account);
+  g_return_val_if_fail(xaccAccountIsPriced(stock_account), NULL);  
   g_return_val_if_fail(category >= 0 && category < GNC_TR_EXP_N_CATEGORIES,
 		       NULL);
-  g_return_val_if_fail(type == STOCK || type == MUTUAL, NULL);
 
   account_frame = xaccAccountGetSlots(stock_account);
   g_return_val_if_fail(account_frame, NULL);
@@ -311,7 +296,7 @@ gnc_tracking_find_expense_accounts(Account *stock_account,
   kvpd_on_account_list = kvp_frame_get_slot(account_frame,
 					    expense_to_key[category]);
 
-  return de_kvp_account_list(kvpd_on_account_list, stock_account->book);
+  return de_kvp_account_list(kvpd_on_account_list, stock_account->inst.book);
 }
 
 /*********************************************************************\
@@ -329,14 +314,12 @@ GList *
 gnc_tracking_find_income_accounts(Account *stock_account, 
                                   GNCTrackingIncomeCategory category)
 {
-  GNCAccountType type;
-  kvp_frame *account_frame, *income_acc_frame;
-  kvp_value *kvpd_on_account_list;
+  KvpFrame *account_frame, *income_acc_frame;
+  KvpValue *kvpd_on_account_list;
 
-  type = xaccAccountGetType(stock_account);
+  g_return_val_if_fail(xaccAccountIsPriced(stock_account), NULL);
   g_return_val_if_fail(category >= 0 && category < GNC_TR_INC_N_CATEGORIES,
 		       NULL);
-  g_return_val_if_fail(type == STOCK || type == MUTUAL, NULL);
 
   account_frame = xaccAccountGetSlots(stock_account);
   g_return_val_if_fail(account_frame, NULL);
@@ -345,7 +328,7 @@ gnc_tracking_find_income_accounts(Account *stock_account,
   kvpd_on_account_list = kvp_frame_get_slot(income_acc_frame,
 					    income_to_key[category]);
   
-  return de_kvp_account_list(kvpd_on_account_list, stock_account->book);
+  return de_kvp_account_list(kvpd_on_account_list, stock_account->inst.book);
 }
 
 /*********************************************************************\
@@ -390,7 +373,7 @@ gnc_tracking_find_all_income_accounts(Account *stock_account)
 
   for(i = 0; i < GNC_TR_EXP_N_CATEGORIES; i++)
   {
-    g_list_concat(complete_list, 
+    complete_list = g_list_concat(complete_list, 
 		  gnc_tracking_find_expense_accounts(stock_account,
 						     i));
   }
@@ -430,9 +413,9 @@ void
 gnc_tracking_dissociate_account(Account *inc_or_expense_account)
 {
   GNCAccountType type;
-  kvp_frame *stock_account_kvpframe, *assoc_acc_kvpframe;
-  kvp_frame *current_account_kvpframe;
-  kvp_value *stock_account_kvpval, *acc_list_kvpval, *category_kvpval;
+  KvpFrame *stock_account_kvpframe, *assoc_acc_kvpframe;
+  KvpFrame *current_account_kvpframe;
+  KvpValue *stock_account_kvpval, *acc_list_kvpval, *category_kvpval;
   const GUID *stock_account_guid, *inc_or_expense_account_guid, *current_guid;
   Account *stock_account;
   char *category_name;
@@ -448,10 +431,6 @@ gnc_tracking_dissociate_account(Account *inc_or_expense_account)
 					    "associated-stock-account");
   
   stock_account_guid = kvp_value_get_guid(stock_account_kvpval);
-  if(!safe_strcmp
-     (xaccGUIDType(stock_account_guid, inc_or_expense_account->book),
-      GNC_ID_NULL))
-    return;
 
   category_kvpval = kvp_frame_get_slot(current_account_kvpframe,
 				       "associated-stock-account-category");
@@ -460,26 +439,26 @@ gnc_tracking_dissociate_account(Account *inc_or_expense_account)
 
   inc_or_expense_account_guid = xaccAccountGetGUID(inc_or_expense_account);
   stock_account = xaccAccountLookup
-    (stock_account_guid, inc_or_expense_account->book);
+    (stock_account_guid, inc_or_expense_account->inst.book);
 
   stock_account_kvpframe = xaccAccountGetSlots(stock_account);
 
-  g_return_if_fail(stock_account_kvpval =
+  g_return_if_fail((stock_account_kvpval =
                    kvp_frame_get_slot(stock_account_kvpframe,
-                                      "associated-accounts"));
+                                      "associated-accounts")));
 
   assoc_acc_kvpframe = kvp_value_get_frame(stock_account_kvpval);
 
-  g_return_if_fail(acc_list_kvpval = kvp_frame_get_slot(assoc_acc_kvpframe, 
-							category_name));
-  g_return_if_fail(assoc_acc_list_start =
-                   kvp_value_get_glist(acc_list_kvpval));
+  g_return_if_fail((acc_list_kvpval = kvp_frame_get_slot(assoc_acc_kvpframe, 
+							category_name)));
+  g_return_if_fail((assoc_acc_list_start =
+                   kvp_value_get_glist(acc_list_kvpval)));
   
   for(assoc_acc_list = assoc_acc_list_start;
       assoc_acc_list;
       assoc_acc_list = g_list_next(assoc_acc_list))
   {
-    g_return_if_fail(current_guid = kvp_value_get_guid(assoc_acc_list->data));
+    g_return_if_fail((current_guid = kvp_value_get_guid(assoc_acc_list->data)));
     if(guid_equal(current_guid, inc_or_expense_account_guid))
     {
       assoc_acc_list_start =
