@@ -1,6 +1,7 @@
 /********************************************************************
  * gnc-pricedb-p.h -- a simple price database for gnucash.          *
  * Copyright (C) 2001 Rob Browning                                  *
+ * Copyright (C) 2003 Linas Vepstas <linas@linas.org>               *
  *                                                                  *
  * This program is free software; you can redistribute it and/or    *
  * modify it under the terms of the GNU General Public License as   *
@@ -16,8 +17,8 @@
  * along with this program; if not, contact:                        *
  *                                                                  *
  * Free Software Foundation           Voice:  +1-617-542-5942       *
- * 59 Temple Place - Suite 330        Fax:    +1-617-542-2652       *
- * Boston, MA  02111-1307,  USA       gnu@gnu.org                   *
+ * 51 Franklin Street, Fifth Floor    Fax:    +1-617-542-2652       *
+ * Boston, MA  02110-1301,  USA       gnu@gnu.org                   *
  *                                                                  *
  *******************************************************************/
 
@@ -25,17 +26,14 @@
 #define GNC_PRICEDB_P_H
 
 #include <glib.h>
-
-#include "GNCIdP.h"
-#include "gnc-book.h"
+#include "qof.h"
 #include "gnc-engine.h"
 #include "gnc-pricedb.h"
 
 struct gnc_price_s
 {
   /* 'public' data fields */
-  GUID    guid;                  /* globally unique price id */
-  GNCBook *book;                 /* book to which this price belongs to */
+  QofInstance inst;              /* globally unique object identifier */
 
   GNCPriceDB *db;
   gnc_commodity *commodity;
@@ -48,20 +46,16 @@ struct gnc_price_s
   guint32  version_check;        /* data aging timestamp */
 
   /* 'private' object management fields */
-  GNCEntityTable *entity_table;  /* table in which price is stored */
   guint32  refcount;             /* garbage collection reference count */
-  gint32   editlevel;            /* nesting level of begin/end edit calls */
-  gboolean not_saved;            /* price edit saved flag */
-  gboolean do_free;              /* price is going to be destroyed soon */
 };
 
 
 
 struct gnc_price_db_s
 {
+  QofInstance inst;              /* globally unique object identifier */
   GHashTable *commodity_hash;
-  GNCBook *book;   /* book to which this database and all the prices belong to */
-  gboolean dirty;
+  gboolean bulk_update;		 /* TRUE while reading XML file, etc. */
 };
 
 /* These structs define the kind of price lookup being done
@@ -82,8 +76,8 @@ struct gnc_price_lookup_s
 {
   PriceLookupType type;
   GNCPriceDB     *prdb;
-  gnc_commodity  *commodity;
-  gnc_commodity  *currency;
+  const gnc_commodity  *commodity;
+  const gnc_commodity  *currency;
   Timespec        date;
 };
 
@@ -94,11 +88,14 @@ typedef struct gnc_price_lookup_helper_s
   Timespec time;
 } GNCPriceLookupHelper;
 
-
-void     gnc_pricedb_mark_clean(GNCPriceDB *db);
+#define  gnc_price_set_guid(P,G)  qof_entity_set_guid(QOF_ENTITY(P),(G))
 void     gnc_pricedb_substitute_commodity(GNCPriceDB *db,
                                           gnc_commodity *old_c,
                                           gnc_commodity *new_c);
-void     gnc_price_set_guid (GNCPrice *p, const GUID *guid);
+
+/** register the pricedb object with the gncObject system */
+gboolean gnc_pricedb_register (void);
+
+QofBackend * xaccPriceDBGetBackend (GNCPriceDB *prdb);
 
 #endif
