@@ -2,6 +2,7 @@
  * dialog-utils.c -- utility functions for creating dialogs         *
  *                   for GnuCash                                    *
  * Copyright (C) 1999-2000 Linas Vepstas                            *
+ * Copyright (C) 2005 David Hampton <hampton@employees.org>         *
  *                                                                  *
  * This program is free software; you can redistribute it and/or    *
  * modify it under the terms of the GNU General Public License as   *
@@ -17,144 +18,40 @@
  * along with this program; if not, contact:                        *
  *                                                                  *
  * Free Software Foundation           Voice:  +1-617-542-5942       *
- * 59 Temple Place - Suite 330        Fax:    +1-617-542-2652       *
- * Boston, MA  02111-1307,  USA       gnu@gnu.org                   *
+ * 51 Franklin Street, Fifth Floor    Fax:    +1-617-542-2652       *
+ * Boston, MA  02110-1301,  USA       gnu@gnu.org                   *
  *                                                                  *
 \********************************************************************/
 
-#define _GNU_SOURCE 1  /* necessary to get RTLD_DEFAULT on linux */
-
 #include "config.h"
 
+#include <gtk/gtk.h>
+#include <gdk/gdkkeysyms.h>
+#include "glib-compat.h"
+#include <glib/gi18n.h>
 #include <glade/glade.h>
-#include <gnome.h>
 #include <gmodule.h>
-#include <dlfcn.h>
+#ifdef HAVE_DLFCN_H
+# include <dlfcn.h>
+#endif
 
 #include "dialog-utils.h"
-#include "global-options.h"
 #include "gnc-commodity.h"
-#include "messages.h"
 #include "Group.h"
 #include "gnc-dir.h"
-#include "gnc-engine-util.h"
+#include "gnc-engine.h"
 #include "gnc-euro.h"
 #include "gnc-ui-util.h"
+#include "gnc-gconf-utils.h"
 
 /* This static indicates the debugging module that this .o belongs to. */
-static short module = MOD_GUI;
+static QofLogModule log_module = GNC_MOD_GUI;
 
+#define WINDOW_POSITION		"window_position"
+#define WINDOW_GEOMETRY		"window_geometry"
 
-/********************************************************************\
- * gnc_ui_source_menu_create                                        *
- *   create the menu of stock quote sources                         *
- *                                                                  *
- * Args:    account - account to use to set default choice          *
- * Returns: the menu                                                *
- \*******************************************************************/
-GtkWidget *
-gnc_ui_source_menu_create(Account *account)
-{
-  gint i;
-  GtkMenu   *menu;
-  GtkWidget *item;
-  GtkWidget *omenu;
-
-  menu = GTK_MENU(gtk_menu_new());
-  gtk_widget_show(GTK_WIDGET(menu));
-
-  for (i = 0; i < NUM_SOURCES; i++)
-  {
-    item = gtk_menu_item_new_with_label(gnc_price_source_enum2name(i));
-    gtk_widget_set_sensitive(item, gnc_price_source_sensitive(i));
-    gtk_widget_show(item);
-    gtk_menu_append(menu, item);
-  }
-
-  omenu = gtk_option_menu_new();
-  gtk_widget_show(omenu);
-  gtk_option_menu_set_menu(GTK_OPTION_MENU(omenu), GTK_WIDGET(menu));
-  gnc_option_menu_init(omenu);
-
-  return omenu;
-}
-
-/********************************************************************\
- * price quote timezone handling
- */
-
-static gchar *
-known_timezones[] =
-{
-  "Asia/Tokyo",
-  "Australia/Sydney",
-  "America/New_York",
-  "America/Chicago",
-  "Europe/London",
-  "Europe/Paris",
-  NULL
-};
-
-guint
-gnc_find_timezone_menu_position(const gchar *timezone)
-{
-  /* returns 0 on failure, position otherwise. */
-  gboolean found = FALSE;
-  guint i = 0;
-  while(!found && known_timezones[i]) {
-    if(safe_strcmp(timezone, known_timezones[i]) != 0) {
-      i++;
-    } else {
-      found = TRUE;
-    }
-  }
-  if(found) return i + 1;
-  return 0;
-}
-
-gchar *
-gnc_timezone_menu_position_to_string(guint pos)
-{
-  if(pos == 0) return NULL;
-  return known_timezones[pos - 1];
-}
-
-GtkWidget *
-gnc_ui_quote_tz_menu_create(Account *account)
-{
-  GtkMenu   *menu;
-  GtkWidget *item;
-  GtkWidget *omenu;
-  gchar     **itemstr;
-
-  /* add items here as needed, but bear in mind that right now these
-     must be timezones that GNU libc understands.  Also, I'd prefer if
-     we only add things here we *know* we need.  That's because in
-     order to be portable to non GNU OSes, we may have to support
-     whatever we add here manually on those systems. */
-
-  menu = GTK_MENU(gtk_menu_new());
-  gtk_widget_show(GTK_WIDGET(menu));
-
-  item = gtk_menu_item_new_with_label(_("Use local time"));
-  /* set user data to non NULL so we can detect this item specially. */
-  gtk_object_set_user_data(GTK_OBJECT(item), (gpointer) 1);
-  gtk_widget_show(item);
-  gtk_menu_append(menu, item);
- 
-  for(itemstr = &known_timezones[0]; *itemstr; itemstr++) {
-    item = gtk_menu_item_new_with_label(*itemstr);
-    gtk_widget_show(item);
-    gtk_menu_append(menu, item);
-  }
-
-  omenu = gtk_option_menu_new();
-  gtk_widget_show(omenu);
-  gtk_option_menu_set_menu(GTK_OPTION_MENU(omenu), GTK_WIDGET(menu));
-  gnc_option_menu_init(omenu);
-
-  return omenu;
-}
+#define DESKTOP_GNOME_INTERFACE "/desktop/gnome/interface"
+#define KEY_TOOLBAR_STYLE	"toolbar_style"
 
 /* =========================================================== */
 
@@ -165,9 +62,9 @@ gnc_option_menu_cb(GtkWidget *w, gpointer data)
   gpointer _index;
   gint index;
 
-  cb = gtk_object_get_data(GTK_OBJECT(w), "gnc_option_cb");
+  cb = g_object_get_data(G_OBJECT(w), "gnc_option_cb");
 
-  _index = gtk_object_get_data(GTK_OBJECT(w), "gnc_option_index");
+  _index = g_object_get_data(G_OBJECT(w), "gnc_option_index");
   index = GPOINTER_TO_INT(_index);
 
   cb(w, index, data);
@@ -178,12 +75,12 @@ option_menu_destroy_cb (GtkObject *obj, gpointer data)
 {
   GtkTooltips *tips = data;
 
-  gtk_object_unref (GTK_OBJECT (tips));
+  g_object_unref (tips);
 }
 
 /********************************************************************\
- * gnc_ui_create_option_button                                      *
- *   create an option button given the option structure             *
+ * gnc_build_option_menu:                                           *
+ *   create an GTK "option menu" given the option structure         *
  *                                                                  *
  * Args: option_info - the option structure to use                  *
  *       num_options - the number of options                        *
@@ -206,7 +103,7 @@ gnc_build_option_menu(GNCOptionInfo *option_info, gint num_options)
 
   tooltips = gtk_tooltips_new();
 
-  gtk_object_ref (GTK_OBJECT (tooltips));
+  g_object_ref (tooltips);
   gtk_object_sink (GTK_OBJECT (tooltips));
 
   for (i = 0; i < num_options; i++)
@@ -215,82 +112,35 @@ gnc_build_option_menu(GNCOptionInfo *option_info, gint num_options)
     gtk_tooltips_set_tip(tooltips, menu_item, option_info[i].tip, NULL);
     gtk_widget_show(menu_item);
 
-    gtk_object_set_data(GTK_OBJECT(menu_item),
-                        "gnc_option_cb",
-                        option_info[i].callback);
+    g_object_set_data(G_OBJECT(menu_item),
+		      "gnc_option_cb",
+		      option_info[i].callback);
 
-    gtk_object_set_data(GTK_OBJECT(menu_item),
-                        "gnc_option_index",
-                        GINT_TO_POINTER(i));
+    g_object_set_data(G_OBJECT(menu_item),
+		      "gnc_option_index",
+		      GINT_TO_POINTER(i));
 
-    gtk_object_set_data(GTK_OBJECT(menu_item),
-                        "gnc_option_menu",
-                        omenu);
+    g_object_set_data(G_OBJECT(menu_item),
+		      "gnc_option_menu",
+		      omenu);
 
     if (option_info[i].callback != NULL)
-      gtk_signal_connect(GTK_OBJECT(menu_item), "activate",
-                         GTK_SIGNAL_FUNC(gnc_option_menu_cb),
-                         option_info[i].user_data);
+      g_signal_connect(menu_item, "activate",
+		       G_CALLBACK(gnc_option_menu_cb),
+		       option_info[i].user_data);
 
-    gtk_menu_append(GTK_MENU(menu), menu_item);
+    gtk_menu_shell_append(GTK_MENU_SHELL(menu), menu_item);
   }
 
   gtk_option_menu_set_menu(GTK_OPTION_MENU(omenu), menu);
 
-  gtk_signal_connect (GTK_OBJECT (omenu), "destroy",
-                      GTK_SIGNAL_FUNC (option_menu_destroy_cb), tooltips);
+  g_signal_connect (omenu, "destroy",
+		    G_CALLBACK (option_menu_destroy_cb), tooltips);
 
   return omenu;
 }
 
 
-/********************************************************************\
- * gnc_get_pixmap                                                   *
- *   returns a GnomePixmap widget given a pixmap filename           *
- *                                                                  *
- * Args: none                                                       *
- * Returns: GnomePixmap widget or NULL if there was a problem       *
- \*******************************************************************/
-GtkWidget *
-gnc_get_pixmap (const char *name)
-{
-  GtkWidget *pixmap;
-  char *fullname;
-
-  g_return_val_if_fail (name != NULL, NULL);
-
-  fullname = g_strconcat (GNC_PIXMAP_DIR, "/", name, NULL);
-  pixmap = gnome_pixmap_new_from_file (fullname);
-  g_free (fullname);
-
-  return pixmap;
-}
-
-
-/********************************************************************\
- * gnc_get_imlib_image                                              *
- *   returns a GdkImlibImage object given a pixmap filename         *
- *                                                                  *
- * Args: none                                                       *
- * Returns: GnomePixmap widget or NULL if there was a problem       *
- \*******************************************************************/
-GdkImlibImage *
-gnc_get_gdk_imlib_image (const char *name)
-{
-  GdkImlibImage *image;
-
-  char *fullname;
-
-  g_return_val_if_fail (name != NULL, NULL);
-
-  fullname = g_strconcat (GNC_PIXMAP_DIR, "/", name, NULL);
-  image = gdk_imlib_load_image (fullname);
-  g_free (fullname);
-
-  return image;
-}
-
-
 /********************************************************************\
  * gnc_get_toolbar_style                                            *
  *   returns the current toolbar style for gnucash toolbars         *
@@ -304,51 +154,24 @@ gnc_get_toolbar_style(void)
   GtkToolbarStyle tbstyle = GTK_TOOLBAR_BOTH;
   char *style_string;
 
-  style_string = gnc_lookup_multichoice_option("General",
-                                               "Toolbar Buttons",
-                                               "icons_and_text");
+  style_string = gnc_gconf_get_string(GCONF_GENERAL,
+				      KEY_TOOLBAR_STYLE, NULL);
+  if (!style_string || strcmp(style_string, "system") == 0) {
+    if (style_string)
+      g_free(style_string);
+    style_string = gnc_gconf_get_string(DESKTOP_GNOME_INTERFACE,
+					KEY_TOOLBAR_STYLE, NULL);
+  }
 
-  if (safe_strcmp(style_string, "icons_and_text") == 0)
-    tbstyle = GTK_TOOLBAR_BOTH;
-  else if (safe_strcmp(style_string, "icons_only") == 0)
-    tbstyle = GTK_TOOLBAR_ICONS;
-  else if (safe_strcmp(style_string, "text_only") == 0)
-    tbstyle = GTK_TOOLBAR_TEXT;
-
-  if (style_string != NULL)
-    free(style_string);
+  if (style_string == NULL)
+    return GTK_TOOLBAR_BOTH;
+  tbstyle = gnc_enum_from_nick(GTK_TYPE_TOOLBAR_STYLE, style_string,
+			       GTK_TOOLBAR_BOTH);
+  g_free(style_string);
 
   return tbstyle;
 }
 
-/********************************************************************
- * gnc_get_mdi_mode                                                 *
- * returns the current Gnome MDI mode preference                    *
- ********************************************************************/
-GnomeMDIMode 
-gnc_get_mdi_mode(void) {
-  GnomeMDIMode mode = GNOME_MDI_DEFAULT_MODE;
-  char * mode_string = gnc_lookup_multichoice_option("_+Advanced",
-                                                     "Application MDI mode",
-                                                     "");
-  if(!safe_strcmp(mode_string, "mdi-notebook")) {
-    mode = GNOME_MDI_NOTEBOOK;
-  }
-  else if(!safe_strcmp(mode_string, "mdi-toplevel")) {
-    mode = GNOME_MDI_TOPLEVEL;    
-  }
-  else if(!safe_strcmp(mode_string, "mdi-modal")) {
-    mode = GNOME_MDI_MODAL;    
-  }
-  else if(!safe_strcmp(mode_string, "mdi-default")) {
-    mode = GNOME_MDI_DEFAULT_MODE;    
-  }
-
-  if(mode_string) free(mode_string);
-  return mode;
-}
-
-
 /********************************************************************\
  * gnc_get_deficit_color                                            *
  *   fill in the 3 color values for the color of deficit values     *
@@ -380,7 +203,7 @@ gnc_set_label_color(GtkWidget *label, gnc_numeric value)
   GdkColormap *cm;
   GtkStyle *style;
 
-  if (!gnc_color_deficits())
+  if (!gnc_gconf_get_bool(GCONF_GENERAL, "red_for_negative", NULL))
     return;
 
   cm = gtk_widget_get_colormap(GTK_WIDGET(label));
@@ -401,12 +224,12 @@ gnc_set_label_color(GtkWidget *label, gnc_numeric value)
 
   gtk_widget_set_style(label, style);
 
-  gtk_style_unref(style);
+  g_object_unref(style);
 }
 
 
 /********************************************************************\
- * gnc_get_window_size                                              *
+ * gnc_restore_window_size                                          *
  *   returns the window size to use for the given option prefix,    *
  *   if window sizes are being saved, otherwise returns 0 for both. *
  *                                                                  *
@@ -416,32 +239,35 @@ gnc_set_label_color(GtkWidget *label, gnc_numeric value)
  * Returns: nothing                                                 *
  \*******************************************************************/
 void
-gnc_get_window_size(const char *prefix, int *width, int *height)
+gnc_restore_window_size(const char *section, GtkWindow *window)
 {
-  int w, h;
-  char *name;
+  GSList *coord_list;
+  gint coords[2];
 
-  if (gnc_lookup_boolean_option("_+Advanced", "Save Window Geometry", TRUE))
-  {
-    name = g_strconcat(prefix, "_width", NULL);
-    w = gnc_lookup_number_option("__gui", name, 0.0);
-    g_free(name);
+  g_return_if_fail(section != NULL);
+  g_return_if_fail(window != NULL);
 
-    name = g_strconcat(prefix, "_height", NULL);
-    h = gnc_lookup_number_option("__gui", name, 0.0);
-    g_free(name);
-  }
-  else
-  {
-    w = 0;
-    h = 0;
+  if (!gnc_gconf_get_bool(GCONF_GENERAL, KEY_SAVE_GEOMETRY, NULL))
+    return;
+  
+  coord_list = gnc_gconf_get_list(section, WINDOW_POSITION,
+				  GCONF_VALUE_INT, NULL);
+  if (coord_list) {
+    coords[0] = GPOINTER_TO_INT(g_slist_nth_data(coord_list, 0));
+    coords[1] = GPOINTER_TO_INT(g_slist_nth_data(coord_list, 1));
+    gtk_window_move(window, coords[0], coords[1]);
+    g_slist_free(coord_list);
   }
 
-  if (width != NULL)
-    *width = w;
-
-  if (height != NULL)
-    *height = h;
+  coord_list = gnc_gconf_get_list(section, WINDOW_GEOMETRY,
+				  GCONF_VALUE_INT, NULL);
+  if (coord_list) {
+    coords[0] = GPOINTER_TO_INT(g_slist_nth_data(coord_list, 0));
+    coords[1] = GPOINTER_TO_INT(g_slist_nth_data(coord_list, 1));
+    if ((coords[0] != 0) && (coords[1] != 0))
+      gtk_window_resize(window, coords[0], coords[1]);
+    g_slist_free(coord_list);
+  }
 }
 
 
@@ -456,62 +282,34 @@ gnc_get_window_size(const char *prefix, int *width, int *height)
  * Returns: nothing                                                 *
 \********************************************************************/
 void
-gnc_save_window_size(const char *prefix, int width, int height)
+gnc_save_window_size(const char *section, GtkWindow *window)
 {
-  char *name;
-  gboolean save;
+  GSList *coord_list = NULL;
+  gint coords[2];
 
-  save = gnc_lookup_boolean_option("_+Advanced", "Save Window Geometry", FALSE);
+  g_return_if_fail(section != NULL);
+  g_return_if_fail(window != NULL);
 
-  name = g_strconcat(prefix, "_width", NULL);
-  if (save)
-    gnc_set_number_option("__gui", name, width);
-  else
-    gnc_set_option_default("__gui", name);
-  g_free(name);
-
-  name = g_strconcat(prefix, "_height", NULL);
-  if (save)
-    gnc_set_number_option("__gui", name, height);
-  else
-    gnc_set_option_default("__gui", name);
-  g_free(name);
-}
-
-
-/********************************************************************\
- * gnc_fill_menu_with_data                                          *
- *   fill the user data values in the menu structure with the given *
- *   value. The filling is done recursively.                        *
- *                                                                  *
- * Args: info - the menu to fill                                    *
- *       data - the value to fill with                              *
- * Returns: nothing                                                 *
-\********************************************************************/
-void
-gnc_fill_menu_with_data(GnomeUIInfo *info, gpointer data)
-{
-  if (info == NULL)
+  if (GTK_OBJECT_FLAGS(window) & GTK_IN_DESTRUCTION)
     return;
 
-  while (1)
-  {
-    switch (info->type)
-    {
-      case GNOME_APP_UI_RADIOITEMS:
-      case GNOME_APP_UI_SUBTREE:
-      case GNOME_APP_UI_SUBTREE_STOCK:
-        gnc_fill_menu_with_data((GnomeUIInfo *) info->moreinfo, data);
-        break;
-      case GNOME_APP_UI_ENDOFINFO:
-        return;
-      default:
-        info->user_data = data;
-        break;
-    }
+  if (!gnc_gconf_get_bool(GCONF_GENERAL, KEY_SAVE_GEOMETRY, NULL))
+    return;
 
-    info++;
-  }
+  gtk_window_get_size(GTK_WINDOW(window), &coords[0], &coords[1]);
+  coord_list = g_slist_append(coord_list, GUINT_TO_POINTER(coords[0]));
+  coord_list = g_slist_append(coord_list, GUINT_TO_POINTER(coords[1]));
+  gnc_gconf_set_list(section, WINDOW_GEOMETRY, GCONF_VALUE_INT,
+		     coord_list, NULL);
+  g_slist_free(coord_list);
+  coord_list = NULL;
+
+  gtk_window_get_position(GTK_WINDOW(window), &coords[0], &coords[1]);
+  coord_list = g_slist_append(coord_list, GUINT_TO_POINTER(coords[0]));
+  coord_list = g_slist_append(coord_list, GUINT_TO_POINTER(coords[1]));
+  gnc_gconf_set_list(section, WINDOW_POSITION, GCONF_VALUE_INT,
+		     coord_list, NULL);
+  g_slist_free(coord_list);
 }
 
 
@@ -528,9 +326,9 @@ gnc_option_menu_init(GtkWidget * w)
   {
     gtk_option_menu_set_history(GTK_OPTION_MENU(w), i);
     active = gtk_menu_get_active(GTK_MENU(menu));
-    gtk_object_set_data(GTK_OBJECT(active), 
-                        "option_index",
-                        GINT_TO_POINTER(i));
+    g_object_set_data(G_OBJECT(active), 
+		      "option_index",
+		      GINT_TO_POINTER(i));
   }
 
   gtk_option_menu_set_history(GTK_OPTION_MENU(w), 0);
@@ -538,23 +336,23 @@ gnc_option_menu_init(GtkWidget * w)
 
 typedef struct {
   int i;
-  GtkSignalFunc f;
+  GCallback f;
   gpointer cb_data;
 } menu_init_data;
 
 static void
 gnc_option_menu_set_one_item (gpointer loop_data, gpointer user_data)
 {
-  GtkObject *item = GTK_OBJECT(loop_data);
+  GObject *item = G_OBJECT(loop_data);
   menu_init_data *args = (menu_init_data *) user_data;
   
-  gtk_object_set_data(item, "option_index", GINT_TO_POINTER(args->i++));
-  gtk_signal_connect(item, "activate", args->f, args->cb_data);
+  g_object_set_data(item, "option_index", GINT_TO_POINTER(args->i++));
+  g_signal_connect(item, "activate", args->f, args->cb_data);
 }
 
 
 void
-gnc_option_menu_init_w_signal(GtkWidget * w, GtkSignalFunc f, gpointer cb_data)
+gnc_option_menu_init_w_signal(GtkWidget * w, GCallback f, gpointer cb_data)
 {
   GtkWidget * menu;
   menu_init_data foo;
@@ -579,8 +377,8 @@ gnc_option_menu_get_active(GtkWidget * w)
   menu     = gtk_option_menu_get_menu(GTK_OPTION_MENU(w));
   menuitem = gtk_menu_get_active(GTK_MENU(menu));
 
-  return GPOINTER_TO_INT(gtk_object_get_data(GTK_OBJECT(menuitem),
-                                             "option_index"));
+  return GPOINTER_TO_INT(g_object_get_data(G_OBJECT(menuitem),
+					   "option_index"));
 }
 
 
@@ -608,7 +406,7 @@ gnc_window_adjust_for_screen(GtkWindow * window)
 
   screen_width = gdk_screen_width();
   screen_height = gdk_screen_height();
-  gdk_window_get_size(GTK_WIDGET(window)->window, &width, &height);
+  gdk_drawable_get_size(GTK_WIDGET(window)->window, &width, &height);
 
   if ((width <= screen_width) && (height <= screen_height))
     return;
@@ -623,29 +421,6 @@ gnc_window_adjust_for_screen(GtkWindow * window)
   gtk_widget_queue_resize(GTK_WIDGET(window));
 }
 
-/*
- * This routine must be removed when GnuCash is ported to GTK 2.0.  It
- * replicates the functionality (as much as possible) of that routine
- * using functions available in GTK 1.4.
- */
-void
-gtk_window_present (GtkWindow *window)
-{
-  GtkWidget *widget;
-
-  g_return_if_fail (GTK_IS_WINDOW (window));
-
-  widget = GTK_WIDGET (window);
-
-  if (GTK_WIDGET_VISIBLE (window)) {
-      g_assert (widget->window != NULL);
-      gdk_window_show(widget->window); 	/* De-iconify */
-      gdk_window_raise(widget->window);	/* Bring to front */
-  } else {
-      gtk_widget_show (widget);
-  }
-}
-
 gboolean
 gnc_handle_date_accelerator (GdkEventKey *event,
                              struct tm *tm,
@@ -689,33 +464,28 @@ gnc_handle_date_accelerator (GdkEventKey *event,
       return TRUE;
 
     case GDK_minus:
+    case GDK_KP_Subtract:
+    case GDK_underscore:
       if ((strlen (date_str) != 0) && (dateSeparator () == '-'))
       {
-        GdkWChar *wcs;
-        int count;
-        int len;
-        int i;
-
-        len = gnc_mbstowcs (&wcs, date_str);
-        if (len < 0)
-          return FALSE;
+        const char *c;
+        gunichar uc;
+        int count = 0;
 
         /* rough check for existing date */
-        for (i = count = 0; i < len; i++)
+        c = date_str;
+        while (*c)
         {
-          if (wcs[i] == '-')
+          uc = g_utf8_get_char (c);
+          if (uc == '-')
             count++;
+          c = g_utf8_next_char (c);          
         }
 
-        g_free (wcs);
-
         if (count < 2)
           return FALSE;
       }
 
-      /* fall through */
-    case GDK_KP_Subtract:
-    case GDK_underscore:
       if (event->state & GDK_SHIFT_MASK)
         g_date_subtract_days (&gdate, 7);
       else if (event->state & GDK_MOD1_MASK)
@@ -792,7 +562,7 @@ gnc_handle_date_accelerator (GdkEventKey *event,
         GTime gtime;
 
         gtime = time (NULL);
-        g_date_set_time (&gdate, gtime);
+        g_date_set_time_t (&gdate, gtime);
         break;
       }
 
@@ -846,13 +616,15 @@ check_realize (GtkWidget *widget, gpointer user_data)
   GList *list;
   GList *node;
   GdkGC *gc;
+  GdkFont *font;
 
   if (check_info->mask)
     return;
 
   style = gtk_widget_get_style (widget);
+  font = gdk_font_from_description(style->font_desc);
 
-  font_height = style->font->ascent + style->font->descent;
+  font_height = font->ascent + font->descent;
   check_size = (font_height > 0) ? font_height - 3 : 9;
 
   check_info->mask = gdk_pixmap_new (NULL, check_size, check_size, 1);
@@ -903,7 +675,7 @@ check_realize (GtkWidget *widget, gpointer user_data)
                  check_size / 3, check_size - 4,
                  check_size - 3, 1);
 
-  gdk_gc_unref (gc);
+  g_object_unref (gc);
 
   clist = GTK_CLIST (widget);
 
@@ -929,13 +701,13 @@ check_unrealize (GtkWidget *widget, gpointer user_data)
   GNCCListCheckInfo *check_info = user_data;
 
   if (check_info->mask)
-    gdk_bitmap_unref (check_info->mask);
+    g_object_unref (check_info->mask);
 
   if (check_info->on_pixmap)
-    gdk_pixmap_unref (check_info->on_pixmap);
+    g_object_unref (check_info->on_pixmap);
 
   if (check_info->off_pixmap)
-    gdk_pixmap_unref (check_info->off_pixmap);
+    g_object_unref (check_info->off_pixmap);
 
   check_info->mask = NULL;
   check_info->on_pixmap = NULL;
@@ -957,11 +729,11 @@ static GNCCListCheckInfo *
 gnc_clist_add_check (GtkCList *list)
 {
   GNCCListCheckInfo *check_info;
-  GtkObject *object;
+  GObject *object;
 
-  object = GTK_OBJECT (list);
+  object = G_OBJECT (list);
 
-  check_info = gtk_object_get_data (object, "gnc-check-info");
+  check_info = g_object_get_data (object, "gnc-check-info");
   if (check_info)
   {
     PWARN ("clist already has check");
@@ -970,14 +742,14 @@ gnc_clist_add_check (GtkCList *list)
 
   check_info = g_new0 (GNCCListCheckInfo, 1);
 
-  gtk_object_set_data (object, "gnc-check-info", check_info);
+  g_object_set_data (object, "gnc-check-info", check_info);
 
-  gtk_signal_connect (object, "realize",
-                      GTK_SIGNAL_FUNC (check_realize), check_info);
-  gtk_signal_connect (object, "unrealize",
-                      GTK_SIGNAL_FUNC (check_unrealize), check_info);
-  gtk_signal_connect (object, "destroy",
-                      GTK_SIGNAL_FUNC (check_destroy), check_info);
+  g_signal_connect (object, "realize",
+		    G_CALLBACK (check_realize), check_info);
+  g_signal_connect (object, "unrealize",
+		    G_CALLBACK (check_unrealize), check_info);
+  g_signal_connect (object, "destroy",
+		    G_CALLBACK (check_destroy), check_info);
 
   if (GTK_WIDGET_REALIZED (GTK_WIDGET (list)))
     check_realize (GTK_WIDGET (list), check_info);
@@ -994,7 +766,7 @@ gnc_clist_set_check (GtkCList *list, int row, int col, gboolean checked)
 
   g_return_if_fail (GTK_IS_CLIST (list));
 
-  check_info = gtk_object_get_data (GTK_OBJECT (list), "gnc-check-info");
+  check_info = g_object_get_data (G_OBJECT (list), "gnc-check-info");
   if (!check_info)
     check_info = gnc_clist_add_check (list);
 
@@ -1022,49 +794,17 @@ gnc_clist_set_check (GtkCList *list, int row, int col, gboolean checked)
     gtk_clist_set_text (list, row, col, "");
 }
 
-void
-gnc_clist_columns_autosize (GtkCList *list)
-{
-  GtkStyle *style;
-  GdkFont *font;
-  gint i;
-
-  if (!list) return;
-  g_return_if_fail (GTK_IS_CLIST (list));
-
-  style = gtk_widget_get_style (GTK_WIDGET(list));
-  if (!style)
-    return;
-
-  font = style->font;
-  if (!font)
-    return;
-
-  for (i = 0; TRUE; i++)
-  {
-    GtkWidget *widget;
-    char *title;
-    gint width;
-
-    widget = gtk_clist_get_column_widget (list, i);
-    if (!widget)
-      break;
-
-    if (!GTK_IS_LABEL (widget))
-      continue;
-
-    gtk_label_get (GTK_LABEL (widget), &title);
-
-    width = gdk_string_width (font, title);
-    gtk_clist_set_column_min_width (list, i, width + 5);
-  }
-
-  gtk_clist_columns_autosize (list);
-}
-
+/*   Glade Stuff
+ *
+ *
+ */
 
 static gboolean glade_inited = FALSE;
 
+/* gnc_glade_xml_new: a convenience wrapper for glade_xml_new
+ *   - takes care of glade initialization, if needed
+ *   - takes care of finding the directory for glade files
+ */
 GladeXML *
 gnc_glade_xml_new (const char *filename, const char *root)
 {
@@ -1080,26 +820,34 @@ gnc_glade_xml_new (const char *filename, const char *root)
     glade_inited = TRUE;
   }
 
-  fname = g_strconcat (GNC_GLADE_DIR, "/", filename, NULL);
+  fname = g_strconcat (GNC_GLADE_DIR, "/", filename, (char *)NULL);
 
-  xml = glade_xml_new (fname, root);
+  xml = glade_xml_new (fname, root, NULL);
 
   g_free (fname);
 
   return xml;
 }
 
+/* gnc_glade_lookup_widget:  Given a root (or at least ancestor) widget,
+ *   find the child widget with the given name.
+ */
 GtkWidget *
 gnc_glade_lookup_widget (GtkWidget *widget, const char *name)
 {
   GladeXML *xml;
+  GtkWidget *wid;
 
   if (!widget || !name) return NULL;
 
   xml = glade_get_widget_tree (widget);
   if (!xml) return NULL;
+  
+  wid = glade_xml_get_widget (xml, name);
+  if (!wid)
+      PWARN("I know nothing of this '%s' whom you seek.", name);
 
-  return glade_xml_get_widget (xml, name);
+  return wid;
 }
 
 /*
@@ -1109,15 +857,15 @@ GModule *allsymbols = NULL;
 
 void
 gnc_glade_autoconnect_full_func(const gchar *handler_name,
-				GtkObject *signal_object,
+				GObject *signal_object,
 				const gchar *signal_name,
 				const gchar *signal_data,
-				GtkObject *other_object,
+				GObject *other_object,
 				gboolean signal_after,
 				gpointer user_data)
 {
-  GtkSignalFunc func;
-  GtkSignalFunc *p_func = &func;
+  GCallback func;
+  GCallback *p_func = &func;
 
   if (allsymbols == NULL) {
     /* get a handle on the main executable -- use this to find symbols */
@@ -1125,8 +873,12 @@ gnc_glade_autoconnect_full_func(const gchar *handler_name,
   }
 
   if (!g_module_symbol(allsymbols, handler_name, (gpointer *)p_func)) {
+#ifdef HAVE_DLSYM
     /* Fallback to dlsym -- necessary for *BSD linkers */
     func = dlsym(RTLD_DEFAULT, handler_name);
+#else
+    func = NULL;
+#endif
     if (func == NULL) {
       g_warning("ggaff: could not find signal handler '%s'.", handler_name);
       return;
@@ -1135,98 +887,157 @@ gnc_glade_autoconnect_full_func(const gchar *handler_name,
 
   if (other_object) {
     if (signal_after)
-      gtk_signal_connect_object_after(signal_object, signal_name, func,
-				      other_object);
+      g_signal_connect_object (signal_object, signal_name, func,
+			       other_object, G_CONNECT_AFTER);
     else
-      gtk_signal_connect_object(signal_object, signal_name, func,
-				other_object);
+      g_signal_connect_object (signal_object, signal_name, func, other_object, 0);
   } else {
     if (signal_after)
-      gtk_signal_connect_after(signal_object, signal_name, func, user_data);
+      g_signal_connect_after(signal_object, signal_name, func, user_data);
     else
-      gtk_signal_connect(signal_object, signal_name, func, user_data);
+      g_signal_connect(signal_object, signal_name, func, user_data);
   }
 }
 
-
-gint
-gnc_mbstowcs (GdkWChar **dest_p, const char *src)
+void
+gnc_gtk_dialog_add_button (GtkWidget *dialog, const gchar *label, const gchar *stock_id, guint response)
 {
-  GdkWChar *dest;
-  gint src_len;
-  gint retval;
+  GtkWidget *button;
 
-  if (!src)
-    return -1;
+  button = gtk_button_new_with_label(label);
+#ifdef HAVE_GTK26
+  if (stock_id) {
+    GtkWidget *image;
 
-  src_len = strlen (src);
-
-  dest = g_new0 (GdkWChar, src_len + 1);
-
-  retval = gdk_mbstowcs (dest, src, src_len);
-
-  if (retval < 0)
-  {
-    PERR ("bad multi-byte conversion");
+    image = gtk_image_new_from_stock (stock_id, GTK_ICON_SIZE_BUTTON);
+    gtk_button_set_image(GTK_BUTTON(button), image);
   }
-
-  if (dest_p)
-    *dest_p = dest;
-  else
-    g_free (dest);
-
-  return retval;
+#else
+  gtk_button_set_use_underline(GTK_BUTTON(button), TRUE);
+#endif
+  gtk_widget_show_all(button);
+  gtk_dialog_add_action_widget(GTK_DIALOG(dialog), button, response);
 }
 
-char *
-gnc_wcstombs (const GdkWChar *src)
+static void
+gnc_perm_button_cb (GtkButton *perm, gpointer user_data)
 {
-  char *retval;
+  gboolean perm_active;
 
-  if (!src)
-    return NULL;
-
-  retval = gdk_wcstombs (src);
-  if (!retval)
-  {
-    PERR ("bad multi-byte conversion");
-  }
-
-  return retval;
+  perm_active = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(perm));
+  gtk_widget_set_sensitive(user_data, !perm_active);
 }
 
 gint
-gnc_wcslen (const GdkWChar *src)
+gnc_dialog_run (GtkDialog *dialog, const gchar *gconf_key)
 {
-  int len = 0;
+    GtkWidget *perm, *temp;
+    gboolean ask = TRUE;
+    gint response;
 
-  if (!src)
-    return 0;
+    /* Does the user want to see this question? If not, return the
+     * previous answer. */
+    response = gnc_gconf_get_int(GCONF_WARNINGS_PERM, gconf_key, NULL);
+    if (response != 0)
+      return response;
+    response = gnc_gconf_get_int(GCONF_WARNINGS_TEMP, gconf_key, NULL);
+    if (response != 0)
+      return response;
 
-  while (src[len])
-    len++;
+    /* Add in the checkboxes to find out if the answer should be remembered. */
+#if 0
+    if (GTK_IS_MESSAGE_DIALOG(dialog)) {
+      GtkMessageType type;
+      g_object_get(dialog, "message-type", &type, (gchar*)NULL);
+      ask = (type == GTK_MESSAGE_QUESTION);
+    } else {
+      ask = FALSE;
+    }
+#endif
+    perm = gtk_check_button_new_with_mnemonic
+      (ask
+       ? _("Remember and don't _ask me again.")
+       : _("Don't _tell me again."));
+    temp = gtk_check_button_new_with_mnemonic
+      (ask
+       ? _("Remember and don't ask me again this _session.")
+       : _("Don't tell me again this _session."));
+    gtk_widget_show(perm);
+    gtk_widget_show(temp);
+    gtk_box_pack_start_defaults(GTK_BOX(dialog->vbox), perm);
+    gtk_box_pack_start_defaults(GTK_BOX(dialog->vbox), temp);
+    g_signal_connect(perm, "clicked", G_CALLBACK(gnc_perm_button_cb), temp);
 
-  return len;
+    /* OK. Present the dialog. */
+    response = gtk_dialog_run(dialog);
+    if ((response == GTK_RESPONSE_NONE) || (response == GTK_RESPONSE_DELETE_EVENT)) {
+      return GTK_RESPONSE_NO;
+    }
+
+    /* Save the answer? */
+    if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(perm))) {
+      gnc_gconf_set_int(GCONF_WARNINGS_PERM, gconf_key, response, NULL);
+    } else if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(temp))) {
+      gnc_gconf_set_int(GCONF_WARNINGS_TEMP, gconf_key, response, NULL);
+    }
+
+    return response;
 }
 
-GdkWChar *
-gnc_wcsdup (const GdkWChar *src)
+#ifndef HAVE_GTK26
+
+/** Find the first GtkLabel in a container. When called on a gtk2.4
+ *  message dialog, there is only one label in the dialog so theis
+ *  should return it. */
+static void
+find_label (GtkWidget *widget, gpointer data)
 {
-  GdkWChar *dest;
-  int len;
-  int i;
+  GtkWidget **label = data;
 
-  if (!src)
-    return NULL;
+  if (*label)
+    return;
 
-  len = gnc_wcslen (src);
+  if (GTK_IS_LABEL(widget)) {
+    *label = widget;
+    return;
+  }
 
-  dest = g_new (GdkWChar, len + 1);
-
-  for (i = 0; i < len; i++)
-    dest[i] = src[i];
-
-  dest[len] = 0;
-
-  return dest;
+  if (GTK_IS_CONTAINER(widget)) {
+    gtk_container_foreach(GTK_CONTAINER(widget), find_label, data);
+  }
 }
+
+/** Mimic the gtk2.6 function to add secondary information to a
+ *  message dialog. */
+void
+gtk_message_dialog_format_secondary_text(GtkMessageDialog *dialog,
+					 const gchar *format,
+					 ...)
+{
+  GtkWidget *label = NULL;
+  const gchar *current;
+  gchar *primary, *secondary;
+  va_list args;
+
+  gtk_container_foreach(GTK_CONTAINER(GTK_DIALOG(dialog)->vbox),
+			find_label, &label);
+  if (!label)
+    return;
+
+  /* Get the current markup. */
+  current = gtk_label_get_label(GTK_LABEL(label));
+
+  /* Format the text to be added. */
+  va_start(args, format);
+  secondary = g_strdup_vprintf(format, args);
+  va_end(args);
+
+  /* Append the two strings, making the first one bold. */
+  primary = g_strdup_printf("<b>%s</b>\n\n%s", current, secondary);
+  gtk_label_set_markup(GTK_LABEL(label), primary);
+
+  g_free(primary);
+  g_free(secondary);
+}
+
+#endif
