@@ -16,25 +16,30 @@
  * along with this program; if not, contact:                        *
  *                                                                  *
  * Free Software Foundation           Voice:  +1-617-542-5942       *
- * 59 Temple Place - Suite 330        Fax:    +1-617-542-2652       *
- * Boston, MA  02111-1307,  USA       gnu@gnu.org                   *
+ * 51 Franklin Street, Fifth Floor    Fax:    +1-617-542-2652       *
+ * Boston, MA  02110-1301,  USA       gnu@gnu.org                   *
 \********************************************************************/
 
-#define _GNU_SOURCE
-
 #include "config.h"
 
-#include <gnome.h>
+#include <gtk/gtk.h>
+#include <glib/gi18n.h>
 #include <stdio.h>
 
 #include "gnc-ui-util.h"
 #include "Query.h"
+#include "QueryCore.h"
+#include "QueryNew.h"
+#include "QueryObject.h"
+#include "SX-book.h"
 #include "Transaction.h"
 #include "dialog-find-transactions.h"
-#include "window-register.h"
-#include "messages.h"
+#include "gnc-main-window.h"
+#include "gnc-plugin-page-register.h"
 #include "search-param.h"
 
+#define GCONF_SECTION "dialogs/find"
+
 struct _ftd_data {
   QueryNew *		q;
   QueryNew *		ledger_q;
@@ -47,6 +52,7 @@ do_find_cb (QueryNew *query, gpointer user_data, gpointer *result)
   struct _ftd_data *ftd = user_data;
   GNCLedgerDisplay *ledger;
   gboolean new_ledger = FALSE;
+  GncPluginPage *page;
 
   ledger = gnc_ledger_display_find_by_query (ftd->ledger_q);
   if(!ledger) {
@@ -59,8 +65,10 @@ do_find_cb (QueryNew *query, gpointer user_data, gpointer *result)
 
   gnc_ledger_display_refresh (ledger);
 
-  if (new_ledger)
-    regWindowLedger(ledger);
+  if (new_ledger) {
+    page = gnc_plugin_page_register_new_ledger (ledger);
+    gnc_main_window_open_page (NULL, page);
+  }
 
   gncQueryDestroy (ftd->q);
 
@@ -158,9 +166,10 @@ gnc_ui_find_transactions_dialog_create(GNCLedgerDisplay * orig_ledg)
     ftd->q = start_q;		/* save this to destroy it later */
   }
 
-  ftd->sw = gnc_search_dialog_create (type, params, NULL, start_q, show_q,
+  ftd->sw = gnc_search_dialog_create (type, _("Find Transaction"),
+				      params, NULL, start_q, show_q,
 				      NULL, do_find_cb, NULL,
-				      ftd, free_ftd_cb);
+				      ftd, free_ftd_cb, GCONF_SECTION, NULL);
 
   if (!ftd->sw) {
     free_ftd_cb (ftd);
