@@ -17,24 +17,27 @@
  * along with this program; if not, contact:                        *
  *                                                                  *
  * Free Software Foundation           Voice:  +1-617-542-5942       *
- * 59 Temple Place - Suite 330        Fax:    +1-617-542-2652       *
- * Boston, MA  02111-1307,  USA       gnu@gnu.org                   *
+ * 51 Franklin Street, Fifth Floor    Fax:    +1-617-542-2652       *
+ * Boston, MA  02110-1301,  USA       gnu@gnu.org                   *
 \********************************************************************/
 
 #include "config.h"
 
-#include <gnome.h>
-#include <guile/gh.h>
+#include <gtk/gtk.h>
+#include <glib/gi18n.h>
+#include <libguile.h>
 
+#include "Account.h"
+#include "gnc-ui-util.h"
 #include "dialog-utils.h"
-#include "gnc-account-tree.h"
+#include "gnc-tree-view-account.h"
 #include "gnc-component-manager.h"
-#include "gnc-engine-util.h"
+#include "qof.h"
 #include "gnc-ui.h"
-#include "messages.h"
 
 
 #define DIALOG_TAX_INFO_CM_CLASS "dialog-tax-info"
+#define GCONF_SECTION "dialogs/tax_info"
 
 /* This static indicates the debugging module that this .o belongs to.  */
 /* static short module = MOD_GUI; */
@@ -62,10 +65,11 @@ typedef struct
 {
   GtkWidget * dialog;
 
-  GtkWidget * account_tree;
+  GtkWidget * account_treeview;
+  GtkWidget * select_button;
 
   GtkWidget * tax_related_button;
-  GtkWidget * txf_category_clist;
+  GtkWidget * txf_category_view;
   GtkWidget * txf_help_text;
   GtkWidget * current_account_button;
   GtkWidget * parent_account_button;
@@ -75,12 +79,12 @@ typedef struct
 
   gboolean income;
   gboolean changed;
+
+  GNCAccountType account_type;
 } TaxInfoDialog;
 
 
 static gboolean getters_initialized = FALSE;
-static gint last_width = 0;
-static gint last_height = 0;
 
 
 static void
@@ -89,12 +93,12 @@ initialize_getters (void)
   if (getters_initialized)
     return;
 
-  getters.payer_name_source = gh_eval_str ("gnc:txf-get-payer-name-source");
-  getters.form              = gh_eval_str ("gnc:txf-get-form");
-  getters.description       = gh_eval_str ("gnc:txf-get-description");
-  getters.help              = gh_eval_str ("gnc:txf-get-help");
+  getters.payer_name_source = scm_c_eval_string ("gnc:txf-get-payer-name-source");
+  getters.form              = scm_c_eval_string ("gnc:txf-get-form");
+  getters.description       = scm_c_eval_string ("gnc:txf-get-description");
+  getters.help              = scm_c_eval_string ("gnc:txf-get-help");
 
-  getters.codes             = gh_eval_str ("gnc:txf-get-codes");
+  getters.codes             = scm_c_eval_string ("gnc:txf-get-codes");
 
   getters_initialized = TRUE;
 }
@@ -132,11 +136,6 @@ destroy_txf_infos (GList *infos)
 static void
 gnc_tax_info_set_changed (TaxInfoDialog *ti_dialog, gboolean changed)
 {
-  GtkWidget *button;
-
-  button = gnc_glade_lookup_widget (ti_dialog->dialog, "apply_button");
-  gtk_widget_set_sensitive (button, changed);
-
   ti_dialog->changed = changed;
 }
 
@@ -149,37 +148,36 @@ load_txf_info (gboolean income)
 
   initialize_getters ();
 
-  category = gh_eval_str (income ?
-                          "txf-income-categories" :
-                          "txf-expense-categories");
+  category = scm_c_eval_string (income ?
+				"txf-income-categories" :
+				"txf-expense-categories");
   if (category == SCM_UNDEFINED)
   {
     destroy_txf_infos (infos);
     return NULL;
   }
 
-  codes = gh_call1 (getters.codes, category);
-  if (!gh_list_p (codes))
+  codes = scm_call_1 (getters.codes, category);
+  if (!SCM_LISTP (codes))
   {
     destroy_txf_infos (infos);
     return NULL;
   }
 
-  while (!gh_null_p (codes))
+  while (!SCM_NULLP (codes))
   {
     TXFInfo *txf_info;
     SCM code_scm;
-    char *str;
+    const gchar *str;
     SCM scm;
 
-    code_scm  = gh_car (codes);
-    codes     = gh_cdr (codes);
+    code_scm  = SCM_CAR (codes);
+    codes     = SCM_CDR (codes);
 
-    scm = gh_call2 (getters.payer_name_source, category, code_scm);
-    str = gh_symbol2newstr (scm, NULL);
+    scm = scm_call_2 (getters.payer_name_source, category, code_scm);
+    str = SCM_SYMBOL_CHARS (scm);
     if (safe_strcmp (str, "not-impl") == 0)
     {
-      free (str);
       continue;
     }
 
@@ -189,26 +187,21 @@ load_txf_info (gboolean income)
       txf_info->payer_name_source = NULL;
     else
       txf_info->payer_name_source = g_strdup (str);
-    free (str);
 
-    str = gh_symbol2newstr (code_scm, NULL);
+    str = SCM_SYMBOLP(code_scm) ? SCM_SYMBOL_CHARS(code_scm) : "";
     txf_info->code = g_strdup (str);
-    free (str);
 
-    scm = gh_call2 (getters.form, category, code_scm);
-    str = gh_scm2newstr (scm, NULL);
+    scm = scm_call_2 (getters.form, category, code_scm);
+    str = SCM_STRINGP(scm) ? SCM_STRING_CHARS(scm) : "";
     txf_info->form = g_strdup (str);
-    free (str);
 
-    scm = gh_call2 (getters.description, category, code_scm);
-    str = gh_scm2newstr (scm, NULL);
+    scm = scm_call_2 (getters.description, category, code_scm);
+    str = SCM_STRINGP(scm) ? SCM_STRING_CHARS(scm) : "";
     txf_info->description = g_strdup (str);
-    free (str);
 
-    scm = gh_call2 (getters.help, category, code_scm);
-    str = gh_scm2newstr (scm, NULL);
+    scm = scm_call_2 (getters.help, category, code_scm);
+    str = SCM_STRINGP(scm) ? SCM_STRING_CHARS(scm) : "";
     txf_info->help = g_strdup (str);
-    free (str);
 
     infos = g_list_prepend (infos, txf_info);
   }
@@ -227,40 +220,58 @@ tax_infos (TaxInfoDialog *ti_dialog)
 static void
 load_category_list (TaxInfoDialog *ti_dialog)
 {
-  GtkCList *clist = GTK_CLIST (ti_dialog->txf_category_clist);
-  char *text[2];
+  GtkTreeView *view;
+  GtkListStore *store;
+  GtkTreeIter iter;
   GList *codes;
 
-  gtk_clist_freeze (clist);
-  gtk_clist_clear (clist);
+  view = GTK_TREE_VIEW(ti_dialog->txf_category_view);
+  store = GTK_LIST_STORE(gtk_tree_view_get_model(view));
+  gtk_tree_view_set_model(view, NULL);
+
+  gtk_list_store_clear(store);
 
   codes = tax_infos (ti_dialog);
-
   for ( ; codes; codes = codes->next)
   {
     TXFInfo *txf_info = codes->data;
 
-    text[0] = txf_info->form;
-    text[1] = txf_info->description;
-
-    gtk_clist_append (clist, text);
+    gtk_list_store_append(store, &iter);
+    gtk_list_store_set(store, &iter,
+		       0, txf_info->form,
+		       1, txf_info->description,
+		       -1);
   }
 
-  gtk_clist_thaw (clist);
+  gtk_tree_view_set_model(view, GTK_TREE_MODEL(store));
 }
 
 static void
 clear_gui (TaxInfoDialog *ti_dialog)
 {
+  GtkTreeView *view;
+  GtkTreeSelection *selection;
+
   gtk_toggle_button_set_active
     (GTK_TOGGLE_BUTTON (ti_dialog->tax_related_button), FALSE);
 
-  gtk_clist_select_row (GTK_CLIST (ti_dialog->txf_category_clist), 0, 0);
+  view = GTK_TREE_VIEW(ti_dialog->txf_category_view);
+  selection = gtk_tree_view_get_selection(view);
+  gtk_tree_selection_unselect_all(selection);
 
   gtk_toggle_button_set_active
     (GTK_TOGGLE_BUTTON (ti_dialog->current_account_button), TRUE);
 }
 
+static gboolean
+gnc_tax_info_dialog_account_filter_func (Account *account,
+					 gpointer data)
+{
+  TaxInfoDialog *dialog = data;
+
+  return xaccAccountGetType (account) == dialog->account_type;
+}
+
 static TXFInfo *
 txf_infos_find_code (GList *infos, const char *code)
 {
@@ -278,6 +289,9 @@ txf_infos_find_code (GList *infos, const char *code)
 static void
 account_to_gui (TaxInfoDialog *ti_dialog, Account *account)
 {
+  GtkTreeView *view;
+  GtkTreeSelection *selection;
+  GtkTreePath *path;
   gboolean tax_related;
   const char *str;
   TXFInfo *info;
@@ -305,11 +319,12 @@ account_to_gui (TaxInfoDialog *ti_dialog, Account *account)
   if (index < 0)
     index = 0;
 
-  gtk_clist_select_row (GTK_CLIST (ti_dialog->txf_category_clist), index, 0);
-  if (gtk_clist_row_is_visible (GTK_CLIST (ti_dialog->txf_category_clist),
-                                index) != GTK_VISIBILITY_FULL)
-    gtk_clist_moveto (GTK_CLIST (ti_dialog->txf_category_clist),
-                      index, 0, 0.5, 0.0);
+  view = GTK_TREE_VIEW(ti_dialog->txf_category_view);
+  selection = gtk_tree_view_get_selection(view);
+  path =  gtk_tree_path_new_from_indices(index, -1);
+  gtk_tree_selection_select_path(selection, path);
+  gtk_tree_view_scroll_to_cell(view, path, NULL, FALSE, 0, 0);
+  gtk_tree_path_free(path);
 
   str = xaccAccountGetTaxUSPayerNameSource (account);
   if (safe_strcmp (str, "parent") == 0)
@@ -323,6 +338,12 @@ account_to_gui (TaxInfoDialog *ti_dialog, Account *account)
 static void
 gui_to_accounts (TaxInfoDialog *ti_dialog)
 {
+  GtkTreeView *view;
+  GtkTreeModel *model;
+  GtkTreeSelection *selection;
+  GtkTreePath *path;
+  GtkTreeIter iter;
+  gint *indices;
   gboolean tax_related;
   const char *code;
   const char *pns;
@@ -336,8 +357,14 @@ gui_to_accounts (TaxInfoDialog *ti_dialog)
 
   infos = tax_infos (ti_dialog);
 
-  info = g_list_nth_data
-    (infos, GTK_CLIST (ti_dialog->txf_category_clist)->focus_row);
+  view = GTK_TREE_VIEW(ti_dialog->txf_category_view);
+  selection = gtk_tree_view_get_selection(view);
+  if (!gtk_tree_selection_get_selected(selection, &model, &iter))
+    return;
+  path = gtk_tree_model_get_path(model, &iter);
+  indices = gtk_tree_path_get_indices(path);
+  info = g_list_nth_data (infos, indices[0]);
+  gtk_tree_path_free(path);
   g_return_if_fail (info != NULL);
 
   code = tax_related ? info->code : NULL;
@@ -354,8 +381,8 @@ gui_to_accounts (TaxInfoDialog *ti_dialog)
   else
     pns = NULL;
 
-  accounts = gnc_account_tree_get_current_accounts
-    (GNC_ACCOUNT_TREE (ti_dialog->account_tree));
+  accounts = gnc_tree_view_account_get_selected_accounts
+    (GNC_TREE_VIEW_ACCOUNT(ti_dialog->account_treeview));
 
   for (node = accounts; node; node = node->next)
   {
@@ -387,48 +414,49 @@ window_destroy_cb (GtkObject *object, gpointer data)
   g_free (ti_dialog);
 }
 
+static void
+cursor_changed_cb (GtkWidget *widget, gpointer data)
+{
+  TaxInfoDialog *ti_dialog = data;
+  GncTreeViewAccount *account_tree;
+  Account *account;
+  gint num_children;
+
+  account_tree = GNC_TREE_VIEW_ACCOUNT (ti_dialog->account_treeview);
+  account = gnc_tree_view_account_get_cursor_account (account_tree);
+  if (!account) {
+    gtk_widget_set_sensitive(ti_dialog->select_button, FALSE);
+    return;
+  }
+
+  num_children = gnc_tree_view_account_count_children(account_tree, account);
+  gtk_widget_set_sensitive(ti_dialog->select_button, num_children > 0);
+}
+
 static void
 select_subaccounts_clicked (GtkWidget *widget, gpointer data)
 {
   TaxInfoDialog *ti_dialog = data;
-  GNCAccountTree *tree;
+  GncTreeViewAccount *account_tree;
   Account *account;
 
-  tree = GNC_ACCOUNT_TREE (ti_dialog->account_tree);
-
-  account = gnc_account_tree_get_focus_account (tree);
+  account_tree = GNC_TREE_VIEW_ACCOUNT (ti_dialog->account_treeview);
+  account = gnc_tree_view_account_get_cursor_account (account_tree);
   if (!account)
     return;
 
-  gnc_account_tree_select_subaccounts (tree, account, FALSE);
+  gnc_tree_view_account_select_subaccounts (account_tree, account);
 
-  gtk_widget_grab_focus (ti_dialog->account_tree);
+  gtk_widget_grab_focus (ti_dialog->account_treeview);
 }
 
 static void
-tax_info_ok_clicked (GtkWidget *widget, gpointer data)
+gnc_tax_info_dialog_response (GtkDialog *dialog, gint response, gpointer data)
 {
   TaxInfoDialog *ti_dialog = data;
 
-  if (ti_dialog->changed)
-    gui_to_accounts (ti_dialog);
-
-  gnc_close_gui_component_by_data (DIALOG_TAX_INFO_CM_CLASS, ti_dialog);
-}
-
-static void
-tax_info_apply_clicked (GtkWidget *widget, gpointer data)
-{
-  TaxInfoDialog *ti_dialog = data;
-
-  gui_to_accounts (ti_dialog);
-  gnc_tax_info_set_changed (ti_dialog, FALSE);
-}
-
-static void
-tax_info_cancel_clicked (GtkWidget *widget, gpointer data)
-{
-  TaxInfoDialog *ti_dialog = data;
+  if (response == GTK_RESPONSE_OK && ti_dialog->changed)
+      gui_to_accounts (ti_dialog);
 
   gnc_close_gui_component_by_data (DIALOG_TAX_INFO_CM_CLASS, ti_dialog);
 }
@@ -436,50 +464,49 @@ tax_info_cancel_clicked (GtkWidget *widget, gpointer data)
 static void
 tax_info_show_income_accounts (TaxInfoDialog *ti_dialog, gboolean show_income)
 {
-  GNCAccountTree *tree;
+  GncTreeViewAccount *tree;
   AccountViewInfo info;
   GNCAccountType type;
   GNCAccountType show_type;
 
   ti_dialog->income = show_income;
 
-  tree = GNC_ACCOUNT_TREE (ti_dialog->account_tree);
+  tree = GNC_TREE_VIEW_ACCOUNT (ti_dialog->account_treeview);
   show_type = show_income ? INCOME : EXPENSE;
 
-  gnc_account_tree_get_view_info (tree, &info);
+  gnc_tree_view_account_get_view_info (tree, &info);
 
   for (type = 0; type < NUM_ACCOUNT_TYPES; type++)
     info.include_type[type] = (type == show_type);
 
-  gnc_account_tree_set_view_info (tree, &info);
+  gnc_tree_view_account_set_view_info (tree, &info);
 
   load_category_list (ti_dialog);
+  cursor_changed_cb(GTK_WIDGET(tree), ti_dialog);
 }
 
 static int
 gnc_tax_info_update_accounts (TaxInfoDialog *ti_dialog)
 {
-  GNCAccountTree *tree;
+  GncTreeViewAccount *tree;
+  GtkTreeSelection* selection;
   GtkWidget *label;
-  GtkWidget *frame;
+  GtkWidget *vbox;
   int num_accounts;
-  GList *accounts;
   char *string;
 
-  tree = GNC_ACCOUNT_TREE (ti_dialog->account_tree);
-
-  accounts = gnc_account_tree_get_current_accounts (tree);
-
-  num_accounts = g_list_length (accounts);
+  tree = GNC_TREE_VIEW_ACCOUNT(ti_dialog->account_treeview);
+  selection = gtk_tree_view_get_selection (GTK_TREE_VIEW(tree));
+  num_accounts = gtk_tree_selection_count_selected_rows (selection);
 
   label = gnc_glade_lookup_widget (ti_dialog->dialog, "num_accounts_label");
-  frame = gnc_glade_lookup_widget (ti_dialog->dialog, "tax_info_frame");
+  vbox = gnc_glade_lookup_widget (ti_dialog->dialog, "tax_info_vbox");
 
   string = g_strdup_printf ("%d", num_accounts);
   gtk_label_set_text (GTK_LABEL (label), string);
   g_free (string);
 
-  gtk_widget_set_sensitive (frame, num_accounts > 0);
+  gtk_widget_set_sensitive (vbox, num_accounts > 0);
 
   return num_accounts;
 }
@@ -494,8 +521,8 @@ gnc_tax_info_income_cb (GtkWidget *w, gpointer data)
 
   tax_info_show_income_accounts (ti_dialog, show_income);
 
-  gnc_account_tree_refresh (GNC_ACCOUNT_TREE (ti_dialog->account_tree));
-  gnc_account_tree_expand_all (GNC_ACCOUNT_TREE (ti_dialog->account_tree));
+  ti_dialog->account_type = show_income ? INCOME : EXPENSE;
+  gnc_tree_view_account_refilter (GNC_TREE_VIEW_ACCOUNT (ti_dialog->account_treeview));
 
   gnc_tax_info_update_accounts (ti_dialog);
 
@@ -503,75 +530,82 @@ gnc_tax_info_income_cb (GtkWidget *w, gpointer data)
 }
 
 static void
-gnc_tax_info_select_account_cb (GNCAccountTree *tree,
-                                Account *account, gpointer data)
+gnc_tax_info_account_changed_cb (GtkTreeSelection *selection,
+				 gpointer data)
 {
   TaxInfoDialog *ti_dialog = data;
+  GncTreeViewAccount *view;
+  GList *accounts;
+  int num_accounts;
 
-  if (gnc_tax_info_update_accounts (ti_dialog) != 1)
-  {
+  g_return_if_fail(GTK_IS_TREE_SELECTION(selection));
+
+  num_accounts = gnc_tax_info_update_accounts (ti_dialog);
+  switch (num_accounts) {
+   case 0:
+    clear_gui (ti_dialog);
+    gnc_tax_info_set_changed (ti_dialog, FALSE);
+    return;
+
+   case 1:
+    /* Get the account. This view is set for multiple selection, so we
+       can only get a list of accounts. */
+    view = GNC_TREE_VIEW_ACCOUNT(ti_dialog->account_treeview);
+    accounts = gnc_tree_view_account_get_selected_accounts (view);
+    account_to_gui (ti_dialog, accounts->data);
+    g_list_free(accounts);
+
+    gnc_tax_info_set_changed (ti_dialog, FALSE);
+    break;
+
+   default:
     gnc_tax_info_set_changed (ti_dialog, TRUE);
     return;
   }
-
-  account_to_gui (ti_dialog, account);
-  gnc_tax_info_set_changed (ti_dialog, FALSE);
 }
 
 static void
-gnc_tax_info_unselect_account_cb (GNCAccountTree *tree,
-                                  Account *account, gpointer data)
-{
-  TaxInfoDialog *ti_dialog = data;
-  GList *accounts;
-
-  accounts = gnc_account_tree_get_current_accounts (tree);
-
-  if (gnc_tax_info_update_accounts (ti_dialog) != 0)
-    return;
-
-  clear_gui (ti_dialog);
-  gnc_tax_info_set_changed (ti_dialog, FALSE);
-}
-
-static void
-txf_code_select_row_cb (GtkCList *clist,
-                        gint row,
-                        gint column,
-                        GdkEventButton *event,
-                        gpointer user_data)
+txf_code_select_row_cb (GtkTreeSelection *selection,
+			gpointer user_data)
 {
   TaxInfoDialog *ti_dialog = user_data;
+  GtkTreeModel *model;
+  GtkTreePath *path;
+  GtkTreeIter iter;
+  gint *indices;
   TXFInfo *txf_info;
   GtkAdjustment *adj;
   GtkWidget *scroll;
-  GtkWidget *frame;
-  GtkEditable *ge;
+  GtkWidget *vbox;
+  GtkTextBuffer *tb;
   const char *text;
-  gint pos = 0;
 
-  txf_info = g_list_nth_data (tax_infos (ti_dialog), row);
+  if (!gtk_tree_selection_get_selected(selection, &model, &iter))
+    return;
+  path = gtk_tree_model_get_path(model, &iter);
+  indices = gtk_tree_path_get_indices(path);
+  txf_info = g_list_nth_data (tax_infos (ti_dialog), indices[0]);
+  gtk_tree_path_free(path);
 
-  ge = GTK_EDITABLE (ti_dialog->txf_help_text);
+  tb = gtk_text_view_get_buffer(GTK_TEXT_VIEW(ti_dialog->txf_help_text));
 
   text = (txf_info && txf_info->help) ? txf_info->help : "";
+  gtk_text_buffer_set_text (tb, text, -1);
 
-  gtk_editable_delete_text (ge, 0, -1);
-  gtk_editable_insert_text (ge, text, strlen (text), &pos);
-
-  scroll = gnc_glade_lookup_widget (GTK_WIDGET (clist), "help_scroll");
+  scroll = gnc_glade_lookup_widget (GTK_WIDGET (ti_dialog->dialog),
+				    "help_scroll");
 
   adj = gtk_scrolled_window_get_vadjustment (GTK_SCROLLED_WINDOW (scroll));
   gtk_adjustment_set_value (adj, 0.0);
 
-  frame = gnc_glade_lookup_widget (GTK_WIDGET (clist),
-                                   "payer_name_source_frame");
+  vbox = gnc_glade_lookup_widget (GTK_WIDGET (ti_dialog->dialog),
+                                   "payer_name_source_vbox");
 
-  if (txf_info->payer_name_source)
+  if (txf_info && txf_info->payer_name_source)
   {
     gboolean current;
 
-    gtk_widget_set_sensitive (frame, TRUE);
+    gtk_widget_set_sensitive (vbox, TRUE);
 
     current = (strcmp ("current", txf_info->payer_name_source) == 0);
 
@@ -584,7 +618,7 @@ txf_code_select_row_cb (GtkCList *clist,
   }
   else
   {
-    gtk_widget_set_sensitive (frame, FALSE);
+    gtk_widget_set_sensitive (vbox, FALSE);
     gtk_toggle_button_set_active
       (GTK_TOGGLE_BUTTON (ti_dialog->current_account_button), TRUE);
   }
@@ -597,14 +631,14 @@ tax_related_toggled_cb (GtkToggleButton *togglebutton,
                         gpointer user_data)
 {
   TaxInfoDialog *ti_dialog = user_data;
-  GtkWidget *frame;
+  GtkWidget *vbox;
   gboolean on;
 
   on = gtk_toggle_button_get_active (togglebutton);
 
-  frame = gnc_glade_lookup_widget (GTK_WIDGET (togglebutton),
-                                   "txf_categories_frame");
-  gtk_widget_set_sensitive (frame, on);
+  vbox = gnc_glade_lookup_widget (GTK_WIDGET (togglebutton),
+                                   "txf_categories_vbox");
+  gtk_widget_set_sensitive (vbox, on);
 
   gnc_tax_info_set_changed (ti_dialog, TRUE);
 }
@@ -624,6 +658,9 @@ gnc_tax_info_dialog_create (GtkWidget * parent, TaxInfoDialog *ti_dialog)
   GtkWidget *dialog;
   GtkObject *tido;
   GladeXML  *xml;
+  GtkTreeView *tree_view;
+  GtkTreeSelection *selection;
+  GtkWidget *label;
 
   xml = gnc_glade_xml_new ("tax.glade", "Tax Information Dialog");
 
@@ -631,45 +668,39 @@ gnc_tax_info_dialog_create (GtkWidget * parent, TaxInfoDialog *ti_dialog)
   ti_dialog->dialog = dialog;
   tido = GTK_OBJECT (dialog);
 
+  ti_dialog->account_type = EXPENSE;
   ti_dialog->income_txf_infos = load_txf_info (TRUE);
   ti_dialog->expense_txf_infos = load_txf_info (FALSE);
 
-  gnome_dialog_button_connect (GNOME_DIALOG (dialog), 0,
-                               GTK_SIGNAL_FUNC (tax_info_ok_clicked),
-                               ti_dialog);
+  g_signal_connect (G_OBJECT (dialog), "response",
+                    G_CALLBACK (gnc_tax_info_dialog_response), ti_dialog);
 
-  gnome_dialog_button_connect (GNOME_DIALOG (dialog), 1,
-                               GTK_SIGNAL_FUNC (tax_info_apply_clicked),
-                               ti_dialog);
-
-  gnome_dialog_button_connect (GNOME_DIALOG (dialog), 2,
-                               GTK_SIGNAL_FUNC (tax_info_cancel_clicked),
-                               ti_dialog);
-
-  gtk_signal_connect (tido, "destroy",
-                      GTK_SIGNAL_FUNC (window_destroy_cb), ti_dialog);
+  g_signal_connect (G_OBJECT (dialog), "destroy",
+                    G_CALLBACK (window_destroy_cb), ti_dialog);
 
   /* parent */
   if (parent != NULL)
-    gnome_dialog_set_parent (GNOME_DIALOG (dialog), GTK_WINDOW (parent));
+    gtk_window_set_transient_for (GTK_WINDOW (dialog), GTK_WINDOW (parent));
 
   /* default to ok */
-  gnome_dialog_set_default (GNOME_DIALOG(dialog), 0);
+  gtk_dialog_set_default_response (GTK_DIALOG(dialog), GTK_RESPONSE_OK);
 
   /* tax information */
   {
+    GtkListStore *store;
+    GtkTreeViewColumn *column;
+    GtkCellRenderer *renderer; 
     GtkWidget *button;
-    GtkWidget *clist;
     GtkWidget *text;
 
     button = glade_xml_get_widget (xml, "tax_related_button");
     ti_dialog->tax_related_button = button;
 
-    gtk_signal_connect (GTK_OBJECT (button), "toggled",
-                        GTK_SIGNAL_FUNC (tax_related_toggled_cb), ti_dialog);
+    g_signal_connect (G_OBJECT (button), "toggled",
+                      G_CALLBACK  (tax_related_toggled_cb), ti_dialog);
 
     text = glade_xml_get_widget (xml, "txf_help_text");
-    gtk_text_set_word_wrap (GTK_TEXT (text), TRUE);
+    gtk_text_view_set_wrap_mode(GTK_TEXT_VIEW(text), GTK_WRAP_WORD);
     ti_dialog->txf_help_text = text;
 
     /* set text height */
@@ -678,18 +709,31 @@ gnc_tax_info_dialog_create (GtkWidget * parent, TaxInfoDialog *ti_dialog)
       GdkFont *font = NULL;
 
       if (style != NULL)
-        font = style->font;
+        font = gdk_font_from_description (style->font_desc);
 
       if (font)
-        gtk_widget_set_usize (text, 0, (font->ascent + font->descent) * 5 + 6);
+        gtk_widget_set_size_request (text, -1, (font->ascent + font->descent) * 5 + 6);
     }
 
-    clist = glade_xml_get_widget (xml, "txf_category_clist");
-    gtk_clist_column_titles_passive (GTK_CLIST (clist));
-    ti_dialog->txf_category_clist = clist;
+    tree_view = GTK_TREE_VIEW(glade_xml_get_widget(xml, "txf_category_view"));
+    store =  gtk_list_store_new(2, G_TYPE_STRING, G_TYPE_STRING);
+    gtk_tree_view_set_model(tree_view, GTK_TREE_MODEL(store));
+    renderer = gtk_cell_renderer_text_new();
+    column = gtk_tree_view_column_new_with_attributes
+      (_("Form"), renderer, "text", 0, NULL);
+    gtk_tree_view_append_column(tree_view, GTK_TREE_VIEW_COLUMN(column));
+    renderer = gtk_cell_renderer_text_new();
+    column = gtk_tree_view_column_new_with_attributes
+      (_("Description"), renderer, "text", 1, NULL);
+    gtk_tree_view_append_column(tree_view, GTK_TREE_VIEW_COLUMN(column));
+    ti_dialog->txf_category_view = GTK_WIDGET(tree_view);
 
-    gtk_signal_connect (GTK_OBJECT (clist), "select-row",
-                        GTK_SIGNAL_FUNC (txf_code_select_row_cb), ti_dialog);
+    selection = gtk_tree_view_get_selection(tree_view);
+    g_signal_connect (G_OBJECT (selection), "changed",
+                      G_CALLBACK  (txf_code_select_row_cb), ti_dialog);
+
+    label = glade_xml_get_widget(xml, "txf_category_label");
+    gtk_label_set_mnemonic_widget(GTK_LABEL(label), GTK_WIDGET(tree_view));
 
     button = glade_xml_get_widget (xml, "current_account_button");
     ti_dialog->current_account_button = button;
@@ -697,74 +741,64 @@ gnc_tax_info_dialog_create (GtkWidget * parent, TaxInfoDialog *ti_dialog)
     button = glade_xml_get_widget (xml, "parent_account_button");
     ti_dialog->parent_account_button = button;
 
-    gtk_signal_connect (GTK_OBJECT (button), "toggled",
-                        GTK_SIGNAL_FUNC (current_account_toggled_cb),
+    g_signal_connect (G_OBJECT (button), "toggled",
+                      G_CALLBACK  (current_account_toggled_cb),
                         ti_dialog);
   }
 
   /* account tree */
   {
-    GtkWidget *income_radio;
-    GNCAccountTree *tree;
-    GtkWidget *scroll;
+    GtkWidget *income_radio, *expense_radio, *box;
 
-    ti_dialog->account_tree = gnc_account_tree_new ();
-    tree = GNC_ACCOUNT_TREE (ti_dialog->account_tree);
+    box = glade_xml_get_widget (xml, "account_hbox");
+    tree_view = gnc_tree_view_account_new (FALSE);
+    gnc_tree_view_account_set_filter (GNC_TREE_VIEW_ACCOUNT(tree_view), 
+				      gnc_tax_info_dialog_account_filter_func,
+				      ti_dialog, NULL);
+    ti_dialog->account_treeview = GTK_WIDGET(tree_view);
 
-    gtk_clist_column_titles_hide (GTK_CLIST (ti_dialog->account_tree));
-    gtk_clist_set_selection_mode (GTK_CLIST (ti_dialog->account_tree),
-                                  GTK_SELECTION_EXTENDED);
-    gnc_account_tree_hide_all_but_name (tree);
+    selection = gtk_tree_view_get_selection (tree_view);
+    gtk_tree_selection_set_mode (selection, GTK_SELECTION_EXTENDED);
+    g_signal_connect (G_OBJECT (selection), "changed",
+                      G_CALLBACK (gnc_tax_info_account_changed_cb),
+                      ti_dialog);
 
-    tax_info_show_income_accounts (ti_dialog, FALSE);
+    gtk_widget_show (ti_dialog->account_treeview);
+    gtk_container_add (GTK_CONTAINER (box), ti_dialog->account_treeview);
 
-    gnc_account_tree_refresh (tree);
-    gnc_account_tree_expand_all (tree);
-
-    gtk_signal_connect (GTK_OBJECT(tree), "select_account",
-                        GTK_SIGNAL_FUNC(gnc_tax_info_select_account_cb),
-                        ti_dialog);
-
-    gtk_signal_connect (GTK_OBJECT(tree), "unselect_account",
-                        GTK_SIGNAL_FUNC(gnc_tax_info_unselect_account_cb),
-                        ti_dialog);
-
-    gtk_widget_show (ti_dialog->account_tree);
-
-    scroll = glade_xml_get_widget (xml, "account_scroll");
-    gtk_container_add (GTK_CONTAINER (scroll), ti_dialog->account_tree);
+    label = glade_xml_get_widget(xml, "accounts_label");
+    gtk_label_set_mnemonic_widget(GTK_LABEL(label), GTK_WIDGET(tree_view));
 
     income_radio = glade_xml_get_widget (xml, "income_radio");
-    gtk_signal_connect (GTK_OBJECT (income_radio), "toggled",
-                        GTK_SIGNAL_FUNC (gnc_tax_info_income_cb),
+    expense_radio = glade_xml_get_widget (xml, "expense_radio");
+    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(expense_radio), TRUE);
+    g_signal_connect (G_OBJECT (income_radio), "toggled",
+                      G_CALLBACK  (gnc_tax_info_income_cb),
                         ti_dialog);
-  }
+    //   gtk_button_clicked (GtkButton *button);
+ }
 
   /* select subaccounts button */
   {
     GtkWidget *button;
 
     button = glade_xml_get_widget (xml, "select_subaccounts_button");
-    gtk_signal_connect (GTK_OBJECT (button), "clicked",
-                        GTK_SIGNAL_FUNC (select_subaccounts_clicked),
-                        ti_dialog);
+    ti_dialog->select_button = button;
+
+    g_signal_connect (G_OBJECT (button), "clicked",
+                      G_CALLBACK  (select_subaccounts_clicked),
+		      ti_dialog);
+    g_signal_connect (G_OBJECT (ti_dialog->account_treeview), "cursor_changed",
+                      G_CALLBACK  (cursor_changed_cb),
+		      ti_dialog);
   }
 
+  tax_info_show_income_accounts (ti_dialog, FALSE);
   gnc_tax_info_update_accounts (ti_dialog);
   clear_gui (ti_dialog);
   gnc_tax_info_set_changed (ti_dialog, FALSE);
 
-  if (last_width == 0)
-    gnc_get_window_size ("tax_info_win", &last_width, &last_height);
-
-  if (last_height == 0)
-  {
-    last_height = 400;
-    last_width = 500;
-  }
-
-  gtk_window_set_default_size (GTK_WINDOW(ti_dialog->dialog),
-                               last_width, last_height);
+  gnc_restore_window_size(GCONF_SECTION, GTK_WINDOW(ti_dialog->dialog));
 }
 
 static void
@@ -772,12 +806,8 @@ close_handler (gpointer user_data)
 {
   TaxInfoDialog *ti_dialog = user_data;
 
-  gdk_window_get_geometry (GTK_WIDGET(ti_dialog->dialog)->window,
-                           NULL, NULL, &last_width, &last_height, NULL);
-
-  gnc_save_window_size ("tax_info_win", last_width, last_height);
-
-  gnome_dialog_close (GNOME_DIALOG (ti_dialog->dialog));
+  gnc_save_window_size(GCONF_SECTION, GTK_WINDOW(ti_dialog->dialog));
+  gtk_widget_destroy (ti_dialog->dialog);
 }
 
 static void
@@ -785,7 +815,7 @@ refresh_handler (GHashTable *changes, gpointer user_data)
 {
   TaxInfoDialog *ti_dialog = user_data;
 
-  gnc_account_tree_refresh (GNC_ACCOUNT_TREE (ti_dialog->account_tree));
+/*  gnc_account_tree_refresh (GNC_ACCOUNT_TREE (ti_dialog->account_tree));*/
 
   gnc_tax_info_update_accounts (ti_dialog);
 }
@@ -813,9 +843,9 @@ gnc_tax_info_dialog (GtkWidget * parent)
 
   gnc_gui_component_watch_entity_type (component_id,
                                        GNC_ID_ACCOUNT,
-                                       GNC_EVENT_MODIFY | GNC_EVENT_DESTROY);
+                                       QOF_EVENT_MODIFY | QOF_EVENT_DESTROY);
 
-  gtk_widget_grab_focus (ti_dialog->account_tree);
+  gtk_widget_grab_focus (ti_dialog->account_treeview);
 
   gtk_widget_show (ti_dialog->dialog);
 }
