@@ -17,37 +17,32 @@
  * along with this program; if not, contact:                        *
  *                                                                  *
  * Free Software Foundation           Voice:  +1-617-542-5942       *
- * 59 Temple Place - Suite 330        Fax:    +1-617-542-2652       *
- * Boston, MA  02111-1307,  USA       gnu@gnu.org                   *
+ * 51 Franklin Street, Fifth Floor    Fax:    +1-617-542-2652       *
+ * Boston, MA  02110-1301,  USA       gnu@gnu.org                   *
 \********************************************************************/
 
 #ifndef GNC_RECONCILE_LIST_H
 #define GNC_RECONCILE_LIST_H
 
 #include "gnc-query-list.h"
-#include "gnc-numeric.h"
+#include "qof.h"
 
-#ifdef __cplusplus
-extern "C" {
-#endif				/* __cplusplus */
+G_BEGIN_DECLS
 
-#define GTK_TYPE_GNC_RECONCILE_LIST (gnc_reconcile_list_get_type ())
-#define GNC_RECONCILE_LIST(obj) (GTK_CHECK_CAST ((obj), GTK_TYPE_GNC_RECONCILE_LIST, GNCReconcileList))
-#define GNC_RECONCILE_LIST_CLASS(klass) (GTK_CHECK_CLASS_CAST ((klass), GTK_TYPE_GNC_RECONCILE_LIST, GNCReconcileListClass))
-#define IS_GNC_RECONCILE_LIST(obj) (GTK_CHECK_TYPE ((obj), GTK_TYPE_GNC_RECONCILE_LIST))
-#define IS_GNC_RECONCILE_LIST_CLASS(klass) (GTK_CHECK_CLASS_TYPE ((klass), GTK_TYPE_GNC_RECONCILE_LIST))
+#define GNC_TYPE_RECONCILE_LIST		(gnc_reconcile_list_get_type ())
+#define GNC_RECONCILE_LIST(o)		(G_TYPE_CHECK_INSTANCE_CAST ((o), GNC_TYPE_RECONCILE_LIST, GNCReconcileList))
+#define GNC_RECONCILE_LIST_CLASS(k)	(G_TYPE_CHECK_CLASS_CAST ((k), GNC_TYPE_RECONCILE_LIST, GNCReconcileListClass))
+#define GNC_IS_RECONCILE_LIST(o)	(G_TYPE_CHECK_INSTANCE_TYPE ((o), GNC_TYPE_RECONCILE_LIST))
+#define GNC_IS_RECONCILE_LIST_CLASS(k)	(G_TYPE_CHECK_CLASS_TYPE ((k), GNC_TYPE_RECONCILE_LIST))
 
-typedef struct _GNCReconcileList      GNCReconcileList;
-typedef struct _GNCReconcileListClass GNCReconcileListClass;
+typedef struct GNCReconcileList GNCReconcileList;
 
-typedef enum
-{
+typedef enum {
   RECLIST_DEBIT,
   RECLIST_CREDIT
 } GNCReconcileListType;
 
-struct _GNCReconcileList
-{
+struct GNCReconcileList {
   GNCQueryList qlist;
 
   GHashTable *reconciled;
@@ -59,24 +54,26 @@ struct _GNCReconcileList
   gboolean no_toggle;
 };
 
-struct _GNCReconcileListClass
-{
+typedef struct {
   GtkCListClass parent_class;
 
-  void (*toggle_reconciled) (GNCReconcileList *list,
-			     Split            *split);
+  void (*toggle_reconciled)  (GNCReconcileList *list,
+			      Split            *split);
   void (*double_click_split) (GNCReconcileList *list,
-                              Split            *split);
-};
+			      Split            *split);
+} GNCReconcileListClass;
+
+#define GCONF_RECONCILE_SECTION "dialogs/reconcile"
 
 /***********************************************************
  *                public functions                         *
  ***********************************************************/
 
-GtkType gnc_reconcile_list_get_type (void);
+GType gnc_reconcile_list_get_type (void);
 
 GtkWidget * gnc_reconcile_list_new (Account * account,
-				    GNCReconcileListType type);
+				    GNCReconcileListType type,
+                                    time_t date);
 
 gint gnc_reconcile_list_get_needed_height(GNCReconcileList *list,
                                           gint num_rows);
@@ -97,8 +94,6 @@ void gnc_reconcile_list_unselect_all(GNCReconcileList *list);
 
 gboolean gnc_reconcile_list_changed(GNCReconcileList *list);
 
-#ifdef __cplusplus
-}
-#endif /* __cplusplus */
+G_END_DECLS
 
 #endif /* RECONCILE_LIST_H */
