@@ -1,8 +1,9 @@
 /********************************************************************\
  * gnc-hbci-transfer.c -- hbci transfer functions                   *
  * Copyright (C) 2002 Christian Stimming                            *
- * Copyright (C) 2004 Bernd Wagner (minor changes for                     *
+ * Copyright (C) 2004 Bernd Wagner (minor changes for               *
  *                     online transaction templates)                *
+ * Copyright (c) 2006 David Hampton <hampton@employees.org>         *
  *                                                                  *
  * This program is free software; you can redistribute it and/or    *
  * modify it under the terms of the GNU General Public License as   *
@@ -18,19 +19,21 @@
  * along with this program; if not, contact:                        *
  *                                                                  *
  * Free Software Foundation           Voice:  +1-617-542-5942       *
- * 59 Temple Place - Suite 330        Fax:    +1-617-542-2652       *
- * Boston, MA  02111-1307,  USA       gnu@gnu.org                   *
+ * 51 Franklin Street, Fifth Floor    Fax:    +1-617-542-2652       *
+ * Boston, MA  02110-1301,  USA       gnu@gnu.org                   *
 \********************************************************************/
 
 #include "config.h"
-#include "gnc-hbci-transfer.h"
 
+#include <gtk/gtk.h>
+#include <glib/gi18n.h>
 #include <aqbanking/banking.h>
 
+#include "gnc-hbci-transfer.h"
 #include "gnc-ui.h"
 #include "gnc-numeric.h"
 #include "dialog-transfer.h"
-#include "date.h"
+#include "gnc-date.h"
 #include "Transaction.h"
 
 #include "hbci-interaction.h"
@@ -86,6 +89,11 @@ gnc_hbci_maketrans (GtkWidget *parent, Account *gnc_acc,
     /* Create new HBCIDialogTrans */
     td = gnc_hbci_dialog_new(parent, h_acc, gnc_acc, 
 			     trans_type, template_list);
+
+    /* Delete the list.  The data is still pointed to by the
+     *  GtkListStore in the dialog. */
+    g_list_free(template_list);
+    template_list = NULL;
 	
     /* Repeat until HBCI action was successful or user pressed cancel */
     do {
@@ -94,12 +102,16 @@ gnc_hbci_maketrans (GtkWidget *parent, Account *gnc_acc,
       result = gnc_hbci_dialog_run_until_ok(td, h_acc);
 
       /* Set the template list in case it got modified. */
+      if (template_list)
+	g_list_free(template_list);
       template_list = gnc_hbci_dialog_get_templ(td);
+
       /* templates changed? If yes, store them */
       if (gnc_hbci_dialog_get_templ_changed(td) )
-	       maketrans_save_templates(parent, gnc_acc, template_list, (result >= 0));
+	       maketrans_save_templates(parent, gnc_acc, template_list,
+					(result == GNC_RESPONSE_NOW));
 
-      if (result < 0) {
+      if ((result != GNC_RESPONSE_NOW) && (result != GNC_RESPONSE_LATER)) {
 	break;
       } 
 	
@@ -114,16 +126,16 @@ gnc_hbci_maketrans (GtkWidget *parent, Account *gnc_acc,
 	/* Check whether we really got a job */
 	if (!job) {
 	  /* Oops, no job, probably not supported by bank. */
-	  if (gnc_verify_dialog_parented
+	  if (gnc_verify_dialog
 	      (parent, 
 	       FALSE,
 	       "%s",
-	       _("The backend found an error during the preparation \n"
+	       _("The backend found an error during the preparation "
 		 "of the job. It is not possible to execute this job. \n"
 		 "\n"
-		 "Most probable the bank does not support your chosen \n"
-		 "job or your HBCI account does not have the permission \n"
-		 "to execute this job. More error messages might be \n"
+		 "Most probable the bank does not support your chosen "
+		 "job or your HBCI account does not have the permission "
+		 "to execute this job. More error messages might be "
 		 "visible on your console log.\n"
 		 "\n"
 		 "Do you want to enter the job again?")))
@@ -144,7 +156,12 @@ gnc_hbci_maketrans (GtkWidget *parent, Account *gnc_acc,
 	  continue;
 	}
 
-	if (result == 0) {
+	/* Result of run_until_ok:
+	 *  GNC_RESPONSE_NOW == execute now
+	 *  GNC_RESPONSE_LATER == scheduled for later execution (currently unimplemented)
+	 *  GTK_RESPONSE_CANCEL == cancel
+	 *  GTK_RESPONSE_DELETE_EVENT == window destroyed */
+	if (result == GNC_RESPONSE_NOW) {
 
 	  /* If the user pressed "execute now", then execute this job
 	     now. This function already delete()s the job. */
@@ -160,12 +177,11 @@ gnc_hbci_maketrans (GtkWidget *parent, Account *gnc_acc,
 	  }
 	  
 	  gnc_hbci_cleanup_job(api, job);
-	} /* result == 0 */
+	} /* result == GNC_RESPONSE_NOW */
 	else {
-	  /* huh? Only result == 0 should be possible. Simply ignore
-	     this case. */
+	  /* Simply ignore any other case. */
 	  break;
-	} /* result == 0 */
+	} /* result == GNC_RESPONSE_NOW */
 	  
       } /* Create a do-transaction (transfer) job */
 	
@@ -193,12 +209,12 @@ gnc_hbci_maketrans (GtkWidget *parent, Account *gnc_acc,
 void maketrans_save_templates(GtkWidget *parent, Account *gnc_acc, 
 			      GList *template_list, gboolean dont_ask)
 {
-  if (dont_ask || gnc_verify_dialog_parented
+  if (dont_ask || gnc_verify_dialog
       (parent, 
        FALSE,
        "%s",
-       _("You have changed the list of online transfer templates,\n"
-	 "but you cancelled the transfer dialog.\n"
+       _("You have changed the list of online transfer templates, "
+	 "but you cancelled the transfer dialog. "
 	 "Do you nevertheless want to store the changes?"))) {
     GList *kvp_list = gnc_trans_templ_kvp_glist_from_glist (template_list);
     /*printf ("Now having %d templates. List: '%s'\n", 
