@@ -16,17 +16,14 @@
  * along with this program; if not, contact:                        *
  *                                                                  *
  * Free Software Foundation           Voice:  +1-617-542-5942       *
- * 59 Temple Place - Suite 330        Fax:    +1-617-542-2652       *
- * Boston, MA  02111-1307,  USA       gnu@gnu.org                   *
+ * 51 Franklin Street, Fifth Floor    Fax:    +1-617-542-2652       *
+ * Boston, MA  02110-1301,  USA       gnu@gnu.org                   *
 \********************************************************************/
 
 #ifndef GNC_HBCI_UTILS_H
 #define GNC_HBCI_UTILS_H
 
-#include <glib.h>
-#include <gnome.h>
 #include <iconv.h>
-
 #include <aqbanking/banking.h>
 #include <aqbanking/transaction.h>
 #include <aqbanking/account.h>
@@ -156,7 +153,7 @@ char *gnc_AB_VALUE_toReadableString(const AB_VALUE *v);
 
 /** Returns a newly allocated gchar, converted according to the given
    handler */
-gchar *gnc_call_iconv(iconv_t handler, const char* input);
+gchar *gnc_call_iconv(GIConv handler, const gchar* input);
 
 /** Returns the encoding of the current book in the format as required
     by iconv_open(3). */
