@@ -13,34 +13,36 @@
  * along with this program; if not, contact:                        *
  *                                                                  *
  * Free Software Foundation           Voice:  +1-617-542-5942       *
- * 59 Temple Place - Suite 330        Fax:    +1-617-542-2652       *
- * Boston, MA  02111-1307,  USA       gnu@gnu.org                   *
+ * 51 Franklin Street, Fifth Floor    Fax:    +1-617-542-2652       *
+ * Boston, MA  02110-1301,  USA       gnu@gnu.org                   *
 \********************************************************************/
 /** @addtogroup Import_Export
     @{ */
 /** @internal
     @file import-main-matcher.c
     @brief Transaction matcher main window
-    @author Copyright (C) 2002 Benoit Grgoire
+    @author Copyright (C) 2002 Benoit Grégoire
     @author Christian Stimming    
 */
 #include "config.h"
 
 #include <gnome.h>
+#include <glib/gi18n.h>
 
 #include "import-main-matcher.h"
 
 #include "dialog-utils.h"
 #include "gnc-ui.h"
-#include "global-options.h"
 #include "gnc-ui-util.h"
-#include "gnc-engine-util.h"
+#include "gnc-engine.h"
 #include "import-settings.h"
 #include "import-match-map.h"
 #include "import-match-picker.h"
 #include "import-backend.h"
 #include "import-account-matcher.h"
 
+#define GCONF_SECTION "dialogs/import/generic_matcher/transaction_list"
+
 struct _main_matcher_info
 {
   GtkWidget *dialog;
@@ -66,7 +68,12 @@ struct _main_matcher_info
 #define DOWNLOADED_CLIST_ACTION_CLEAR 6
 #define DOWNLOADED_CLIST_ACTION_EDIT 7
 #define DOWNLOADED_CLIST_ACTION_INFO 8
-static short module = MOD_IMPORT;
+static QofLogModule log_module = GNC_MOD_IMPORT;
+
+/* Local prototypes */
+static void automatch_clist_transactions(GNCImportMainMatcher *info, GtkCList *clist, int starting_row);
+
+
 
 static char * fleche_xpm[] = {
 "17 22 41 1",
@@ -219,6 +226,7 @@ void gnc_gen_trans_list_delete (GNCImportMainMatcher *info)
   if (info == NULL) 
     return;
 
+  gnc_save_window_size(GCONF_SECTION, GTK_WINDOW(info->dialog));
   gnc_import_Settings_delete (info->user_settings);
   gtk_widget_destroy (GTK_WIDGET (info->dialog));
   g_free (info);
@@ -245,6 +253,43 @@ on_matcher_cancel_clicked (GtkButton *button,
   gnc_gen_trans_list_delete (info);
 }
 
+static void 
+on_matcher_help_close_clicked (GtkButton *button,
+			       gpointer user_data)
+{
+  GtkWidget *help_dialog = user_data;
+
+  gtk_widget_destroy(help_dialog);
+}
+
+static void 
+on_matcher_help_clicked (GtkButton *button,
+			 gpointer user_data)
+{
+  GNCImportMainMatcher *info = user_data;
+  GladeXML *xml;
+  GtkWidget *help_dialog, *box;
+  
+  xml = gnc_glade_xml_new ("generic-import.glade", "matcher_help");
+
+  box = glade_xml_get_widget (xml, "red");
+  gtk_widget_modify_bg(box, GTK_STATE_NORMAL, &info->color_back_red);
+  box = glade_xml_get_widget (xml, "yellow");
+  gtk_widget_modify_bg(box, GTK_STATE_NORMAL, &info->color_back_yellow);
+  box = glade_xml_get_widget (xml, "green");
+  gtk_widget_modify_bg(box, GTK_STATE_NORMAL, &info->color_back_green);
+
+  help_dialog = glade_xml_get_widget (xml, "matcher_help");
+  gtk_window_set_transient_for(GTK_WINDOW(help_dialog),
+			       GTK_WINDOW(info->dialog));
+
+  glade_xml_signal_connect_data(xml, "on_matcher_help_close_clicked",
+				G_CALLBACK(on_matcher_help_close_clicked), 
+				help_dialog);
+
+  gtk_widget_show(help_dialog);
+}
+
 static void 
 run_account_picker_dialog (GNCImportMainMatcher *info, 
 			   gint row, GNCImportTransInfo *trans_info)
@@ -253,7 +298,8 @@ run_account_picker_dialog (GNCImportMainMatcher *info,
   gboolean ok_pressed;
   g_assert (trans_info);
   old_acc = gnc_import_TransInfo_get_destacc (trans_info);
-  new_acc = gnc_import_select_account(NULL,
+  new_acc = gnc_import_select_account(info->dialog,
+				      NULL,
 				      TRUE,
 				      _("Destination account for the auto-balance split."),
 				      xaccTransGetCurrency(gnc_import_TransInfo_get_trans(trans_info)),
@@ -265,6 +311,9 @@ run_account_picker_dialog (GNCImportMainMatcher *info,
       gnc_import_TransInfo_set_destacc (trans_info,
 					new_acc,
 					TRUE);
+
+      /* Iterate through the transactions in a given clist to auto match them */
+      automatch_clist_transactions(info, (GtkCList*)info->clist, row);
     }
 }
 
@@ -379,37 +428,38 @@ GNCImportMainMatcher *gnc_gen_trans_list_new (GtkWidget *parent,
   GladeXML *xml;
   GtkWidget *heading_label;
   
-  gnc_should_log(MOD_IMPORT, GNC_LOG_TRACE);
-
   info = g_new0 (GNCImportMainMatcher, 1);
 
   /* Initialize user Settings. */
   info->user_settings = gnc_import_Settings_new ();
   gnc_import_Settings_set_match_date_hardlimit (info->user_settings, match_date_hardlimit);
 
-  /* Initialize the GnomeDialog. */
+  /* Initialize the GtkDialog. */
   xml = gnc_glade_xml_new ("generic-import.glade", "transaction_matcher");
 
-  g_assert
-    (info->dialog = glade_xml_get_widget (xml, "transaction_matcher"));
-  g_assert 
-    (info->clist = glade_xml_get_widget (xml, "downloaded_clist"));
-  g_assert
-    (heading_label = glade_xml_get_widget (xml, "heading_label"));
+  info->dialog = glade_xml_get_widget (xml, "transaction_matcher");
+  g_assert (info->dialog != NULL);
+  info->clist = glade_xml_get_widget (xml, "downloaded_clist");
+  g_assert (info->clist != NULL);
+  heading_label = glade_xml_get_widget (xml, "heading_label");
+  g_assert (heading_label != NULL);
 
-  /*if (parent)
-    gnome_dialog_set_parent (GNOME_DIALOG (info->dialog), 
-			     GTK_WINDOW (parent));*/
+  /* if (parent)
+    gtk_window_set_transient_for (GTK_WINDOW (info->dialog), 
+				  GTK_WINDOW (parent));*/
 
   /* Connect signals */
   glade_xml_signal_connect_data(xml, "downloaded_transaction_select_cb",
-				GTK_SIGNAL_FUNC(clist_select_row_cb), 
+				G_CALLBACK(clist_select_row_cb), 
 				info);
   glade_xml_signal_connect_data(xml, "on_matcher_ok_clicked", 
-				GTK_SIGNAL_FUNC(on_matcher_ok_clicked),
+				G_CALLBACK(on_matcher_ok_clicked),
 				info);
   glade_xml_signal_connect_data(xml, "on_matcher_cancel_clicked", 
-				GTK_SIGNAL_FUNC(on_matcher_cancel_clicked),
+				G_CALLBACK(on_matcher_cancel_clicked),
+				info);
+  glade_xml_signal_connect_data(xml, "on_matcher_help_clicked", 
+				G_CALLBACK(on_matcher_help_clicked),
 				info);
 
   /*Initialise pixmaps*/
@@ -478,9 +528,7 @@ GNCImportMainMatcher *gnc_gen_trans_list_new (GtkWidget *parent,
   if (heading)
     gtk_label_set_text (GTK_LABEL (heading_label), heading);
   
-  /* Hide on close instead of destroy since we still need the values
-     from the boxes. */
-  /*gnome_dialog_close_hides (GNOME_DIALOG (info->dialog), TRUE);*/
+  gnc_restore_window_size(GCONF_SECTION, GTK_WINDOW(info->dialog));
   gtk_widget_show_all (GTK_WIDGET (info->dialog));
   return info;
 }
@@ -491,7 +539,7 @@ gboolean gnc_gen_trans_list_run (GNCImportMainMatcher *info)
   
   /* DEBUG("Begin"); */
 
-  result = gnome_dialog_run_and_close (GNOME_DIALOG (info->dialog));
+  result = gtk_dialog_run (GTK_DIALOG (info->dialog));
 
   /* DEBUG("Result was %d", result); */
 
@@ -557,7 +605,7 @@ refresh_clist_row (GNCImportMainMatcher *gui,
   /*Date*/
 
   text[DOWNLOADED_CLIST_DATE] = 
-    g_strdup( xaccPrintDateSecs ( xaccTransGetDate( gnc_import_TransInfo_get_trans(info) ) ));
+    qof_print_date ( xaccTransGetDate( gnc_import_TransInfo_get_trans(info) ) );
   gtk_clist_set_text (GTK_CLIST (gui->clist), row_number, 
 		      DOWNLOADED_CLIST_DATE, 
 		      text[DOWNLOADED_CLIST_DATE]);
@@ -619,8 +667,7 @@ refresh_clist_row (GNCImportMainMatcher *gui,
 	      gtk_clist_set_background (GTK_CLIST (gui->clist), row_number, 
 					&(gui->color_back_green));
 	      tmp = xaccAccountGetFullName 
-		(gnc_import_TransInfo_get_destacc (info),
-		 gnc_get_account_separator ());
+		(gnc_import_TransInfo_get_destacc (info));
 	      if(gnc_import_TransInfo_get_destacc_selected_manually(info)
 		 == TRUE)
 		{
@@ -825,4 +872,24 @@ void gnc_gen_trans_list_add_trans(GNCImportMainMatcher *gui, Transaction *trans)
   return;
 }/* end gnc_import_add_trans() */
 
+/* Iterate through the rows of the clist and try to automatch each of them */
+static void automatch_clist_transactions(GNCImportMainMatcher *info, GtkCList *clist, int starting_row)
+{
+  int row;
+  GNCImportTransInfo *trans_info;
+  
+  gtk_clist_freeze(clist);	/* prevent a lot of visual updates at once */
+  for(row = starting_row+1; row < clist->rows; row++)
+    {
+      trans_info = gtk_clist_get_row_data(clist, row);
+      
+      /* returns TRUE if we changed this row, so update it */
+      if(gnc_import_TransInfo_refresh_destacc(trans_info, NULL))
+	{
+	  refresh_clist_row(info, row, trans_info);
+	}
+    }
+  gtk_clist_thaw(clist);	/* let all the updates be shown */
+}
+
 /** @} */
