@@ -1,4 +1,4 @@
-/********************************************************************\
+/*******************************************************************\
  * This program is free software; you can redistribute it and/or    *
  * modify it under the terms of the GNU General Public License as   *
  * published by the Free Software Foundation; either version 2 of   *
@@ -13,8 +13,8 @@
  * along with this program; if not, contact:                        *
  *                                                                  *
  * Free Software Foundation           Voice:  +1-617-542-5942       *
- * 59 Temple Place - Suite 330        Fax:    +1-617-542-2652       *
- * Boston, MA  02111-1307,  USA       gnu@gnu.org                   *
+ * 51 Franklin Street, Fifth Floor    Fax:    +1-617-542-2652       *
+ * Boston, MA  02110-1301,  USA       gnu@gnu.org                   *
 \********************************************************************/
 /** @addtogroup Import_Export
     @{ */
@@ -23,17 +23,14 @@
      @brief Ofx import module code
      @author Copyright (c) 2002 Benoit Grgoire <bock@step.polymtl.ca>
  */
-#define _GNU_SOURCE
-
 #include "config.h"
 
+#include <gtk/gtk.h>
+#include <glib/gi18n.h>
 #include <stdio.h>
 #include <string.h>
 #include <sys/time.h>
-
-#include <glib.h>
-#include <guile/gh.h>
-#include <gmodule.h>
+#include <libguile.h>
 
 #include <libofx/libofx.h>
 #include "import-account-matcher.h"
@@ -43,36 +40,28 @@
 
 #include "Account.h"
 #include "Transaction.h"
-#include "global-options.h"
 #include "gnc-associate-account.h"
 #include "gnc-ofx-import.h"
-#include "gnc-file-dialog.h"
-#include "gnc-engine-util.h"
+#include "gnc-file.h"
+#include "gnc-engine.h"
 #include "gnc-book.h"
 #include "gnc-ui-util.h"
+#include "gnc-gconf-utils.h"
+#include "gnc-glib-utils.h"
 
+#define GCONF_SECTION "dialogs/import/ofx"
 
-#include "dialog-utils.h"
-
-
-static short module = MOD_IMPORT;
+static QofLogModule log_module = GNC_MOD_IMPORT;
 
 /********************************************************************\
  * gnc_file_ofx_import
  * Entry point
 \********************************************************************/
 
-SCM  scm_gnc_file_ofx_import ()
-{
-  gnc_file_ofx_import();
-  return SCM_EOL;
-}
-
 /* CS: Store the reference to the created importer gui so that the
    ofx_proc_transaction_cb can use it. */
 GNCImportMainMatcher *gnc_ofx_importer_gui = NULL;
 
-
 /*
 int ofx_proc_status_cb(struct OfxStatusData data)
 {
@@ -86,13 +75,13 @@ int ofx_proc_account_cb(struct OfxAccountData data, void * account_user_data);
 
 int ofx_proc_security_cb(const struct OfxSecurityData data, void * security_user_data)
 {
-  char * tmp_exchange_code=NULL;
+  char * tmp_cusip=NULL;
   char * tmp_default_fullname=NULL;
   char * tmp_default_mnemonic=NULL;
  
   if(data.unique_id_valid==true)
     {
-      tmp_exchange_code=(char *)data.unique_id;
+      tmp_cusip=(char *)data.unique_id;
     }
   if(data.secname_valid==true)
     {
@@ -103,7 +92,7 @@ int ofx_proc_security_cb(const struct OfxSecurityData data, void * security_user
       tmp_default_mnemonic=(char *)data.ticker;
     }
   
-  gnc_import_select_commodity(tmp_exchange_code,
+  gnc_import_select_commodity(tmp_cusip,
 	        	      true,
 			      tmp_default_fullname,
 			      tmp_default_mnemonic);
@@ -132,9 +121,16 @@ int ofx_proc_transaction_cb(struct OfxTransactionData data, void * transaction_u
   g_assert(gnc_ofx_importer_gui);
 
   if(data.account_id_valid==true){
-    account = gnc_import_select_account(data.account_id, 0, NULL, NULL, NO_TYPE, NULL, NULL);
+    account = gnc_import_select_account(NULL,
+					data.account_id, 0, NULL, NULL, NO_TYPE, NULL, NULL);
     if(account!=NULL)
       {
+	/********** Validate the input strings to ensure utf8 ********************/
+	if (data.name_valid)
+	  gnc_utf8_strip_invalid(data.name);
+	if (data.memo_valid)
+	  gnc_utf8_strip_invalid(data.memo);
+
 	/********** Create the transaction and setup transaction data ************/
 	book = xaccAccountGetBook(account);
 	transaction = xaccMallocTransaction(book);
@@ -296,13 +292,13 @@ int ofx_proc_transaction_cb(struct OfxTransactionData data, void * transaction_u
 	  g_free(tmp);
 	}
 	
-	PERR("WRITEME: Gnucash ofx_proc_transaction():Add PAYEE and ADRESS here once supported by libofx!\n");
+	PERR("WRITEME: GnuCash ofx_proc_transaction():Add PAYEE and ADRESS here once supported by libofx!\n");
 	
 	/* Ideally, gnucash should process the corrected transactions */
 	if(data.fi_id_corrected_valid==true){
-	  PERR("WRITEME: Gnucash ofx_proc_transaction(): WARNING: This transaction corrected a previous transaction, but we created a new one instead!\n");
+	  PERR("WRITEME: GnuCash ofx_proc_transaction(): WARNING: This transaction corrected a previous transaction, but we created a new one instead!\n");
 	  tmp=notes;
-	  notes=g_strdup_printf("%s%s%s%s",tmp,"|This corrects transaction #",data.fi_id_corrected,"but Gnucash didn't process the correction!");
+	  notes=g_strdup_printf("%s%s%s%s",tmp,"|This corrects transaction #",data.fi_id_corrected,"but GnuCash didn't process the correction!");
 	  g_free(tmp);
 	}
 	xaccTransSetNotes(transaction, notes);
@@ -311,7 +307,7 @@ int ofx_proc_transaction_cb(struct OfxTransactionData data, void * transaction_u
 	  {
 	    DEBUG("Currency from libofx: %s",data.account_ptr->currency);
 	    currency = gnc_commodity_table_lookup( gnc_get_current_commodities (),
-						   GNC_COMMODITY_NS_ISO,
+						   GNC_COMMODITY_NS_CURRENCY,
 						   data.account_ptr->currency);
 	  }
 	else
@@ -357,7 +353,8 @@ int ofx_proc_transaction_cb(struct OfxTransactionData data, void * transaction_u
 								  in any translations.  */
 							      _("Stock account for security \"%s\""),
 							      data.security_data_ptr->secname);
-		    investment_account = gnc_import_select_account(data.unique_id,
+		    investment_account = gnc_import_select_account(NULL,
+								   data.unique_id,
 								   1,
 								   investment_account_text, 
 								   investment_commodity,
@@ -365,6 +362,7 @@ int ofx_proc_transaction_cb(struct OfxTransactionData data, void * transaction_u
 								   NULL,
 								   NULL);
 		    g_free (investment_account_text);
+		    investment_account_text = NULL;
 		    if(investment_account!=NULL&&
 		       data.unitprice_valid==true&&
 		       data.units_valid==true&&
@@ -422,6 +420,7 @@ int ofx_proc_transaction_cb(struct OfxTransactionData data, void * transaction_u
 								      _("Income account for security \"%s\""),
 								      data.security_data_ptr->secname);
 			    income_account=gnc_import_select_account(NULL,
+								     NULL,
 								     1,
 								     investment_account_text, 
 								     currency,
@@ -446,9 +445,7 @@ int ofx_proc_transaction_cb(struct OfxTransactionData data, void * transaction_u
 			  }
 		      }
 		    if(income_account!=NULL&&
-		       (data.invtransactiontype==OFX_REINVEST||
-			data.invtransactiontype==OFX_INCOME)
-		       )
+		       data.invtransactiontype==OFX_REINVEST)
 		      {
 			DEBUG("Adding investment split; Money flows from the income account"); 
 			split=xaccMallocSplit(book);
@@ -468,6 +465,27 @@ int ofx_proc_transaction_cb(struct OfxTransactionData data, void * transaction_u
 			  xaccSplitSetMemo(split, data.memo);
 			}    
 		      }
+		    if(income_account!=NULL&&
+			data.invtransactiontype==OFX_INCOME)
+		      {
+			DEBUG("Adding investment split; Money flows from the income account"); 
+			split=xaccMallocSplit(book);
+			xaccTransAppendSplit(transaction,split);
+			xaccAccountInsertSplit(income_account,split);
+
+			gnc_amount = double_to_gnc_numeric (-data.amount,/*OFX_INCOME amounts come in as positive numbers*/
+							    gnc_commodity_get_fraction(xaccTransGetCurrency(transaction)),
+							    GNC_RND_ROUND);
+			xaccSplitSetBaseValue(split, gnc_amount, xaccTransGetCurrency(transaction));
+		    
+			/* Also put the ofx transaction name in the splits memo field, or ofx memo if name is unavailable */ 
+			if(data.name_valid==true){
+			  xaccSplitSetMemo(split, data.name);
+			}
+			else if(data.memo_valid==true){
+			  xaccSplitSetMemo(split, data.memo);
+			}    
+		      }
 		
 
 		    if(data.invtransactiontype!=OFX_REINVEST)
@@ -501,7 +519,7 @@ int ofx_proc_transaction_cb(struct OfxTransactionData data, void * transaction_u
 	else
 	  {
 	    PERR("The transaction doesn't have a valid amount");
-	    xaccTransRollbackEdit(transaction);
+	    xaccTransDestroy(transaction);
 	    xaccTransCommitEdit(transaction);
 	  }
 	
@@ -542,7 +560,7 @@ int ofx_proc_account_cb(struct OfxAccountData data, void * account_user_data)
       {
 	DEBUG("Currency from libofx: %s",data.currency);
 	default_commodity = gnc_commodity_table_lookup(commodity_table,
-						       GNC_COMMODITY_NS_ISO,
+						       GNC_COMMODITY_NS_CURRENCY,
 						       data.currency);
       }
     else
@@ -584,6 +602,7 @@ int ofx_proc_account_cb(struct OfxAccountData data, void * account_user_data)
       }
     }
 
+    gnc_utf8_strip_invalid(data.account_name);
     account_description = g_strdup_printf( /* This string is a default account
 					      name. It MUST NOT contain the
 					      character ':' anywhere in it or
@@ -591,13 +610,16 @@ int ofx_proc_account_cb(struct OfxAccountData data, void * account_user_data)
 					  "%s \"%s\"",
 					  account_type_name,
 					  data.account_name);
-    selected_account = gnc_import_select_account(data.account_id, 1, account_description, default_commodity, default_type, NULL, NULL);
+    selected_account = gnc_import_select_account(NULL,
+						 data.account_id, 1,
+						 account_description, default_commodity,
+						 default_type, NULL, NULL);
     g_free(account_description);
     g_free(account_type_name);
   }
   else
     {
-      PERR("Gnucash ofx_proc_account():FATAL ERROR' account online ID not available\n");
+      PERR("account online ID not available");
     }
 
   return 0;
@@ -622,22 +644,26 @@ void gnc_file_ofx_import (void)
   ofx_INFO_msg = true;
   ofx_STATUS_msg = false;
 
-  gnc_should_log(MOD_IMPORT, GNC_LOG_TRACE);
+  qof_log_check(GNC_MOD_IMPORT, QOF_LOG_TRACE);
   DEBUG("gnc_file_ofx_import(): Begin...\n");
 
-  default_dir = gnc_lookup_string_option("__paths", "Import OFX", NULL);
+  default_dir = gnc_gconf_get_string(GCONF_SECTION, KEY_LAST_PATH, NULL);
   if (default_dir == NULL)
     gnc_init_default_directory(&default_dir);
   selected_filename = gnc_file_dialog(_("Select an OFX/QFX file to process"),
 				      NULL,
-				      default_dir);
+				      default_dir,
+				      GNC_FILE_DIALOG_IMPORT);
+  g_free(default_dir);
+  default_dir = NULL;
 
   if(selected_filename!=NULL)
     {
       /* Remember the directory as the default. */
       gnc_extract_directory(&default_dir, selected_filename);
-      gnc_set_string_option("__paths", "Import OFX", default_dir);
+      gnc_gconf_set_string(GCONF_SECTION, KEY_LAST_PATH, default_dir, NULL);
       g_free(default_dir);
+      default_dir = NULL;
 
       /*strncpy(file,selected_filename, 255);*/
       DEBUG("Filename found: %s",selected_filename);
