@@ -1,13 +1,12 @@
-0;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;  qif-parse.scm
 ;;;  routines to parse values and dates in QIF files. 
 ;;;
 ;;;  Bill Gribble <grib@billgribble.com> 20 Feb 2000 
-;;;  $Id: qif-parse.scm,v 1.6.2.1 2003/05/02 04:08:50 warlord Exp $
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 
 (define qif-category-compiled-rexp 
-  (make-regexp "^ *(\\[)?([^]/\\|]*)(]?)(/?)([^\|]*)(\\|(\\[)?([^]/]*)(]?)(/?)(.*))? *$"))
+  (make-regexp "^ *(\\[)?([^]/\\|]*)(]?)(/?)([^\\|]*)(\\|(\\[)?([^]/]*)(]?)(/?)(.*))? *$"))
 
 (define qif-date-compiled-rexp 
   (make-regexp "^ *([0-9]+) *[-/.'] *([0-9]+) *[-/.'] *([0-9]+).*$|^ *([0-9][0-9][0-9][0-9][0-9][0-9][0-9][0-9]).*$"))
@@ -24,7 +23,7 @@
 
 (define comma-radix-regexp
   (make-regexp 
-   "^ *\\$?[+-]?\\$?[0-9]+$|^ *\\$?[+-]?\\$?[0-9]?[0-9]?[0-9]?(\\.[0-9][0-9][0-9])*(,[0-9]*) *$|^ *\\$?[+-]?\\$?[0-9]+,[0-9]* *$"))
+   "^ *\\$?[+-]?\\$?[0-9]+$|^ *\\$?[+-]?\\$?[0-9]?[0-9]?[0-9]?(\\.[0-9][0-9][0-9])*(,[0-9]*)? *$|^ *\\$?[+-]?\\$?[0-9]+,[0-9]* *$"))
 
 (define integer-regexp (make-regexp "^\\$?[+-]?\\$?[0-9]+ *$"))
 
@@ -271,7 +270,7 @@
 ;          ((vest) 
 ;           'vest)
           (else
-           (gnc:warning-dialog 
+           (gnc:warning-dialog #f
             (string-append
              (sprintf #f (_ "The file contains an unknown Action '%s'.")
                       read-value)
@@ -430,25 +429,26 @@
         (numeric-date-parts '())
         (retval date-string)
         (match (regexp-exec qif-date-compiled-rexp date-string)))
-    (if (match:substring match 1)
-        (set! date-parts (list (match:substring match 1)
-                               (match:substring match 2)
-                               (match:substring match 3)))
-	;; This is of the form XXXXXXXX; split the string based on
-	;; whether the format is YYYYxxxx or xxxxYYYY
-	(let ((date-str (match:substring match 4)))
-	  (case format
-	    ((d-m-y m-d-y)
-	     (let ((m (regexp-exec qif-date-mdy-compiled-rexp date-str)))
-	       (set! date-parts (list (match:substring m 1)
-				      (match:substring m 2)
-				      (match:substring m 3)))))
-	  ((y-m-d y-d-m)
-	     (let ((m (regexp-exec qif-date-ymd-compiled-rexp date-str)))
-	       (set! date-parts (list (match:substring m 1)
-				      (match:substring m 2)
-				      (match:substring m 3)))))
-	  )))
+    (if match
+        (if (match:substring match 1)
+             (set! date-parts (list (match:substring match 1)
+                                    (match:substring match 2)
+                                    (match:substring match 3)))
+             ;; This is of the form XXXXXXXX; split the string based on
+             ;; whether the format is YYYYxxxx or xxxxYYYY
+             (let ((date-str (match:substring match 4)))
+               (case format
+                 ((d-m-y m-d-y)
+                  (let ((m (regexp-exec qif-date-mdy-compiled-rexp date-str)))
+                    (set! date-parts (list (match:substring m 1)
+                                           (match:substring m 2)
+                                           (match:substring m 3)))))
+                 ((y-m-d y-d-m)
+                  (let ((m (regexp-exec qif-date-ymd-compiled-rexp date-str)))
+                    (set! date-parts (list (match:substring m 1)
+                                           (match:substring m 2)
+                                           (match:substring m 3)))))
+                 ))))
 	
     ;; get the strings into numbers (but keep the strings around)
     (set! numeric-date-parts
