@@ -4,7 +4,6 @@
 ;;;  Gnucash account tree.
 ;;;
 ;;;  Copyright 2000-2001 Bill Gribble <grib@billgribble.com> 
-;;;  $Id: qif-to-gnc.scm,v 1.11 2003/01/17 03:18:31 warlord Exp $
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
@@ -16,11 +15,11 @@
 (define (qif-import:find-or-make-acct acct-info check-types? commodity
 				      check-commodity? default-currency
                                       gnc-acct-hash old-group new-group)
-  (let* ((separator (string-ref (gnc:account-separator-char) 0))
+  (let* ((separator (string-ref (gnc:account-separator-string) 0))
          (gnc-name (qif-map-entry:gnc-name acct-info))
          (existing-account (hash-ref gnc-acct-hash gnc-name))
          (same-gnc-account 
-          (gnc:get-account-from-full-name old-group gnc-name separator))
+          (gnc:get-account-from-full-name old-group gnc-name))
          (allowed-types 
           (qif-map-entry:allowed-types acct-info))
          (make-new-acct #f)
@@ -39,13 +38,12 @@
 	     #t))))
     
     (define (make-unique-name-variant long-name short-name)
-      (if (gnc:get-account-from-full-name old-group long-name separator)
+      (if (gnc:get-account-from-full-name old-group long-name)
           (let loop ((count 2))
             (let* ((test-name 
                     (string-append long-name (sprintf #f " %a" count)))
                    (test-acct 
-                    (gnc:get-account-from-full-name old-group test-name 
-                                                    separator)))
+                    (gnc:get-account-from-full-name old-group test-name)))
               (if (and test-acct (not (compatible? test-acct)))
                   (loop (+ 1 count))
                   (string-append short-name (sprintf #f " %a" count)))))
@@ -176,17 +174,17 @@
 (define (qif-import:qif-to-gnc qif-files-list 
                                qif-acct-map qif-cat-map 
                                qif-memo-map stock-map 
-                               default-currency-name)
+                               default-currency-name window)
   (gnc:backtrace-if-exception 
    (lambda ()
      (let* ((old-group (gnc:get-current-group))
             (new-group (gnc:malloc-account-group (gnc:get-current-book)))
             (gnc-acct-hash (make-hash-table 20))
-            (separator (string-ref (gnc:account-separator-char) 0))
+            (separator (string-ref (gnc:account-separator-string) 0))
             (default-currency 
               (gnc:commodity-table-find-full 
                (gnc:book-get-commodity-table (gnc:get-current-book))
-               GNC_COMMODITY_NS_ISO default-currency-name))
+               GNC_COMMODITY_NS_CURRENCY default-currency-name))
             (sorted-accounts-list '())
             (markable-xtns '())
             (sorted-qif-files-list 
@@ -298,11 +296,10 @@
        
        (if (> work-to-do 100)
            (begin 
-             (set! progress-dialog (gnc:progress-dialog-new #f #f))
+             (set! progress-dialog (gnc:progress-dialog-new window #f))
              (gnc:progress-dialog-set-title progress-dialog (_ "Progress"))
              (gnc:progress-dialog-set-heading progress-dialog
-                                              (_ "Importing transactions..."))
-             (gnc:progress-dialog-set-limits progress-dialog 0.0 100.0)))
+                                              (_ "Importing transactions..."))))
        
 
        ;; now run through the markable transactions marking any
@@ -314,7 +311,7 @@
              (if progress-dialog 
                  (begin 
                    (gnc:progress-dialog-set-value 
-                    progress-dialog (* 100 (/ work-done work-to-do)))
+                    progress-dialog (/ work-done work-to-do))
                    (gnc:progress-dialog-update progress-dialog))) 
              (if (not (qif-xtn:mark xtn))
                  (qif-import:mark-matching-xtns xtn rest))
@@ -331,7 +328,7 @@
              (if progress-dialog 
                  (begin 
                    (gnc:progress-dialog-set-value 
-                    progress-dialog (* 100 (/ work-done work-to-do)))
+                    progress-dialog (/ work-done work-to-do))
                    (gnc:progress-dialog-update progress-dialog))) 
              (if (not (qif-xtn:mark xtn))
                  (begin 
