@@ -28,11 +28,16 @@
 #include "Account.h"
 #include "Group.h"
 #include "Query.h"
+#include "QueryCore.h"
+#include "QueryNew.h"
+#include "SX-book.h"
 #include "Transaction.h"
-#include "date.h"
-#include "global-options.h"
 #include "gnc-component-manager.h"
-#include "gnc-engine-util.h"
+#include "gnc-book.h"
+#include "gnc-date.h"
+#include "gnc-engine.h"
+#include "gnc-event.h"
+#include "gnc-gconf-utils.h"
 #include "gnc-ledger-display.h"
 #include "gnc-ui-util.h"
 #include "split-register-control.h"
@@ -68,7 +73,7 @@ struct gnc_ledger_display
 
 
 /** GLOBALS *********************************************************/
-static short module = MOD_LEDGER;
+static QofLogModule log_module = GNC_MOD_LEDGER;
 
 
 /** Declarations ****************************************************/
@@ -77,7 +82,7 @@ gnc_ledger_display_internal (Account *lead_account, Query *q,
                              GNCLedgerDisplayType ld_type,
                              SplitRegisterType reg_type,
                              SplitRegisterStyle style,
-			     gboolean use_double_line,
+                             gboolean use_double_line,
                              gboolean is_template);
 static void gnc_ledger_display_refresh_internal (GNCLedgerDisplay *ld,
                                                  GList *splits);
@@ -163,25 +168,6 @@ find_by_leader (gpointer find_data, gpointer user_data)
   return (account == gnc_ledger_display_leader (ld));
 }
 
-static gboolean
-find_by_account (gpointer find_data, gpointer user_data)
-{
-  Account *account = find_data;
-  GNCLedgerDisplay *ld = user_data;
-
-  if (!account || !ld)
-    return FALSE;
-
-  if (account == gnc_ledger_display_leader (ld))
-    return TRUE;
-
-  if (ld->ld_type == LD_SINGLE)
-    return FALSE;
-
-  /* Hack. */
-  return TRUE;
-}
-
 static gboolean
 find_by_query (gpointer find_data, gpointer user_data)
 {
@@ -221,16 +207,14 @@ gnc_get_default_register_style (GNCAccountType type)
 #endif
 
   default:
-    style_string = gnc_lookup_multichoice_option("Register", 
-						 "Default Register Style",
-						 "ledger");
-
-    if (safe_strcmp(style_string, "ledger") == 0)
-      new_style = REG_STYLE_LEDGER;
+    style_string = gnc_gconf_get_string(GCONF_GENERAL_REGISTER,
+					"default_style", NULL);
+    if (safe_strcmp(style_string, "journal") == 0)
+      new_style = REG_STYLE_JOURNAL;
     else if (safe_strcmp(style_string, "auto_ledger") == 0)
       new_style = REG_STYLE_AUTO_LEDGER;
-    else if (safe_strcmp(style_string, "journal") == 0)
-      new_style = REG_STYLE_JOURNAL;
+    else
+      new_style = REG_STYLE_LEDGER;
     
     if (style_string != NULL)
       free(style_string);
@@ -241,13 +225,17 @@ gnc_get_default_register_style (GNCAccountType type)
   return new_style;
 }
 
+static gpointer
+look_for_portfolio_cb (Account *account, gpointer data)
+{
+    return xaccAccountIsPriced(account) ? (gpointer) PORTFOLIO_LEDGER : NULL;
+}
+
 static SplitRegisterType
 gnc_get_reg_type (Account *leader, GNCLedgerDisplayType ld_type)
 {
   GNCAccountType account_type;
   SplitRegisterType reg_type;
-  GList *subaccounts;
-  GList *node;
 
   if (ld_type == LD_GL)
     return GENERAL_LEDGER;
@@ -274,10 +262,10 @@ gnc_get_reg_type (Account *leader, GNCLedgerDisplayType ld_type)
         return LIABILITY_REGISTER;
 
       case PAYABLE:
-	return PAYABLE_REGISTER;
+        return PAYABLE_REGISTER;
 
       case RECEIVABLE:
-	return RECEIVABLE_REGISTER;
+        return RECEIVABLE_REGISTER;
 
       case STOCK:
       case MUTUAL:
@@ -307,8 +295,6 @@ gnc_get_reg_type (Account *leader, GNCLedgerDisplayType ld_type)
     return BANK_REGISTER;
   }
 
-  subaccounts = xaccGroupGetSubAccounts (xaccAccountGetChildren (leader));
-
   switch (account_type)
   {
     case BANK:
@@ -318,25 +304,19 @@ gnc_get_reg_type (Account *leader, GNCLedgerDisplayType ld_type)
     case LIABILITY:
     case RECEIVABLE:
     case PAYABLE:
-      /* if any of the sub-accounts have STOCK or MUTUAL types,
+    {
+      /* If any of the sub-accounts have STOCK or MUTUAL types,
        * then we must use the PORTFOLIO_LEDGER ledger. Otherwise,
        * a plain old GENERAL_LEDGER will do. */
+      gpointer ret;
       reg_type = GENERAL_LEDGER;
 
-      for (node = subaccounts; node; node = node->next)
-      {
-        GNCAccountType le_type;
-
-        le_type = xaccAccountGetType (node->data);
-        if ((STOCK    == le_type) ||
-            (MUTUAL   == le_type) ||
-            (CURRENCY == le_type))
-        {
-          reg_type = PORTFOLIO_LEDGER;
-          break;
-        }
-      }
+      ret = xaccGroupForEachAccount (xaccAccountGetChildren (leader),
+                                    look_for_portfolio_cb,
+                                    NULL, TRUE);
+      if (ret) reg_type = PORTFOLIO_LEDGER;
       break;
+    }
 
     case STOCK:
     case MUTUAL:
@@ -359,8 +339,6 @@ gnc_get_reg_type (Account *leader, GNCLedgerDisplayType ld_type)
       break;
   }
 
-  g_list_free (subaccounts);
-
   return reg_type;
 }
 
@@ -370,7 +348,7 @@ gboolean
 gnc_ledger_display_default_double_line (GNCLedgerDisplay *gld)
 {
   return (gld->use_double_line_default ||
-	  gnc_lookup_boolean_option ("Register", "Double Line Mode", FALSE));
+          gnc_gconf_get_bool(GCONF_GENERAL_REGISTER, "double_line_mode", NULL));
 }
 
 /* Opens up a register window to display a single account */
@@ -409,7 +387,7 @@ gnc_ledger_display_subaccounts (Account *account)
 
   return gnc_ledger_display_internal (account, NULL, LD_SUBACCOUNT,
                                       reg_type, REG_STYLE_JOURNAL, FALSE,
-				      FALSE);
+                                      FALSE);
 }
 
 /* Opens up a general ledger window. */
@@ -418,7 +396,7 @@ gnc_ledger_display_gl (void)
 {
   Query *query;
   time_t start;
-  struct tm *tm;
+  struct tm tm;
 
   query = xaccMallocQuery ();
 
@@ -429,7 +407,7 @@ gnc_ledger_display_gl (void)
    * transactions.  While these are in a seperate AccountGroup just for this
    * reason, the query engine makes no distinction between AccountGroups.
    * See Gnome Bug 86302.
-   * 	-- jsled */
+   *         -- jsled */
   {
     AccountGroup *tAG;
     AccountList *al;
@@ -442,14 +420,9 @@ gnc_ledger_display_gl (void)
     tAG = NULL;
   }
 
-  start = time (NULL);
-  tm = localtime (&start);
-  tm->tm_mon--;
-  tm->tm_hour = 0;
-  tm->tm_min = 0;
-  tm->tm_sec = 0;
-  tm->tm_isdst = -1;
-  start = mktime (tm);
+  gnc_tm_get_today_start(&tm);
+  tm.tm_mon--; /* Default the register to the last month's worth of transactions. */
+  start = mktime (&tm);
   xaccQueryAddDateMatchTT (query, 
                            TRUE, start, 
                            FALSE, 0, 
@@ -461,8 +434,12 @@ gnc_ledger_display_gl (void)
 }
 
 /**
- * id is some identifier that can be:
- * FIXME: what's the correct description of 'id'?
+ * @param id: The string version of the GUID of the context of template
+ * transaction being edited in this template GL.  As used by scheduled
+ * transactions, this is the GUID of the SX itself which is magically the
+ * *name* of the (template) account which contains the transactions for this
+ * scheduled transaction.  That's right.  The stringified GUID of the SX is
+ * the name of the SX'es template account.
  **/
 GNCLedgerDisplay *
 gnc_ledger_display_template_gl (char *id)
@@ -473,8 +450,10 @@ gnc_ledger_display_template_gl (char *id)
   SplitRegister *sr;
   AccountGroup *ag;
   Account *acct;
+  gboolean isTemplateModeTrue;
 
   acct = NULL;
+  isTemplateModeTrue = TRUE;
 
   q = xaccMallocQuery ();
 
@@ -491,7 +470,8 @@ gnc_ledger_display_template_gl (char *id)
   ld = gnc_ledger_display_internal (NULL, q, LD_GL,
                                     SEARCH_LEDGER,
                                     REG_STYLE_JOURNAL,
-                                    FALSE, TRUE); /* TRUE : template mode */
+                                    FALSE,
+                                    isTemplateModeTrue);
 
   sr = gnc_ledger_display_get_split_register (ld);
   if ( acct ) {
@@ -529,7 +509,8 @@ gnc_ledger_display_set_watches (GNCLedgerDisplay *ld, GList *splits)
 
   gnc_gui_component_watch_entity_type (ld->component_id,
                                        GNC_ID_ACCOUNT,
-                                       GNC_EVENT_MODIFY | GNC_EVENT_DESTROY);
+                                       QOF_EVENT_MODIFY | QOF_EVENT_DESTROY
+				       | GNC_EVENT_ITEM_CHANGED);
 
   for (node = splits; node; node = node->next)
   {
@@ -538,7 +519,7 @@ gnc_ledger_display_set_watches (GNCLedgerDisplay *ld, GList *splits)
 
     gnc_gui_component_watch_entity (ld->component_id,
                                     xaccTransGetGUID (trans),
-                                    GNC_EVENT_MODIFY);
+                                    QOF_EVENT_MODIFY);
   }
 }
 
@@ -568,14 +549,19 @@ refresh_handler (GHashTable *changes, gpointer user_data)
   if (changes && has_leader)
   {
     info = gnc_gui_get_entity_events (changes, &ld->leader);
-    if (info && (info->event_mask & GNC_EVENT_DESTROY))
+    if (info && (info->event_mask & QOF_EVENT_DESTROY))
     {
       gnc_close_gui_component (ld->component_id);
       return;
     }
   }
 
-  splits = xaccQueryGetSplits (ld->query);
+  /* Its not clear if we should re-run the query, or if we should
+   * just use qof_query_last_run().  Its possible that the dates
+   * changed, requiring a full new query.  Similar considerations
+   * needed for multi-user mode.
+   */
+  splits = qof_query_run (ld->query);
 
   gnc_ledger_display_set_watches (ld, splits);
 
@@ -606,7 +592,7 @@ close_handler (gpointer user_data)
 
 static void
 gnc_ledger_display_make_query (GNCLedgerDisplay *ld,
-                               gboolean show_all,
+                               gint limit,
                                SplitRegisterType type)
 {
   Account *leader;
@@ -636,8 +622,8 @@ gnc_ledger_display_make_query (GNCLedgerDisplay *ld,
    * configurable, or maybe we should go back a time range instead
    * of picking a number, or maybe we should be able to exclude
    * based on reconciled status. Anyway, this works for now. */
-  if (!show_all && (type != SEARCH_LEDGER))
-    xaccQuerySetMaxSplits (ld->query, 30);
+  if ((limit != 0) && (type != SEARCH_LEDGER))
+    xaccQuerySetMaxSplits (ld->query, limit);
 
   xaccQuerySetBook (ld->query, gnc_get_current_book());
 
@@ -662,7 +648,7 @@ gnc_ledger_display_query (Query *query, SplitRegisterType type,
                           SplitRegisterStyle style)
 {
   return gnc_ledger_display_internal (NULL, query, LD_GL, type, style,
-				      FALSE, FALSE);
+                                      FALSE, FALSE);
 }
 
 static GNCLedgerDisplay *
@@ -670,11 +656,11 @@ gnc_ledger_display_internal (Account *lead_account, Query *q,
                              GNCLedgerDisplayType ld_type,
                              SplitRegisterType reg_type,
                              SplitRegisterStyle style,
-			     gboolean use_double_line,
+                             gboolean use_double_line,
                              gboolean is_template )
 {
   GNCLedgerDisplay *ld;
-  gboolean show_all;
+  gint limit;
   const char *class;
   GList *splits;
 
@@ -754,15 +740,13 @@ gnc_ledger_display_internal (Account *lead_account, Query *q,
   ld->get_parent = NULL;
   ld->user_data = NULL;
 
-  show_all = gnc_lookup_boolean_option ("_+Advanced",
-                                        "Show All Transactions",
-                                        TRUE);
+  limit = gnc_gconf_get_float(GCONF_GENERAL_REGISTER, "max_transactions", NULL);
 
   /* set up the query filter */
   if (q)
     ld->query = xaccQueryCopy (q);
   else
-    gnc_ledger_display_make_query (ld, show_all, reg_type);
+    gnc_ledger_display_make_query (ld, limit, reg_type);
 
   ld->component_id = gnc_register_gui_component (class,
                                                  refresh_handler,
@@ -774,11 +758,11 @@ gnc_ledger_display_internal (Account *lead_account, Query *q,
 
   ld->use_double_line_default = use_double_line;
   ld->reg = gnc_split_register_new (reg_type, style, use_double_line,
-				    is_template);
+                                    is_template);
 
   gnc_split_register_set_data (ld->reg, ld, gnc_ledger_display_parent);
 
-  splits = xaccQueryGetSplits (ld->query);
+  splits = qof_query_run (ld->query);
 
   gnc_ledger_display_set_watches (ld, splits);
 
@@ -819,11 +803,7 @@ gnc_ledger_display_refresh_internal (GNCLedgerDisplay *ld, GList *splits)
     return;
 
   if (!gnc_split_register_full_refresh_ok (ld->reg))
-  {
-    gnc_split_register_load_xfer_cells (ld->reg,
-                                        gnc_ledger_display_leader (ld));
     return;
-  }
 
   ld->loading = TRUE;
 
@@ -882,41 +862,6 @@ gnc_ledger_display_refresh_by_split_register (SplitRegister *reg)
   }
 }
 
-/********************************************************************\
- * xaccDestroyLedgerDisplay()
-\********************************************************************/
-
-static void
-gnc_destroy_ledger_display_class (Account *account,
-                                  const char *component_class)
-{
-  GList *list;
-  GList *node;
-
-  list = gnc_find_gui_components (component_class, find_by_account, account);
-
-  for (node = list; node; node = node->next)
-  {
-    GNCLedgerDisplay *ld = node->data;
-
-    gnc_close_gui_component (ld->component_id);
-  }
-
-  g_list_free (list);
-}
-
-void
-gnc_ledger_display_destroy_by_account (Account *account)
-{
-  if (!account)
-    return;
-
-  gnc_destroy_ledger_display_class (account, REGISTER_SINGLE_CM_CLASS);
-  gnc_destroy_ledger_display_class (account, REGISTER_SUBACCOUNT_CM_CLASS);
-  gnc_destroy_ledger_display_class (account, REGISTER_GL_CM_CLASS);
-  /* no TEMPLATE_CM_CLASS, because it doesn't correspond to any account */
-}
-
 void
 gnc_ledger_display_close (GNCLedgerDisplay *ld)
 {
