@@ -674,17 +674,19 @@ gnc_split_register_auto_completion (SplitRegister *reg,
         if (auto_trans == NULL)
           return FALSE;
 
+	gnc_suspend_gui_refresh ();
+
         /* now perform the completion */
-
-        gnc_suspend_gui_refresh ();
-
-        info->pending_trans_guid = *xaccTransGetGUID(trans);
         if ((pending_trans != NULL) && (pending_trans != trans)) {
+            if (gnc_split_register_begin_edit_or_warn(info, trans))
+	    {
+	        gnc_resume_gui_refresh ();
+                return TRUE;
+	    }
+
             if (xaccTransIsOpen (pending_trans))
                 xaccTransCommitEdit (pending_trans);
             else g_assert_not_reached();
-            g_assert(!xaccTransIsOpen(trans));
-            xaccTransBeginEdit(trans);
         }
         g_assert(xaccTransIsOpen(trans));
         pending_trans = trans;
