@@ -48,7 +48,7 @@
 (export gnc:case-exchange-time-fn)
 (export gnc:sum-collector-commodity)
 (export gnc:sum-collector-stocks)
-
+(export gnc:commodity-collector-contains-commodity?)
 
 ;; options-utilities.scm 
 
@@ -65,9 +65,11 @@
 (export gnc:options-add-plot-size!)
 (export gnc:options-add-marker-choice!)
 (export gnc:options-add-sort-method!)
+(export gnc:options-add-subtotal-view!)
 
 ;; html-utilities.scm 
 
+(export gnc:html-make-empty-cell)
 (export gnc:html-make-empty-cells)
 (export gnc:account-anchor-text)
 (export gnc:split-anchor-text)
@@ -84,9 +86,11 @@
 (export gnc:html-acct-table-cell)
 (export gnc:html-acct-table-row-helper! )
 (export gnc:html-acct-table-comm-row-helper!)
-(export gnc:html-build-acct-table )
+(export gnc:html-build-acct-table)
+(export gnc:first-html-build-acct-table)
 (export gnc:html-make-exchangerates)
 (export gnc:html-make-no-account-warning)
+(export gnc:html-make-generic-options-warning)
 (export gnc:html-make-empty-data-warning)
 
 ;; report.scm
@@ -103,7 +107,6 @@
 
 (export gnc:define-report)
 (export <report>)
-(export *gnc:_reports_*)
 (export gnc:report-template-new-options/name)
 (export gnc:report-template-menu-name/name)
 (export gnc:report-template-renderer/name)
@@ -144,8 +147,6 @@
 (export gnc:report-stylesheet)
 (export gnc:report-set-stylesheet!)
 (export gnc:all-report-template-names)
-(export gnc:report-remove-by-id)
-(export gnc:find-report)
 (export gnc:find-report-template)
 (export gnc:report-generate-restore-forms)
 (export gnc:report-generate-saved-forms)
@@ -153,6 +154,7 @@
 (export gnc:report-render-html)
 (export gnc:report-run)
 (export gnc:report-templates-for-each)
+(export gnc:report-embedded-list)
 
 ;; html-barchart.scm
 
@@ -213,6 +215,8 @@
 (export gnc:make-html-document)
 (export gnc:html-document-set-title!)
 (export gnc:html-document-title)
+(export gnc:html-document-set-headline!)
+(export gnc:html-document-headline)
 (export gnc:html-document-set-style-sheet!)
 (export gnc:html-document-style-sheet)
 (export gnc:html-document-set-style-stack!)
@@ -403,6 +407,43 @@
 (export gnc:html-style-sheet-find)
 (export gnc:html-style-sheet-remove)
 
+;; html-acct-table.scm 
+
+(export gnc:colspans-are-working-right)
+(export <html-acct-table>)
+(export gnc:html-acct-table?)
+(export gnc:_make-html-acct-table_)
+(export gnc:make-html-acct-table)
+(export gnc:make-html-acct-table/env)
+(export gnc:make-html-acct-table/env/accts)
+(export gnc:_html-acct-table-matrix_)
+(export gnc:_html-acct-table-set-matrix!_)
+(export gnc:_html-acct-table-env_)
+(export gnc:_html-acct-table-set-env!_)
+(export gnc:html-acct-table-add-accounts!)
+(export gnc:html-acct-table-num-rows)
+(export gnc:html-acct-table-num-cols)
+(export gnc:html-acct-table-get-row)
+(export gnc:html-acct-table-get-cell)
+(export gnc:html-acct-table-set-cell!)
+(export gnc:html-acct-table-get-row-env)
+(export gnc:html-acct-table-set-row-env!)
+(export gnc:html-acct-table-append-row)
+(export gnc:html-acct-table-prepend-row!)
+(export gnc:html-acct-table-append-col)
+(export gnc:html-acct-table-prepend-col!)
+(export gnc:html-acct-table-remove-last-row!)
+(export gnc:html-acct-table-render)
+(export gnc:account-code-less-p)
+(export gnc:account-name-less-p)
+(export gnc:account-path-less-p)
+;;(export gnc:identity)
+(export gnc:html-table-add-labeled-amount-line!)
+(export gnc:html-table-add-account-balances)
+(export gnc:second-html-build-acct-table)
+(export gnc:commodity-table)
+(export gnc:uniform-commodity?)
+
 ;; html-table.scm 
 
 (export <html-table>)
@@ -467,9 +508,11 @@
 (export gnc:html-table-append-row!)
 (export gnc:html-table-remove-last-row!)
 (export gnc:html-table-prepend-row!)
+(export gnc:html-table-get-cell)
 (export gnc:html-table-set-cell!)
 (export gnc:html-table-append-column!)
 (export gnc:html-table-prepend-column!)
+(export gnc:html-table-merge)
 (export gnc:html-table-render)
 
 ;; html-text.scm
@@ -511,7 +554,6 @@
 
 (export list-ref-safe)
 (export list-set-safe!)
-(export gnc:amount->string)
 (export gnc:commodity-value->string)
 (export gnc:monetary->string)
 (export gnc:account-has-shares?)
@@ -526,12 +568,12 @@
 (export gnc:account-get-immediate-subaccounts)
 (export gnc:account-get-all-subaccounts)
 (export gnc:acccounts-get-all-subaccounts)
-(export gnc:transaction-map-splits)
 (export gnc:make-stats-collector)
 (export gnc:make-drcr-collector)
 (export gnc:make-value-collector)
 (export gnc:make-numeric-collector)
 (export gnc:make-commodity-collector)
+(export gnc:commodity-collector-commodity-count)
 (export gnc:account-get-balance-at-date)
 (export gnc:account-get-comm-balance-at-date)
 (export gnc:accounts-get-balance-helper)
@@ -543,6 +585,8 @@
 (export gnc:account-get-balance-interval)
 (export gnc:account-get-comm-balance-interval)
 (export gnc:group-get-comm-balance-interval)
+(export gnc:accountlist-get-comm-balance-interval)
+(export gnc:accountlist-get-comm-balance-at-date)
 (export gnc:query-set-match-non-voids-only!)
 (export gnc:query-set-match-voids-only!)
 (export gnc:split-voided?)
@@ -551,6 +595,10 @@
 (export gnc:report-percent-done)
 (export gnc:report-finished)
 (export gnc:accounts-count-splits)
+(export gnc:commodity-collector-allzero?)
+(export gnc:account-get-trans-type-balance-interval)
+(export gnc:account-get-pos-trans-total-interval)
+(export gnc:double-col)
 
 (load-from-path "commodity-utilities.scm")
 (load-from-path "html-barchart.scm")
@@ -562,6 +610,7 @@
 (load-from-path "html-style-sheet.scm")
 (load-from-path "html-table.scm")
 (load-from-path "html-text.scm")
+(load-from-path "html-acct-table.scm")
 (load-from-path "html-utilities.scm")
 (load-from-path "options-utilities.scm")
 (load-from-path "report-utilities.scm")
