@@ -290,7 +290,7 @@
   (gnc:register-reg-option
    (gnc:make-simple-boolean-option
     (N_ "Display") (N_ "Price")
-    "hb" "Display the shares price?" #f))
+    "hb" (N_ "Display the shares price?") #f))
 
   (gnc:register-reg-option
    (gnc:make-multichoice-option
@@ -579,12 +579,12 @@
            (gnc:make-html-text
             (gnc:html-markup-br)
             ;;(gnc:option-value
-            ;; (gnc:lookup-global-option "User Info" "User Name"))
+            ;; (gnc:gconf-get-string "user_info" "name"))
 	    "User Name"
             (gnc:html-markup-br)
             (string-expand
              ;;(gnc:option-value
-	     ;; (gnc:lookup-global-option "User Info" "User Address"))
+	     ;; (gnc:gconf-get-string "user_info" "address"))
 	     "User Address"
              #\newline
              "<br>")
